@@ -42,6 +42,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #include "irc.h"
 #include "aim.h"
 #include "safestring.h"
+#include "connwrap.h"
 
 typedef void (*ptrtotoc)(void *, ...);
 typedef void (*sighandler_t)(int);
@@ -401,7 +402,7 @@ ipv6fail:
 			goto ipv4fail;
 		if (fcntl(s, F_SETFL, O_NONBLOCK))
 			goto ipv4fail;
-		i = connect(s,(const struct sockaddr *)inet4_ip,sizeof(struct sockaddr_in));
+		i = cw_connect(s,(const struct sockaddr *)inet4_ip,sizeof(struct sockaddr_in), 0);
 		if (i != 0 && errno != EINPROGRESS)
 			goto ipv4fail;
 		return s;
@@ -1078,7 +1079,7 @@ void firetalk_callback_chat_invited(client_t c, const char * const room, const c
 	return;
 }
 
-void firetalk_callback_chat_user_joined(client_t c, const char * const room, const char * const who) {
+void firetalk_callback_chat_user_joined(client_t c, const char * const room, const char * const who, const char * const email) {
 	struct s_firetalk_handle *conn;
 	conn = firetalk_find_handle(c);
 	if (conn == NULL)
@@ -1086,7 +1087,7 @@ void firetalk_callback_chat_user_joined(client_t c, const char * const room, con
 	if (firetalk_chat_internal_add_member(conn,room,who) != FE_SUCCESS)
 		return;
 	if (conn->callbacks[FC_CHAT_USER_JOINED])
-		conn->callbacks[FC_CHAT_USER_JOINED](conn,conn->clientstruct,room,who);
+		conn->callbacks[FC_CHAT_USER_JOINED](conn,conn->clientstruct,room,who,email);
 	return;
 }
 
@@ -1340,6 +1341,9 @@ void firetalk_callback_subcode_reply(client_t c, const char * const from, const
 void firetalk_callback_file_offer(client_t c, const char * const from, const char * const filename, const long size, const char * const ipstring, const char * const ip6string, const uint16_t port, const int type) {
 	struct s_firetalk_handle *conn;
 	struct s_firetalk_file *iter;
+	struct in_addr addr;
+	int ret;
+
 	conn = firetalk_find_handle(c);
 	if (conn == NULL)
 		return;
@@ -1359,16 +1363,27 @@ void firetalk_callback_file_offer(client_t c, const char * const from, const cha
 	conn->file_head->type = type;
 	conn->file_head->next = iter;
 	conn->file_head->clientfilestruct = NULL;
-	if (inet_pton(AF_INET,ipstring,&conn->file_head->inet_ip) == 0) {
-		firetalk_file_cancel(c,conn->file_head);
-		return;
+
+#ifdef HAVE_INET_ATON
+	ret = inet_aton(ipstring, &addr);
+	conn->file_head->inet_ip.s_addr = addr.s_addr;
+#else
+	ret = inet_pton(AF_INET, ipstring, &conn->file_head->inet_ip);
+#endif
+
+	if(!ret) {
+	    firetalk_file_cancel(c,conn->file_head);
+	    return;
 	}
+
+/*
 #ifdef _FC_USE_IPV6
 	conn->file_head->tryinet6 = 0;
 	if (ip6string)
 		if (inet_pton(AF_INET6,ip6string,&conn->file_head->inet6_ip) != 0)
 			conn->file_head->tryinet6 = 1;
 #endif
+*/
 	if (conn->callbacks[FC_FILE_OFFER])
 		conn->callbacks[FC_FILE_OFFER](conn,conn->clientstruct,(void *)conn->file_head,from,filename,size);
 	return;
@@ -1629,6 +1644,9 @@ void firetalk_destroy_handle(firetalk_t conn) {
 	if (firetalk_check_handle(conn) != FE_SUCCESS)
 		return;
 #endif
+	if (conn->handle)
+		protocol_functions[conn->protocol].destroy_handle(conn->handle);
+
 	free(conn->buffer);
 	if (conn->prev)
 		conn->prev->next = conn->next;
@@ -1636,8 +1654,6 @@ void firetalk_destroy_handle(firetalk_t conn) {
 		handle_head = conn->next;
 	if (conn->next)
 		conn->next->prev = conn->prev;
-	if (conn->handle)
-		protocol_functions[conn->protocol].destroy_handle(conn->handle);
 
 	free(conn);
 	return;
@@ -2093,7 +2109,7 @@ enum firetalk_error firetalk_im_searchemail(firetalk_t conn, const char * const
 	return protocol_functions[conn->protocol].im_searchemail(conn->handle,email);
 }
 
-enum firetalk_error firetalk_chat_join(firetalk_t conn, const char * const room) {
+enum firetalk_error firetalk_chat_join(firetalk_t conn, const char * const room, const char * const passwd) {
 	const char *normalroom;
 
 #ifdef DEBUG
@@ -2108,7 +2124,7 @@ enum firetalk_error firetalk_chat_join(firetalk_t conn, const char * const room)
 	if (!normalroom)
 		return FE_ROOMUNAVAILABLE;
 
-	return protocol_functions[conn->protocol].chat_join(conn->handle,normalroom);
+	return protocol_functions[conn->protocol].chat_join(conn->handle,normalroom,passwd);
 }
 
 enum firetalk_error firetalk_chat_part(firetalk_t conn, const char * const room) {
@@ -2347,11 +2363,11 @@ enum firetalk_error firetalk_subcode_register_reply_callback(firetalk_t conn, co
 	return FE_SUCCESS;
 }
 
-enum firetalk_error firetalk_file_offer(firetalk_t conn, const char * const nickname, const char * const filename) {
+enum firetalk_error firetalk_file_offer(firetalk_t conn, const char * const nickname, const char * const filename, void **fhandle) {
 	struct s_firetalk_file *iter;
 	struct stat s;
 	struct sockaddr_in addr;
-	char args[256];
+	char args[1024], *p, *cp;
 	unsigned int l;
 
 #ifdef DEBUG
@@ -2409,8 +2425,15 @@ enum firetalk_error firetalk_file_offer(firetalk_t conn, const char * const nick
 	conn->file_head->port = ntohs(addr.sin_port);
 	conn->file_head->next = iter;
 	conn->file_head->type = FF_TYPE_DCC;
-	safe_snprintf(args,256,"SEND %s %lu %u %ld",conn->file_head->filename,conn->localip,conn->file_head->port,conn->file_head->size);
-	return firetalk_subcode_send_request(conn,nickname,"DCC",args);
+
+	*fhandle = conn->file_head;
+
+	cp = conn->file_head->filename;
+	while(p = strstr(cp, "/"))
+	    cp = p+1;
+
+	sprintf(args, "SEND %s %lu %lu %ld", cp, conn->localip, conn->file_head->port, conn->file_head->size);
+	return firetalk_subcode_send_request(conn, nickname, "DCC", args);
 }
 
 enum firetalk_error firetalk_file_accept(firetalk_t conn, void *filehandle, void *clientfilestruct, const char * const localfile) {
@@ -2491,7 +2514,9 @@ enum firetalk_error firetalk_compare_nicks(firetalk_t conn, const char * const n
 }
 
 enum firetalk_error firetalk_select() {
-	return firetalk_select_custom(0,NULL,NULL,NULL,NULL);
+	struct timeval tv;
+	tv.tv_sec = tv.tv_usec = 0;
+	return firetalk_select_custom(0,NULL,NULL,NULL,&tv);
 }
 
 enum firetalk_error firetalk_select_custom(int n, fd_set *fd_read, fd_set *fd_write, fd_set *fd_except, struct timeval *timeout) {
