@@ -17,6 +17,16 @@
 #include <ncurses.h>
 #include <ctype.h>
 
+#ifdef __sun__
+#include <sys/termio.h>
+#include <sys/filio.h>
+#endif
+
+#ifdef __CYGWIN__
+#include <sys/termios.h>
+#include <sys/socket.h>
+#endif
+
 #undef box
 #undef clear
 #undef erase
@@ -25,8 +35,8 @@
 
 /* Fucking ncurses stuff */
 
-#define boldcolor(c)		COLOR_PAIR(c)|A_BOLD
-#define normalcolor(c)		COLOR_PAIR(c)
+#define boldcolor(c)            COLOR_PAIR(c)|A_BOLD
+#define normalcolor(c)          COLOR_PAIR(c)
 
 #ifdef LOCALES_HACK
 #define KT_DISP_FILTER(c)       ( ((c > ' ') && (c != '\177')) ? c : ' ' )
@@ -61,15 +71,15 @@
 #define ALTL_PRESSED    8
 #define CONTROL_PRESSED 4
 
-extern bool kintf_graph, kintf_refresh;
+extern bool kintf_graph, kintf_refresh, use_fribidi;
 extern void (*kt_resize_event)(void);
 
 void printchar(char c);
-void printstring(const string s);
-int string2key(const string adef);
+void printstring(const string &s);
+int string2key(const string &adef);
 
-const string makebidi(const string buf, int lpad = 0);
-int findcolor(const string s);
+string makebidi(const string &buf, int lpad = 0);
+int findcolor(const string &s);
 
 __KTOOL_BEGIN_C
 
@@ -88,7 +98,6 @@ void kgotoxy(int x, int y);
 void hidecursor();
 void showcursor();
 void setbeep(int freq, int duration);
-void delay(int milisec);
 int kwherex();
 int kwherey();
 
