@@ -46,13 +46,23 @@ class treeview : public abstractuicontrol {
 	treeview();
 	~treeview();
 
-        bool empty();
+	bool empty();
+
+	int addnode(int parent, int color, void *ref, const string &text);
+	int addnode(int parent, int color, int ref, const string &text);
+	int addnode(const string &text);
 
 	int addnodef(int parent, int color, void *ref, const char *fmt, ...);
-	int addleaff(int parent, int color, void *ref, const char *fmt, ...);
+	int addnodef(int parent, int color, int ref, const char *fmt, ...);
+	int addnodef(const char *fmt, ...);
 
-	int addnode(int parent, int color, void *ref, string text);
-	int addleaf(int parent, int color, void *ref, string text);
+	int addleaf(int parent, int color, void *ref, const string &text);
+	int addleaf(int parent, int color, int ref, const string &text);
+	int addleaf(const string &text);
+
+	int addleaff(int parent, int color, void *ref, const char *fmt, ...);
+	int addleaff(int parent, int color, int ref, const char *fmt, ...);
+	int addleaff(const char *fmt, ...);
 
 	void opennode(int mpos);
 	void closenode(int mpos);
@@ -72,7 +82,7 @@ class treeview : public abstractuicontrol {
 	void *open(int *n);
 
 	void setcur(int id);
-	void setcoord(int nx1, int ny1, int nx2, int ny2);
+	void setcoords(int nx1, int ny1, int nx2, int ny2);
 };
 
 __KTOOL_END_NAMESPACE
