@@ -1,7 +1,7 @@
 /*
 *
 * kkconsui various textmode menus classes
-* $Id: cmenus.cc,v 1.15 2002/07/03 14:38:56 konst Exp $
+* $Id: cmenus.cc,v 1.18 2003/07/22 21:50:35 konst Exp $
 *
 * Copyright (C) 1999-2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -220,7 +220,6 @@ void verticalmenu::scale() {
 }
 
 int verticalmenu::open() {
-    bool finished = false;
     int go, k;
 
     if(!window.empty()) window.open();
@@ -231,6 +230,8 @@ int verticalmenu::open() {
     if(clearonfocuslost && curelem >= 0)
 	shownelem(curelem, 1);
 
+    finished = aborted = false;
+
     while(!finished) {
 	if(idle) go = keypressed(); else go = 1;
 	if(go) {
@@ -453,8 +454,10 @@ int horizontalmenu::menu_otherkeys(verticalmenu &ref, int k) {
 	case KEY_LEFT  : return HM_LEFT;
 	case KEY_F(10) : return HM_CLOSE;
 	default:
-	    currenthmenu->finished = currenthmenu->otherkeys(*currenthmenu, k);
-	    return currenthmenu->finished ? 0 : -1;
+	    if(currenthmenu->otherkeys) {
+		currenthmenu->finished = currenthmenu->otherkeys(*currenthmenu, k);
+		return currenthmenu->finished ? 0 : -1;
+	    }
     }
 
     return -1;
