@@ -1,7 +1,7 @@
 /*
 *
 * kkstrtext string related and text processing routines
-* $Id: kkstrtext.cc,v 1.25 2002/07/03 14:38:58 konst Exp $
+* $Id: kkstrtext.cc,v 1.32 2003/07/16 23:41:10 konst Exp $
 *
 * Copyright (C) 1999-2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -23,6 +23,7 @@
 */
 
 #include "kkstrtext.h"
+#include <errno.h>
 
 char *strcut(char *strin, int frompos, int count) {
     if(count > 0) {
@@ -137,7 +138,7 @@ string unmime(const string &text) {
     char *buf = new char[text.size()+1];
     strcpy(buf, text.c_str());
     r = unmime(buf);
-    delete buf;
+    delete[] buf;
     return r;
 }
 
@@ -145,6 +146,22 @@ string mime(const string &text) {
     string r;
     char *buf = new char[text.size()*3+1];
     r = mime(buf, text.c_str());
+    delete[] buf;
+    return r;
+}
+
+string fromutf8(const string &text) {
+    string r;
+    char *buf = (char *) utf8_to_str(text.c_str());
+    r = buf;
+    delete buf;
+    return r;
+}
+
+string toutf8(const string &text) {
+    string r;
+    char *buf = str_to_utf8(text.c_str());
+    r = buf;
     delete buf;
     return r;
 }
@@ -452,6 +469,7 @@ string trailcut(const string &base, const string &delim) {
 string getword(string &base, const string &delim) {
     string sub;
     int i;
+    bool found = false;
 
     base = leadcut(base, delim);
     
@@ -460,10 +478,11 @@ string getword(string &base, const string &delim) {
 	sub.resize(i);
 	base.replace(0, i, "");
 	base = leadcut(base, delim);
+	found = true;
 	break;
     }
 
-    if(sub == base) base = "";
+    if(!found) base = "";
     return sub;
 }
 
@@ -597,7 +616,10 @@ void breakintolines(string text, vector<string> &lst, int linelen) {
 		    if(dpos) nlen = dpos; else nlen = 1;
 		}
 
-		sub = i->substr(nlen + ((dpos != -1) ? 1 : 0));
+		if(dpos != -1)
+		    nlen++;
+
+		sub = i->substr(nlen);
 		i->erase(nlen);
 		lst.insert(i+1, sub);
 		i = lst.begin();
@@ -742,12 +764,15 @@ string strdateandtime(struct tm *tms, const string &fmt) {
     string afmt = fmt;
 
     if(afmt.empty()) {
-	if(current_time > when + 6L * 30L * 24L * 60L * 60L /* Old. */
-	|| current_time < when - 60L * 60L)                 /* Future. */ {
+	afmt = "%b %e %Y %H:%M";
+/*
+	if(current_time > when + 6L * 30L * 24L * 60L * 60L // Old.
+	|| current_time < when - 60L * 60L) {               // Future.
 	    afmt = "%b %e  %Y";
 	} else {
 	    afmt = "%b %e %H:%M";
 	}
+*/
     }
 
     strftime(buf, 512, afmt.c_str(), tms);
@@ -856,44 +881,53 @@ string ruscase(const string &s, const string &mode) {
     return r;
 }
 
-string siconv(const string &text, const string &fromcs, const string &tocs) {
+#ifdef ICONV_CONST
+#define ICONV_CCHAR (const char **)
+#else
+#define ICONV_CCHAR
+#endif
+
+string siconv(const string &atext, const string &fromcs, const string &tocs) {
 #ifdef HAVE_ICONV_H
     iconv_t cd = iconv_open(tocs.c_str(), fromcs.c_str());
 
     if(((int) cd) != -1) {
-	string r;
+	string r, text(atext);
 	size_t inleft, outleft, soutleft;
 	char *inbuf, *outbuf, *sinbuf, *soutbuf;
 
-	sinbuf = inbuf = strdup(text.c_str());
-	inleft = strlen(inbuf);
+	while(!text.empty()) {
+	    sinbuf = inbuf = strdup(text.c_str());
+	    inleft = strlen(inbuf);
 
-	soutleft = outleft = inleft*4;
-	soutbuf = outbuf = new char[outleft];
+	    soutleft = outleft = inleft*4;
+	    soutbuf = outbuf = new char[outleft];
 
-	size_t res = iconv(cd,
-#ifdef ICONV_CONST
-	    (const char **)
-#endif
-	    &inbuf, &inleft, &outbuf, &outleft);
+	    size_t res = iconv(cd, ICONV_CCHAR &inbuf, &inleft,
+		&outbuf, &outleft);
 
-	iconv_close(cd);
-
-	if(res != -1) {
 	    soutbuf[soutleft-outleft] = 0;
-	    r = soutbuf;
-	} else {
-	    r = text;
+	    r += soutbuf;
+	    text.erase(0, text.size()-inleft);
+
+	    delete soutbuf;
+	    delete sinbuf;
+
+	    if(res == -1 && errno != EILSEQ)
+		break;
+
+	    if(!text.empty()) {
+		text.erase(0, 1);
+		r += " ";
+	    }
 	}
 
-	delete soutbuf;
-	delete sinbuf;
-
+	iconv_close(cd);
 	return r;
     }
 #endif
 
-    return text;
+    return atext;
 }
 
 string cuthtml(const string &html, bool cutbrs) {
@@ -924,3 +958,73 @@ string cuthtml(const string &html, bool cutbrs) {
 
     return r;
 }
+
+char *utf8_to_str(const char *pin) {
+    int n = 0, i = 0, inlen;
+    unsigned char *result;
+    const unsigned char *in = (unsigned char *) pin;
+
+    if(!in) return NULL;
+
+    inlen = strlen(pin);
+    result = new unsigned char[inlen + 1];
+
+    while (n <= inlen - 1) {
+	long c = (long)in[n];
+	if(c < 0x80) result[i++] = (char)c; else {
+	    if((c & 0xC0) == 0xC0) result[i++] = (char)(((c & 0x03) << 6) | (((unsigned char)in[++n]) & 0x3F));
+	    else if ((c & 0xE0) == 0xE0) {
+		if (n + 2 <= inlen) {
+		    result[i] = (char)(((c & 0xF) << 4) | (((unsigned char)in[++n]) & 0x3F));
+		    result[i] = (char)(((unsigned char)result[i]) | (((unsigned char)in[++n]) & 0x3F));
+		    i++;
+		} else n += 2;
+	    } else if ((c & 0xF0) == 0xF0) n += 3;
+	    else if ((c & 0xF8) == 0xF8)
+		n += 4;
+	    else if ((c & 0xFC) == 0xFC)
+		n += 5;
+	}
+	n++;
+    }
+
+    result[i] = '\0';
+    return (char *) result;
+}
+
+char *str_to_utf8(const char *pin) {
+    int n = 0, i = 0;
+    int inlen;
+    char *result = NULL;
+    const unsigned char *in = (unsigned char *) pin;
+
+    if(!in)
+	return NULL;
+
+    inlen = strlen(pin);
+    result = new char[inlen * 2 + 1];
+
+    while (n < inlen) {
+	long c = (long)in[n];
+	if (c == 27) {
+	    n += 2;
+	    if (in[n] == 'x')
+		n++;
+	    if (in[n] == '3')
+		n++;
+	    n += 2;
+	    continue;
+	}
+
+	if (c < 128)
+	    result[i++] = (char)c;
+	else {
+	    result[i++] = (char)((c >> 6) | 192);
+	    result[i++] = (char)((c & 63) | 128);
+	}
+	n++;
+    }
+
+    result[i] = '\0';
+    return result;
+}
