@@ -19,6 +19,10 @@
 #include <iconv.h>
 #endif
 
+//
+// Gives out a number with the both limits _included_
+//
+
 #define randlimit(l, h) ((int)((float)rand()/(float)RAND_MAX*(float)(h+1-l)+(float)l))
 
 #define SWAPVAL(v1, v2) { \
@@ -99,6 +103,9 @@ bool getstring(istream &f, string &buf);
 string unmime(const string &text);
 string mime(const string &text);
 
+string toutf8(const string &text);
+string fromutf8(const string &text);
+
 string ruscase(const string &s, const string &mode);
 string siconv(const string &text, const string &fromcs, const string &tocs);
 string cuthtml(const string &html, bool cutbrs = false);
@@ -145,6 +152,9 @@ time_t str2time(char *sdate, char *mask, time_t *t);
 char *unmime(char *text);
 char *mime(char *dst, const char *src);
 
+char *utf8_to_str(const char *pin);
+char *str_to_utf8(const char *pin);
+
 __KTOOL_END_C
 
 #endif
