@@ -26,7 +26,7 @@
 #include <map>
 
 #include <libicq2000/Contact.h>
-#include <sigc++/signal_system.h>
+#include <libicq2000/sigslot.h>
 
 namespace ICQ2000 {
 
@@ -105,7 +105,7 @@ namespace ICQ2000 {
     const_iterator begin() const;
     const_iterator end() const;
 
-    SigC::Signal1<void,ContactListEvent*> contactlist_signal;
+    sigslot::signal1<ContactListEvent*> contactlist_signal;
   };
 
 }
