@@ -24,16 +24,24 @@
 
 #include <libicq2000/Cache.h>
 
-#include <sigc++/signal_system.h>
-
-using SigC::Signal1;
+#include <libicq2000/sigslot.h>
 
 namespace ICQ2000 {
 
-  // fd -> DirectClient
+  /* fd -> DirectClient cache
+   *
+   * Where the party is at. Once a DirectClient object is added to the
+   * cache MM for it is assumed on the cache. Also lookups for a
+   * Contact are done in linear time now, it was just too much of a
+   * head-ache maintaining the uin map in Client.
+   */
   class DCCache : public Cache<int, DirectClient*> {
    public:
     DCCache() { }
+    ~DCCache()
+    {
+      removeAll();
+    }
 
     void removeItem(const DCCache::literator& l) {
       delete ((*l).getValue());
@@ -43,22 +51,51 @@ namespace ICQ2000 {
     void expireItem(const DCCache::literator& l) {
       expired.emit( (*l).getValue() );
       Cache<int, DirectClient*>::expireItem(l);
+      /* this will removeItem(..), which'll delete the DirectClient
+       * (see above).
+       */
     }
 
-    void removeContact(Contact *c) {
+    void removeContact(const ContactRef& c) {
       literator curr = m_list.begin();
       literator next = curr;
       while ( curr != m_list.end() ) {
 	DirectClient *dc = (*curr).getValue();
 	++next;
-	if ( dc->getContact() == c ) {
-	  dc->setContact(NULL); // invalidate contact so the DC doesn't attempt to redeliver
+	if ( dc->getContact().get() != NULL
+	     /* Direct Connections won't have a contact associated
+	      * with them initially just after having been accepted as
+	      * an incoming connection (we don't know who they are
+	      * yet) - so Contact could be a NULL ref.
+	      */
+	     && dc->getContact()->getUIN() == c->getUIN() ) {
 	  removeItem(curr);
 	}
 	curr = next;
       }
     }
 
+    DirectClient* getByContact(const ContactRef& c)
+    {
+      // linear lookup
+      literator curr = m_list.begin();
+      while ( curr != m_list.end() ) {
+	DirectClient *dc = (*curr).getValue();
+	if (  dc->getContact().get() != NULL
+	     /* Direct Connections won't have a contact associated
+	      * with them initially just after having been accepted as
+	      * an incoming connection (we don't know who they are
+	      * yet) - so Contact could be a NULL ref.
+	      */
+	     && dc->getContact()->getUIN() == c->getUIN() )
+	  return dc; // found it
+
+	++curr;
+      }
+
+      return NULL; // not found
+    }
+
     void clearoutMessagesPoll() {
       literator curr = m_list.begin();
       while ( curr != m_list.end() ) {
@@ -68,7 +105,7 @@ namespace ICQ2000 {
       }
     }
 
-    Signal1<void,DirectClient*> expired;
+    sigslot::signal1<DirectClient*> expired;
   };
   
 }
