@@ -23,9 +23,8 @@
 #define DIRECTCLIENT_H
 
 #include <list>
-#include <string>
 
-#include <sigc++/signal_system.h>
+#include <libicq2000/sigslot.h>
 
 #include <stdlib.h>
 
@@ -40,12 +39,6 @@
 #include <libicq2000/SocketClient.h>
 #include <libicq2000/MessageHandler.h>
 
-using std::string;
-using std::list;
-using std::exception;
-using SigC::Signal0;
-using SigC::Signal1;
-
 namespace ICQ2000 {
 
   class UINICQSubType;
@@ -96,7 +89,7 @@ namespace ICQ2000 {
     static unsigned char client_check_data[];
     Translator *m_translator;
     SeqNumCache m_msgcache;
-    list<MessageEvent*> m_msgqueue;
+    std::list<MessageEvent*> m_msgqueue;
     unsigned short m_seqnum;
 
     unsigned short NextSeqNum();
