@@ -215,6 +215,19 @@ namespace ICQ2000 {
       SrvUpdateAboutInfo(unsigned int my_uin, const std::string& about_info);
       unsigned short Subtype() const { return SNAC_SRV_Send; }
  };
+
+  class SrvChangePassword : public SrvFamilySNAC, public OutSNAC {
+    private:
+      unsigned int m_my_uin;
+      const std::string& m_new_password;
+
+    protected:
+      void OutputBody(Buffer& b) const;
+
+    public:
+      SrvChangePassword(unsigned int my_uin, const std::string& new_password);
+      unsigned short Subtype() const { return SNAC_SRV_Send; }
+ };
   
   class SrvRequestRandomChat : public SrvFamilySNAC, public OutSNAC {
     private:
@@ -247,6 +260,7 @@ namespace ICQ2000 {
   const unsigned short SrvResponse_AckWorkInfoChange     = 0x006E;
   const unsigned short SrvResponse_AckHomepageInfoChange = 0x0078;
   const unsigned short SrvResponse_AckAboutInfoChange    = 0x0082;
+  const unsigned short SrvResponse_AckPasswordChange     = 0x00aa;
   const unsigned short SrvResponse_AckRandomGroupSet     = 0x0370;
   const unsigned short SrvResponse_SMS_Done       = 0x0096;
   const unsigned short SrvResponse_SimpleUI       = 0x0190;
@@ -284,6 +298,7 @@ namespace ICQ2000 {
       AckHomepageInfoChange,
       AckWorkInfoChange,
       AckAboutInfoChange,
+      AckPasswordChange,
       RandomChatFound,
       RandomGroupSet
     };
