@@ -25,20 +25,24 @@
 #include <libicq2000/Cache.h>
 #include <libicq2000/events.h>
 
-#include <sigc++/signal_system.h>
+#include <libicq2000/sigslot.h>
 
 namespace ICQ2000 {
 
   class SeqNumCache : public Cache<unsigned short, MessageEvent*> {
    public:
     SeqNumCache() { }
-
+    ~SeqNumCache()
+    {
+      removeAll();
+    }
+    
     void expireItem(const SeqNumCache::literator& l) {
       expired.emit( (*l).getValue() );
       Cache<unsigned short, MessageEvent*>::expireItem(l);
     }
 
-    SigC::Signal1<void,MessageEvent*> expired;
+    sigslot::signal1<MessageEvent*> expired;
   };
   
 }
