@@ -352,7 +352,8 @@ namespace ICQ2000 {
       EmailEx,
       UserAdd,
       Email,
-      WebPager
+      WebPager,
+      Contacts
     };
 
     enum DeliveryFailureReason {
@@ -638,6 +639,24 @@ namespace ICQ2000 {
     MessageType getType() const;
   };
 
+  /**
+   *  A contacts message
+   */
+  class ContactMessageEvent : public ICQMessageEvent {
+   private:
+    std::list<ContactRef> m_content;
+
+   public:
+    ContactMessageEvent(ContactRef c, std::list<ContactRef> content);
+
+    MessageType getType() const;
+    unsigned int getSenderUIN() const;
+
+    std::list<ContactRef> getContacts() const;
+
+    ICQMessageEvent* copy() const;
+  };
+
   // ============================================================================
   //  Search Events
   // ============================================================================
@@ -781,6 +800,24 @@ namespace ICQ2000 {
     unsigned int getMaxAvg() const { return m_maxavg; }
   };
 
+  /**
+   *  Password change
+   */
+  class PasswordChangeEvent : public Event {
+   private:
+    bool m_success;
+    std::string m_new_password;
+
+   public:
+    PasswordChangeEvent(const std::string &new_pass): 
+      m_new_password(new_pass) { };
+
+    bool isSuccess() const { return m_success; }
+    std::string getPassword() const { return m_new_password; }
+
+    void setSuccess(bool success) { m_success = success; }
+  };
+
 } 
 
 #endif
