@@ -27,8 +27,6 @@
 
 #include "sstream_fix.h"
 
-#include <sigc++/bind.h>
-
 #include <vector>
 
 using std::string;
@@ -109,35 +107,35 @@ namespace ICQ2000 {
 
     m_cookiecache.setDefaultTimeout(30);
     // 30 seconds is hopefully enough for even the slowest connections
-    m_cookiecache.expired.connect( slot(this,&Client::ICBMCookieCache_expired_cb) );
+    m_cookiecache.expired.connect( this,&Client::ICBMCookieCache_expired_cb );
 
     m_dccache.setDefaultTimeout(30);
     // set timeout on direct connections to 30 seconds
     // this will be increased once they are established
-    m_dccache.expired.connect( slot(this,&Client::dccache_expired_cb) );
+    m_dccache.expired.connect( this,&Client::dccache_expired_cb );
 
-    m_reqidcache.expired.connect( slot(this, &Client::reqidcache_expired_cb) );
+    m_reqidcache.expired.connect( this, &Client::reqidcache_expired_cb );
     
-    m_smtp.logger.connect( slot(this, &Client::dc_log_cb) );
-    m_smtp.messageack.connect( slot(this, &Client::dc_messageack_cb) );
-    m_smtp.socket.connect( slot(this, &Client::dc_socket_cb) );
+    m_smtp.logger.connect( this, &Client::dc_log_cb );
+    m_smtp.messageack.connect( this, &Client::dc_messageack_cb );
+    m_smtp.socket.connect( this, &Client::dc_socket_cb );
 
     /* contact list callbacks */
-    m_contact_list.contactlist_signal.connect( slot(this, &Client::contactlist_cb) );
+    m_contact_list.contactlist_signal.connect( this, &Client::contactlist_cb );
     
     /* visible, invisible lists callbacks */
-    m_visible_list.contactlist_signal.connect( slot(this, &Client::visiblelist_cb) );
-    m_invisible_list.contactlist_signal.connect( slot(this, &Client::invisiblelist_cb) );
+    m_visible_list.contactlist_signal.connect( this, &Client::visiblelist_cb );
+    m_invisible_list.contactlist_signal.connect( this, &Client::invisiblelist_cb );
 
     /* self contact callbacks */
-    m_self->status_change_signal.connect( self_contact_status_change_signal.slot() );
-    m_self->userinfo_change_signal.connect( self_contact_userinfo_change_signal.slot() );
+    m_self->status_change_signal.connect( self_contact_status_change_signal );
+    m_self->userinfo_change_signal.connect( self_contact_userinfo_change_signal );
     
     /* message handler callbacks */
-    m_message_handler.messaged.connect( messaged.slot() );
-    m_message_handler.messageack.connect( messageack.slot() );
-    m_message_handler.want_auto_resp.connect( want_auto_resp.slot() );
-    m_message_handler.logger.connect( logger.slot() );
+    m_message_handler.messaged.connect( messaged );
+    m_message_handler.messageack.connect( messageack );
+    m_message_handler.want_auto_resp.connect( want_auto_resp );
+    m_message_handler.logger.connect( logger );
   }
 
   unsigned short Client::NextSeqNum() {
@@ -167,6 +165,7 @@ namespace ICQ2000 {
       }
       m_serverSocket.setRemoteHost(m_authorizerHostname.c_str());
       m_serverSocket.setRemotePort(m_authorizerPort);
+      m_serverSocket.setBindHost(m_client_bind_host.c_str());
 
       m_serverSocket.setBlocking(false);
 
@@ -201,6 +200,7 @@ namespace ICQ2000 {
     try {
       m_serverSocket.setRemoteHost(m_bosHostname.c_str());
       m_serverSocket.setRemotePort(m_bosPort);
+      m_serverSocket.setBindHost(m_client_bind_host.c_str());
 
       SignalLog(LogEvent::INFO, "Establishing TCP Connection to BOS Server");
       m_serverSocket.setBlocking(false);
@@ -583,6 +583,28 @@ namespace ICQ2000 {
 	SignalLog(LogEvent::WARN, e.what());
       }
 
+    } else if (snac->getType() == SrvResponseSNAC::AckPasswordChange) {
+
+      if ( m_reqidcache.exists( snac->RequestID() ) ) {
+	RequestIDCacheValue *v = m_reqidcache[ snac->RequestID() ];
+
+	if ( v->getType() == RequestIDCacheValue::PasswordChange ) {
+	  PasswordChangeCacheValue *sv = static_cast<PasswordChangeCacheValue*>(v);
+	  PasswordChangeEvent *ev = sv->getEvent();
+
+	  m_password = ev->getPassword();
+
+	  ev->setSuccess(true);
+	  password_changed.emit(ev);
+
+	  delete ev;
+	  m_reqidcache.remove( snac->RequestID() );
+	  
+	} else {
+	  SignalLog(LogEvent::WARN, "Request ID cached value is not for a Password Change request");
+	}
+      }
+
     } else if (snac->getType() == SrvResponseSNAC::RandomChatFound) {
 
       if ( m_reqidcache.exists( snac->RequestID() ) ) {
@@ -681,7 +703,7 @@ namespace ICQ2000 {
     SignalLog(LogEvent::WARN, "Direct connection timeout reached");
   }
 
-  void Client::dc_connected_cb(DirectClient *dc) {
+  void Client::dc_connected_cb(SocketClient *dc) {
     m_dccache.setTimeout(dc->getfd(), 600);
     // once we are properly connected a direct
     // connection will only timeout after 10 mins
@@ -841,7 +863,7 @@ namespace ICQ2000 {
     SignalLog(LogEvent::INFO, "Sending Rate Info Request");
     FLAPwrapSNACandSend( RequestRateInfoSNAC() );
   }
-
+  
   void Client::SendRateInfoAck() {
     SignalLog(LogEvent::INFO, "Sending Rate Info Ack");
     FLAPwrapSNACandSend( RateInfoAckSNAC() );
@@ -863,6 +885,7 @@ namespace ICQ2000 {
     // startup listening server at this point, so we
     // know the listening port and ip
     if (m_in_dc) {
+      m_listenServer.setBindHost(m_client_bind_host.c_str());
       if (m_use_portrange) {
 	m_listenServer.StartServer(m_lower_port, m_upper_port);
       } else {
@@ -1176,13 +1199,13 @@ namespace ICQ2000 {
 	break;
       case SNAC_SBL_Edit_Access_Granted:
 	SignalLog(LogEvent::INFO, "Access to the server-based contact list was granted\n");
-	break;
+      break;
       case SNAC_SBL_Modification_Ack: {
 	vector<ModificationAckSBLSNAC::Result> r = static_cast<ModificationAckSBLSNAC*>(snac)->getResults();
 	vector<ModificationAckSBLSNAC::Result>::iterator ir;
 	vector<ServerBasedContactEvent::UploadResult> updresults;
 	vector<ServerBasedContactEvent::UploadResult>::iterator iur;
-
+	
 	for(ir = r.begin(); ir != r.end(); ++ir)
 	switch( *ir ) {
 	  case ModificationAckSBLSNAC::Success:
@@ -1194,7 +1217,7 @@ namespace ICQ2000 {
 	    updresults.push_back(ServerBasedContactEvent::Failed);
 	    break;
 	  case ModificationAckSBLSNAC::AuthRequired:
-	    SignalLog(LogEvent::INFO, "Authentification is required to perform the server-based modification\n");
+	    SignalLog(LogEvent::INFO, "Succeeded, though authentification is required to perform the server-based modification\n");
 	    updresults.push_back(ServerBasedContactEvent::AuthRequired);
 	    break;
 	  case ModificationAckSBLSNAC::AlreadyExists:
@@ -1216,7 +1239,8 @@ namespace ICQ2000 {
 	      iur = updresults.begin();
 
 	      while(iur != updresults.end() && ic != cl.end()) {
-		if(*iur == ServerBasedContactEvent::Success) {
+		if(*iur == ServerBasedContactEvent::Success
+		|| *iur == ServerBasedContactEvent::AuthRequired) {
 		    (*ic)->setServerBased(ev->getType() == ServerBasedContactEvent::Upload);
 
 		    ContactRef ct = getContact((*ic)->getUIN());
@@ -1246,7 +1270,7 @@ namespace ICQ2000 {
 	break;
       }
       break;
-
+	
     } // switch(Family)
 
     if (dynamic_cast<RawSNAC*>(snac)) {
@@ -1458,10 +1482,10 @@ namespace ICQ2000 {
       DirectClient *dc = new DirectClient(m_self, sock, &m_message_handler, &m_contact_list,
 					  m_ext_ip, m_listenServer.getPort(), &m_translator);
       m_dccache[ sock->getSocketHandle() ] = dc;
-      dc->logger.connect( slot(this, &Client::dc_log_cb) );
-      dc->messageack.connect( slot(this, &Client::dc_messageack_cb) );
-      dc->connected.connect( SigC::bind<DirectClient*>( slot(this, &Client::dc_connected_cb), dc ) );
-      dc->socket.connect( slot(this, &Client::dc_socket_cb) );
+      dc->logger.connect( this, &Client::dc_log_cb );
+      dc->messageack.connect( this, &Client::dc_messageack_cb );
+      dc->connected.connect( this, &Client::dc_connected_cb );
+      dc->socket.connect( this, &Client::dc_socket_cb );
       SignalAddSocket( sock->getSocketHandle(), SocketEvent::READ );
 
     } else {
@@ -1574,6 +1598,7 @@ namespace ICQ2000 {
       case MessageEvent::Normal:
       case MessageEvent::URL:
       case MessageEvent::AwayMessage:
+      case MessageEvent::Contacts:
       if (!SendDirect(ev)) SendViaServer(ev);
 	break;
 
@@ -1610,10 +1635,10 @@ namespace ICQ2000 {
       SignalLog(LogEvent::INFO, "Establishing direct connection");
       dc = new DirectClient(m_self, c, &m_message_handler,
 			    m_ext_ip, (m_in_dc ? m_listenServer.getPort() : 0), &m_translator);
-      dc->logger.connect( slot(this, &Client::dc_log_cb) );
-      dc->messageack.connect( slot(this, &Client::dc_messageack_cb) );
-      dc->connected.connect( SigC::bind<DirectClient*>( slot(this, &Client::dc_connected_cb), dc ) );
-      dc->socket.connect( slot(this, &Client::dc_socket_cb) );
+      dc->logger.connect( this, &Client::dc_log_cb );
+      dc->messageack.connect( this, &Client::dc_messageack_cb );
+      dc->connected.connect( this, &Client::dc_connected_cb );
+      dc->socket.connect( this, &Client::dc_socket_cb );
 
       try {
 	dc->Connect();
@@ -1640,7 +1665,8 @@ namespace ICQ2000 {
     ContactRef c = ev->getContact();
 
     if (ev->getType() == MessageEvent::Normal
-	|| ev->getType() == MessageEvent::URL) {
+	|| ev->getType() == MessageEvent::URL
+	|| ev->getType() == MessageEvent::Contacts) {
       
       /*
        * Normal messages and URL messages sent via the server
@@ -1761,7 +1787,7 @@ namespace ICQ2000 {
     ev->setDirect(false);
     ICQMessageEvent *cev;
     if ((cev = dynamic_cast<ICQMessageEvent*>(ev)) != NULL) cev->setOfflineMessage(true);
-
+    
     if (ev->getType() == MessageEvent::AuthReq) {
       ev->getContact()->setAuthAwait(true);
     }
@@ -1788,7 +1814,7 @@ namespace ICQ2000 {
     
     Send(b);
   }
-
+    
   void Client::uploadServerBasedContactList()
   {
     uploadServerBasedContactList(m_contact_list);
@@ -1946,6 +1972,20 @@ namespace ICQ2000 {
     }
   }
 
+  void Client::changePassword(const string &new_pass) {
+    if(m_password != new_pass && m_state != NOT_CONNECTED) {
+      PasswordChangeEvent *ev = new PasswordChangeEvent( new_pass );
+
+      unsigned int reqid = NextRequestID();
+      m_reqidcache.insert( reqid, new PasswordChangeCacheValue( ev ) );
+      SrvChangePassword ssnac( m_self->getUIN(), new_pass );
+      ssnac.setRequestID( reqid );
+
+      SignalLog(LogEvent::INFO, "Changing our password");
+      FLAPwrapSNACandSend( ssnac );
+    }
+  }
+
   void Client::setServerSideGroup(const std::string &group_name, unsigned short group_id) {
     m_default_group = group_id;
     m_default_group_name = group_name;
@@ -2075,10 +2115,10 @@ namespace ICQ2000 {
       m_contact_list.add(c);
 
       if(!c->getServerBased())
-	c->setServerSideInfo(m_default_group, 0);
+      c->setServerSideInfo(m_default_group, 0);
 
-      c->status_change_signal.connect( contact_status_change_signal.slot() );
-      c->userinfo_change_signal.connect( contact_userinfo_change_signal.slot() );
+      c->status_change_signal.connect( contact_status_change_signal );
+      c->userinfo_change_signal.connect( contact_userinfo_change_signal );
     }
 
   }
@@ -2141,7 +2181,7 @@ namespace ICQ2000 {
       m_invisible_list.remove(uin);
     }
   }
-
+  
   void Client::SignalServerBasedContactList(const ContactList& l) {
     ContactRef ct;
     ContactList::const_iterator curr = l.begin();
@@ -2149,7 +2189,7 @@ namespace ICQ2000 {
     while(curr != l.end()) {
       ct = getContact((*curr)->getUIN());
       if(ct.get()) {
-        ct->setServerBased(true);
+	ct->setServerBased(true);
 	ct->setServerSideInfo((*curr)->getServerSideGroupID(), (*curr)->getServerSideID());
       }
 
@@ -2652,5 +2692,16 @@ namespace ICQ2000 {
     return m_use_portrange;
   }
 
+  void Client::setClientBindHost(const std::string& host)
+  {
+    m_client_bind_host = host;
+    m_smtp.setClientBindHost(host);
+  }
+
+  std::string Client::getClientBindHost() const
+  {
+    return m_client_bind_host;
+  }
+
 }
 
