@@ -431,7 +431,7 @@ namespace ICQ2000 {
 	b.setAutoSizeMarker(m1);
 	b.setAutoSizeMarker(m2);
     }
-
+    
   SrvSetRandomChatGroup::SrvSetRandomChatGroup(unsigned int my_uin, unsigned short random_group)
     : m_my_uin(my_uin), m_random_group(random_group) { }
     
@@ -465,6 +465,27 @@ namespace ICQ2000 {
 	b.setAutoSizeMarker(m1);
 	b.setAutoSizeMarker(m2);
     }
+
+  SrvChangePassword::SrvChangePassword(unsigned int my_uin, const string& new_password)
+    : m_my_uin(my_uin), m_new_password(new_password) { }
+    
+  void SrvChangePassword::OutputBody(Buffer& b) const {
+	b << (unsigned short)0x0001;
+
+	Buffer::marker m1 = b.getAutoSizeShortMarker();
+    
+	b.setLittleEndian();
+	Buffer::marker m2 = b.getAutoSizeShortMarker();
+
+	b << m_my_uin;
+
+	b << (unsigned short)2000   /* type 9808 */
+	    << (unsigned short)m_requestID /* low word of the request ID */
+	    << (unsigned short)0x042e; /* subtype */
+	b.PackUint16TranslatedNull(m_new_password);
+	b.setAutoSizeMarker(m1);
+	b.setAutoSizeMarker(m2);
+    }
     
   SrvResponseSNAC::SrvResponseSNAC() : m_icqsubtype(NULL) { }
 
@@ -584,6 +605,7 @@ namespace ICQ2000 {
     case SrvResponse_AckWorkInfoChange:
     case SrvResponse_AckHomepageInfoChange:
     case SrvResponse_AckAboutInfoChange:
+    case SrvResponse_AckPasswordChange:
       ParseInfoChangeAck(b, subtype);
       break;
     case SrvResponse_RandomChatFound:
@@ -614,6 +636,9 @@ namespace ICQ2000 {
     case SrvResponse_AckAboutInfoChange:
       m_type = AckAboutInfoChange;
       break;
+    case SrvResponse_AckPasswordChange:
+      m_type = AckPasswordChange;
+      break;
     default:
       throw ParseException("Unknown info change acknowledgment");
     }
