@@ -946,7 +946,7 @@ namespace ICQ2000 {
    * @param t time the message was sent
    */
   AuthAckEvent::AuthAckEvent(ContactRef c, const string& msg,
-			     bool granted, time_t t)
+                             bool granted, time_t t)
     : ICQMessageEvent(c), m_message(msg), m_granted(granted) {
     setOfflineMessage(true);
     m_time = t;
@@ -1038,6 +1038,21 @@ namespace ICQ2000 {
 
   EmailMessageEvent::MessageType EmailMessageEvent::getType() const { return MessageEvent::Email; }
 
+  ContactMessageEvent::ContactMessageEvent(ContactRef c, std::list<ContactRef> content)
+  : ICQMessageEvent(c), m_content(content) {
+  }
+
+  MessageEvent::MessageType ContactMessageEvent::getType() const { return MessageEvent::Contacts; }
+
+  unsigned int ContactMessageEvent::getSenderUIN() const { return m_contact->getUIN(); }
+
+  std::list<ContactRef> ContactMessageEvent::getContacts() const { return m_content; }
+
+  ICQMessageEvent* ContactMessageEvent::copy() const
+  {
+    return new ContactMessageEvent(*this);
+  }
+
   // ============================================================================
   //  New UIN
   // ============================================================================
@@ -1084,14 +1099,14 @@ namespace ICQ2000 {
    * @param maxavg the maximum average
    */
   RateInfoChangeEvent::RateInfoChangeEvent(unsigned short code, 
-					   unsigned short rateclass,
-					   unsigned int windowsize,
-					   unsigned int clear,
-					   unsigned int alert,
-					   unsigned int limit,
-					   unsigned int disconnect,
-					   unsigned int currentavg,
-					   unsigned int maxavg) 
+                                           unsigned short rateclass,
+                                           unsigned int windowsize,
+                                           unsigned int clear,
+                                           unsigned int alert,
+                                           unsigned int limit,
+                                           unsigned int disconnect,
+                                           unsigned int currentavg,
+                                           unsigned int maxavg) 
     : m_code(code), m_rateclass(rateclass), m_windowsize(windowsize),
       m_clear(clear), m_alert(alert), m_limit(limit), m_disconnect(disconnect), 
       m_currentavg(currentavg), m_maxavg(maxavg) { }
