@@ -1,3 +1,27 @@
+/*
+*
+* centericq account manager dialog implementation
+* $Id: accountmanager.cc,v 1.29 2003/07/12 17:14:20 konst Exp $
+*
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or (at
+* your option) any later version.
+*
+* This program is distributed in the hope that it will be useful, but
+* WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+* General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+* USA
+*
+*/
+
 #include "accountmanager.h"
 #include "icqface.h"
 #include "icqhook.h"
@@ -25,9 +49,10 @@ void accountmanager::exec() {
     dialogbox db;
     protocolname pname;
     icqconf::imaccount account;
-    int n, b, i, citem, action, pos, capab;
+    int n, b, i, citem, action, pos;
+    set<hookcapab::enumeration> capab;
     string spname, tmp;
-    bool fin, proceed;
+    bool fin;
 
     face.blockmainscreen();
     fopen = true;
@@ -51,19 +76,26 @@ void accountmanager::exec() {
 	t.clear();
 
 	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	    if(pname != infocard) {
+	    account = conf.getourid(pname);
+
+	    if(pname != rss)
+	    if(gethook(pname).enabled() || !account.empty()) {
 		account = conf.getourid(pname);
 		n = t.addnode(0, 0, 0, " " + conf.getprotocolname(pname) + " ");
 		citem = ((int) (pname)+1) * 100;
-		capab = gethook(pname).getcapabilities();
+		capab = gethook(pname).getCapabs();
 
-		if(!account.empty() && (capab & hoptChangableServer)) {
+		if(!account.empty()) {
 		    tmp = "";
 
 		    if(!account.server.empty() && account.port)
 			tmp = account.server + ":" + i2str(account.port);
 
 		    t.addleaff(n, 0, citem+9, _(" Server : %s "), tmp.c_str());
+
+		    if(capab.count(hookcapab::ssl))
+			t.addleaff(n, 0, citem+13, _(" Secured : %s "),
+			    stryesno(account.additional["ssl"] == "1"));
 		}
 
 		switch(pname) {
@@ -78,24 +110,24 @@ void accountmanager::exec() {
 			break;
 		}
 
-		if(!(capab & hoptNoPasswords)) {
-		    t.addleaff(n, 0, citem+5, _(" Password : %s "),
-			string(account.password.size(), '*').c_str());
-		}
+		t.addleaff(n, 0, citem+5, capab.count(hookcapab::optionalpassword) ?
+		    _(" Password (optional) : %s ") : _(" Password : %s "),
+		    string(account.password.size(), '*').c_str());
 
 		if(account.empty()) {
 		    t.addnode(n, 0, citem+6, _(" Register "));
 		} else {
-		    if(capab & hoptCanUpdateDetails)
-			t.addnode(n, 0, citem+7, _(" Update user details "));
 
-		    if(capab & hoptCanSetAwayMsg)
+		    if(capab.count(hookcapab::changedetails))
+			t.addnode(n, 0, citem+7,
+			    pname == msn ?
+			    _(" Change nickname ") :
+			    _(" Update user details "));
+
+		    if(capab.count(hookcapab::setaway))
 			t.addnode(n, 0, citem+10, _(" Set away message "));
 
-		    if(pname == irc)
-			t.addnode(n, 0, citem+11, _(" Channel manager "));
-
-		    if(capab & hoptCanSyncList)
+		    if(capab.count(hookcapab::synclist))
 			t.addnode(n, 0, citem+12, _(" Synchronize contact list "));
 
 		    t.addnode(n, 0, citem+8, _(" Drop "));
@@ -127,7 +159,8 @@ void accountmanager::exec() {
 
 		case 5:
 		    tmp = face.inputstr(spname + _(" password: "), account.password, '*');
-		    if(face.getlastinputkey() != KEY_ESC && !tmp.empty())
+		    if(face.getlastinputkey() != KEY_ESC &&
+		      (!tmp.empty() || capab.count(hookcapab::optionalpassword)))
 			account.password = tmp;
 		    break;
 
@@ -176,10 +209,6 @@ void accountmanager::exec() {
 		    }
 		    break;
 
-		case 11:
-		    imcontrol.channels(account);
-		    break;
-
 		case 12:
 		    if(!hook.online()) {
 			face.status(_("You must be logged in to preform the synchronization"));
@@ -188,6 +217,19 @@ void accountmanager::exec() {
 			fin = true;
 		    }
 		    break;
+
+		case 13:
+		    account.additional["ssl"] =
+			(account.additional["ssl"] == "") ? "1" : "";
+
+		    if(account.additional["ssl"] == "1") {
+			if(account.port == icqconf::defservers[account.pname].port)
+			    account.port = icqconf::defservers[account.pname].secureport;
+		    } else {
+			if(account.port == icqconf::defservers[account.pname].secureport)
+			    account.port = icqconf::defservers[account.pname].port;
+		    }
+		    break;
 	    }
 
 	    if(action != 7) {
@@ -199,7 +241,6 @@ void accountmanager::exec() {
     db.close();
     face.unblockmainscreen();
 
-    clist.checkdefault();
     conf.save();
     face.relaxedupdate();
     fopen = false;
