@@ -1,9 +1,9 @@
 /*
 *
 * centericq core routines
-* $Id: centericq.cc,v 1.114 2002/08/19 08:25:17 konst Exp $
+* $Id: centericq.cc,v 1.169 2003/10/01 18:48:11 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,19 +25,21 @@
 #include "centericq.h"
 #include "icqconf.h"
 #include "icqhook.h"
+#include "irchook.h"
 #include "yahoohook.h"
+#include "rsshook.h"
 #include "icqface.h"
 #include "icqcontact.h"
 #include "icqcontacts.h"
 #include "icqmlist.h"
 #include "icqgroups.h"
 #include "accountmanager.h"
+#include "imexternal.h"
 
 centericq::centericq()
     : timer_checkmail(0), timer_update(0), timer_resend(0),
-      regmode(false)
+      timer_autosave(/*time(0)*/0), regmode(false)
 {
-//    timer_keypress = time(0)-50;
 }
 
 centericq::~centericq() {
@@ -53,6 +55,7 @@ void centericq::exec() {
     sigaction(SIGALRM, &sact, 0);
     sigaction(SIGUSR1, &sact, 0);
     sigaction(SIGUSR2, &sact, 0);
+    sigaction(SIGTERM, &sact, 0);
 
     kinterface();
     raw();
@@ -61,14 +64,19 @@ void centericq::exec() {
     face.init();
 
     if(regmode = !conf.getouridcount()) {
+	bool rus = false;
 	char *p = getenv("LANG");
 
 	if(p)
-	if(((string) p).substr(0, 2) == "ru")
-	    conf.setrussian(true);
+	if(rus = (((string) p).substr(0, 2) == "ru"))
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		conf.setrussian(pname, true);
 
 	if(updateconf()) {
 	    manager.exec();
+#ifdef BUILD_KONST
+	    defaultcontacts(rus);
+#endif
 	}
 
 	regmode = false;
@@ -102,6 +110,10 @@ void centericq::exec() {
     conf.save();
 }
 
+void centericq::defaultcontacts(bool rus) {
+    clist.addnew(imcontact(17502151, icq), false);
+}
+
 bool centericq::checkpasswords() {
     protocolname pname;
     icqconf::imaccount ia;
@@ -110,7 +122,7 @@ bool centericq::checkpasswords() {
     r = regmode = true;
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	if(!(gethook(pname).getcapabilities() & hoptNoPasswords)) {
+	if(!gethook(pname).getCapabs().count(hookcapab::optionalpassword)) {
 	    if(!(ia = conf.getourid(pname)).empty()) {
 		if(ia.password.empty()) {
 		    conf.setsavepwd(false);
@@ -143,7 +155,7 @@ void centericq::inithooks() {
 }
 
 void centericq::mainloop() {
-    bool finished = false;
+    bool finished = false, r;
     string text, url;
     int action, old, gid;
     icqcontact *c;
@@ -172,6 +184,9 @@ void centericq::mainloop() {
 	    case ACT_ORG_GROUPS:
 		face.organizegroups();
 		break;
+	    case ACT_TRANSFERS:
+		face.transfermonitor();
+		break;
 	    case ACT_HIDEOFFLINE:
 		conf.sethideoffline(!conf.gethideoffline());
 		face.update();
@@ -182,6 +197,12 @@ void centericq::mainloop() {
 	    case ACT_FIND:
 		find();
 		break;
+	    case ACT_JOINDIALOG:
+		joindialog();
+		break;
+	    case ACT_RSS:
+		linkfeed();
+		break;
 	    case ACT_CONF:
 		updateconf();
 		break;
@@ -206,24 +227,41 @@ void centericq::mainloop() {
 		    c->getpostponed()), icqface::ok);
 		break;
 
-	    case ACT_IGNORE:
-		sprintf(buf, _("Ignore all events from %s?"), c->getdesc().totext().c_str());
-		if(face.ask(buf, ASK_YES | ASK_NO, ASK_NO) == ASK_YES) {
-		    lst.push_back(modelistitem(c->getdispnick(), c->getdesc(), csignore));
-		    clist.remove(c->getdesc());
-		    face.update();
-		}
+	    case ACT_CONTACT:
+		sendevent(imcontacts(c->getdesc(), imevent::outgoing, vector< pair<unsigned int, string> >()),
+		    icqface::ok);
+		break;
+
+	    case ACT_AUTH:
+		sendevent(imauthorization(c->getdesc(), imevent::outgoing,
+		    imauthorization::Request, ""), icqface::ok);
 		break;
 
 	    case ACT_FILE:
-		if(c->getstatus() != offline) {
-//                    sendfiles(c->getdesc());
+		sendevent(imfile(c->getdesc(), imevent::outgoing), icqface::ok);
+		break;
+
+	    case ACT_IGNORE:
+		if(lst.inlist(c->getdesc(), csignore)) {
+		    lst.del(c->getdesc(), csignore);
+		} else {
+		    sprintf(buf, _("Ignore all events from %s?"), c->getdesc().totext().c_str());
+		    if(face.ask(buf, ASK_YES | ASK_NO, ASK_NO) == ASK_YES) {
+			lst.push_back(modelistitem(c->getdispnick(), c->getdesc(), csignore));
+			if(!ischannel(c)) {
+			    sprintf(buf, _("Remove %s from the contact list as well?"), c->getdesc().totext().c_str());
+			    if(face.ask(buf, ASK_YES | ASK_NO, ASK_NO) == ASK_YES) {
+				clist.remove(c->getdesc());
+				face.update();
+			    }
+			}
+		    }
 		}
 		break;
 
 	    case ACT_EDITUSER:
 		c->save();
-		if(face.updatedetails(c)) {
+		if(face.updatedetails(c, c->getdesc().pname)) {
 		    if(c->getdesc().pname == infocard)
 			c->setdispnick(c->getnick());
 
@@ -235,15 +273,13 @@ void centericq::mainloop() {
 		break;
 
 	    case ACT_ADD:
-		if(!c->inlist()) {
+		if(!c->inlist())
+		if(!ischannel(c) || !gethook(c->getdesc().pname).getCapabs().count(hookcapab::conferencesaretemporary))
 		    addcontact(c->getdesc());
-		}
 		break;
 
 	    case ACT_INFO:
-		if(c->getdesc() != contactroot)
-		if(!ischannel(c))
-		    userinfo(c->getdesc());
+		userinfo(c->getdesc());
 		break;
 
 	    case ACT_REMOVE:
@@ -258,6 +294,7 @@ void centericq::mainloop() {
 		text = face.inputstr(_("New nickname to show: "), c->getdispnick());
 		if(face.getlastinputkey() != KEY_ESC) {
 		    c->setdispnick(text);
+		    gethook(c->getdesc().pname).updatecontact(c);
 		    face.update();
 		}
 		break;
@@ -266,6 +303,7 @@ void centericq::mainloop() {
 		if(gid = face.selectgroup(_("Select a group to move the user to"))) {
 		    c->setgroupid(gid);
 		    face.fillcontactlist();
+		    face.update();
 		}
 		break;
 
@@ -285,21 +323,48 @@ void centericq::mainloop() {
 		}
 		break;
 
+	    case ACT_VERSION:
+		gethook(c->getdesc().pname).requestversion(c->getdesc());
+		break;
+
+	    case ACT_PING:
+		gethook(c->getdesc().pname).ping(c->getdesc());
+		break;
+
+	    case ACT_CONFER:
+		if(gethook(c->getdesc().pname).getCapabs().count(hookcapab::conferencesaretemporary))
+		    createconference(c);
+		break;
+
+	    case ACT_JOIN:
+	    case ACT_LEAVE:
+		{
+		    icqcontact::basicinfo cb = c->getbasicinfo();
+		    cb.requiresauth = c->getstatus() == offline;
+		    c->setstatus(cb.requiresauth ? available : offline);
+		    c->setbasicinfo(cb);
+		}
+		break;
+
+	    case ACT_EXTERN:
+		face.userinfoexternal(c->getdesc());
+		break;
+
 	    case ACT_MSG:
 	    case 0:
-		if(conf.getchatmode()) {
+		if(conf.getchatmode(c->getdesc().pname)) {
 		    face.chat(c->getdesc());
 
 		} else {
-		    if(c->getmsgcount()) {
+		    if(c->hasevents()) {
 			readevents(c->getdesc());
 		    } else {
-			if(c->getdesc() != contactroot)
-			if(c->getdesc().pname != infocard) {
+			if(c->getdesc().pname != infocard && c->getdesc().pname != rss) {
 			    sendevent(immessage(c->getdesc(), imevent::outgoing,
 				c->getpostponed()), icqface::ok);
 			}
 		    }
+
 		}
 		break;
 	}
@@ -308,34 +373,49 @@ void centericq::mainloop() {
 
 void centericq::changestatus() {
     imstatus st;
-    protocolname ipname, pname;
     bool proceed, setaway;
-    string tmp, prompt;
+    string tmp, prompt, awaymsg;
     icqconf::imaccount ia;
 
-    if(face.changestatus(pname, st)) {
+    vector<protocolname> pnames;
+    vector<protocolname>::const_iterator ipname;
+
+    if(face.changestatus(pnames, st)) {
 	if(!conf.enoughdiskspace()) {
-	    face.log("! cannot connect, free disk space is less than 10k");
+	    face.log(_("! cannot connect, free disk space is less than 10k"));
 
 	} else {
 	    proceed = true;
+	    setaway = false;
 
-	    setaway = conf.getaskaway();
-	    if(pname != proto_all) {
-		setaway = setaway &&
-		    (gethook(pname).getcapabilities() & hoptCanSetAwayMsg);
+	    bool same = true;
+
+	    for(ipname = pnames.begin(); ipname != pnames.end(); ++ipname) {
+		if(conf.getaskaway())
+		    setaway = setaway || gethook(*ipname).getCapabs().count(hookcapab::setaway);
+
+		if(!conf.getourid(*ipname).empty()) {
+		    tmp = conf.getawaymsg(*ipname);
+		    if(!tmp.empty()) {
+			if(awaymsg.empty()) awaymsg = tmp; else
+			    same = same && tmp == awaymsg;
+		    }
+		}
 	    }
 
+	    if(!same)
+		awaymsg = "";
+
 	    if(setaway)
 	    switch(st) {
 		case away:
 		case notavail:
 		case occupied:
 		case dontdisturb:
-		    if(pname != proto_all)
-			prompt = conf.getprotocolname(pname) + ": ";
+		    if(pnames.size() == 1)
+			prompt = conf.getprotocolname(pnames.front()) + ": ";
 		    prompt += _("away message");
-		    proceed = setaway = face.edit(tmp = conf.getawaymsg(pname), prompt);
+		    proceed = setaway = face.edit(tmp = conf.getawaymsg(pnames.front()), prompt);
 		    break;
 		default:
 		    setaway = false;
@@ -343,16 +423,20 @@ void centericq::changestatus() {
 	    }
 
 	    if(proceed)
-	    for(ipname = icq; ipname != protocolname_size; (int) ipname += 1) {
-		if(!(ia = conf.getourid(ipname)).empty())
-		if((pname == proto_all) || (ipname == pname)) {
-		    abstracthook &hook = gethook(ipname);
+	    for(ipname = pnames.begin(); ipname != pnames.end(); ++ipname) {
+		if(!gethook(*ipname).enabled()) {
+		    face.log(_("! support for %s was disabled at build time"),
+			conf.getprotocolname(*ipname).c_str());
+
+		} else {
+		    abstracthook &hook = gethook(*ipname);
 
 		    if(setaway && !tmp.empty())
-			if(hook.getcapabilities() & hoptCanSetAwayMsg)
-			    conf.setawaymsg(ipname, tmp);
+			if(hook.getCapabs().count(hookcapab::setaway))
+			    conf.setawaymsg(*ipname, tmp);
 
 		    hook.setstatus(st);
+
 		}
 	    }
 
@@ -361,45 +445,78 @@ void centericq::changestatus() {
     }
 }
 
+void centericq::joindialog() {
+    static imsearchparams s;
+    icqcontact *c;
+
+    if(face.finddialog(s, icqface::fschannel)) {
+	imcontact ic(s.nick, s.pname);
+	abstracthook &h = gethook(s.pname);
+
+	if(ic.nickname.substr(0, 1) != "#")
+	    ic.nickname.insert(0, "#");
+
+	if(h.getCapabs().count(hookcapab::groupchatservices)) {
+	    if(ic.nickname.find("@") == -1)
+		ic.nickname += "@" + s.service;
+/*
+	    c->setnick(ic.nickname);
+	    c->setdispnick(c->getnick());
+	    c->setdesc(imcontact(c->getnick(), ic.pname));
+*/
+	}
+
+	if(h.getCapabs().count(hookcapab::conferencesaretemporary)) {
+	    createconference(imcontact("", s.pname));
+
+	} else if(c = cicq.addcontact(ic)) {
+	    icqcontact::basicinfo cb = c->getbasicinfo();
+
+	    if(h.getCapabs().count(hookcapab::channelpasswords))
+		cb.zip = s.password;
+
+	    cb.requiresauth = true;
+	    c->setbasicinfo(cb);
+	    c->setstatus(available);
+
+	}
+    }
+}
+
+void centericq::linkfeed() {
+    icqcontact *c;
+    imsearchparams s;
+
+    s.pname = rss;
+    s.checkfrequency = 120;
+
+    if(face.finddialog(s, icqface::fsrss)) {
+	if(c = cicq.addcontact(imcontact(0, rss))) {
+	    icqcontact::workinfo wi = c->getworkinfo();
+	    wi.homepage = s.url; // syndication url
+	    c->setworkinfo(wi);
+
+	    icqcontact::moreinfo mi = c->getmoreinfo();
+	    mi.birth_day = s.checkfrequency;
+	    c->setmoreinfo(mi);
+
+	    c->setnick(s.nick);
+	    c->setdispnick(s.nick);
+	    c->save();
+	}
+    }
+}
+
 void centericq::find() {
     static imsearchparams s;
     bool ret = true;
     icqcontact *c;
 
     while(ret) {
-	if(ret = face.finddialog(s)) {
+	if(ret = face.finddialog(s, icqface::fsuser)) {
 	    switch(s.pname) {
 		case icq:
-		    if(s.uin) {
-			addcontact(imcontact(s.uin, s.pname));
-			ret = false;
-		    } else {
-			ret = face.findresults(s);
-		    }
-
-		    break;
-
-		case msn:
-		    if(s.nick.size() > 12)
-		    if(s.nick.substr(s.nick.size()-12) == "@hotmail.com") {
-			s.nick.erase(s.nick.size()-12);
-		    }
-
-		case irc:
-		    if(!s.nick.empty()) {
-			addcontact(imcontact(s.nick, s.pname));
-			ret = false;
-		    } else {
-			ret = face.findresults(s);
-		    }
-		    break;
-
-		case yahoo:
-		case aim:
-		    if(!s.nick.empty()) {
-			addcontact(imcontact(s.nick, s.pname));
-			ret = false;
-		    }
+		    ret = face.findresults(s);
 		    break;
 
 		case infocard:
@@ -420,24 +537,37 @@ void centericq::find() {
 		    face.update();
 		    ret = false;
 		    break;
+
+		default:
+		    ret = face.findresults(s);
+		    break;
 	    }
 	}
     }
 }
 
 void centericq::userinfo(const imcontact &cinfo) {
-    imcontact realuin = cinfo;
-    icqcontact *c = clist.get(cinfo);
+    if(ischannel(cinfo)) {
+	imsearchparams s;
+	s.pname = cinfo.pname;
+	s.room = cinfo.nickname;
+	face.findresults(s, true);
 
-    if(!c) {
-	c = clist.get(contactroot);
-	realuin = contactroot;
-	c->clear();
-	gethook(cinfo.pname).requestinfo(cinfo);
-    }
+    } else {
+	imcontact realuin = cinfo;
+	icqcontact *c = clist.get(cinfo);
+
+	if(!c) {
+	    c = clist.get(contactroot);
+	    realuin = contactroot;
+	    c->clear();
+	    gethook(cinfo.pname).requestinfo(cinfo);
+	}
+
+	if(c) {
+	    face.userinfo(cinfo, realuin);
+	}
 
-    if(c) {
-	face.userinfo(cinfo, realuin);
     }
 }
 
@@ -452,6 +582,7 @@ bool centericq::updateconf() {
 	    conf.setregsound(snd);
 	    unlink(conf.getconfigfname("sounds").c_str());
 	    conf.loadsounds();
+	    configstats.erase("sound");
 	}
 
 	if(clr != icqconf::rcdontchange) {
@@ -461,6 +592,7 @@ bool centericq::updateconf() {
 	    face.done();
 	    face.init();
 	    face.draw();
+	    configstats.erase("colorscheme");
 	}
 
 	face.update();
@@ -470,7 +602,6 @@ bool centericq::updateconf() {
 }
 
 void centericq::checkmail() {
-    static int fsize = -1;
     const char *fname = getenv("MAIL");
     struct stat st;
     int mcount = 0;
@@ -479,11 +610,20 @@ void centericq::checkmail() {
     bool prevempty, header;
     FILE *f;
 
+    static int fsize = -1;
+    static int oldcount = -1;
+
     if(conf.getmailcheck() && fname)
     if(!stat(fname, &st))
     if(st.st_size != fsize) {
 
-	if(fsize != -1) {
+	if(fsize != -1 && st.st_size > fsize) {
+	    /*
+	    *
+	    * I.e. only if mailbox has grown after the recent check.
+	    *
+	    */
+
 	    if(f = fopen(fname, "r")) {
 		prevempty = header = true;
 
@@ -503,11 +643,10 @@ void centericq::checkmail() {
 		fclose(f);
 	    }
 
-	    if(mcount) {
-		time_t tnow;
+	    if(mcount && (oldcount == -1 || (mcount > oldcount))) {
 		const struct tm *lt;
 
-		lt = localtime(&(tnow = time(0)));
+		lt = localtime(&timer_current);
 
 		face.log(_("+ [%02d:%02d:%02d] new mail arrived, %d messages"),
 		    lt->tm_hour, lt->tm_min, lt->tm_sec, mcount);
@@ -516,18 +655,64 @@ void centericq::checkmail() {
 
 		clist.get(contactroot)->playsound(imevent::email);
 	    }
+
+	    oldcount = mcount;
 	}
 
 	fsize = st.st_size;
     }
 }
 
+void centericq::checkconfigs() {
+    static const char *configs[] = {
+	"sounds", "colorscheme", "actions", "external", 0
+    };
+
+    struct stat st;
+    const char *p;
+
+    for(int i = 0; p = configs[i]; i++) {
+	if(stat(conf.getconfigfname(p).c_str(), &st))
+	    st.st_mtime = 0;
+
+	if(configstats.find(p) != configstats.end()) {
+	    if(st.st_mtime != configstats[p]) {
+		switch(i) {
+		    case 0:
+			conf.loadsounds();
+			break;
+		    case 1:
+			endwin();
+			initscr();
+			conf.loadcolors();
+			face.redraw();
+			break;
+		    case 2:
+			conf.loadactions();
+			break;
+		    case 3:
+			external.load();
+			break;
+		}
+
+		face.log(_("+ the %s configuration was changed, reloaded"), p);
+	    }
+	}
+
+	configstats[p] = st.st_mtime;
+    }
+}
+
 void centericq::handlesignal(int signum) {
-    int status;
+    int status, pid;
 
     switch(signum) {
 	case SIGCHLD:
-	    while(wait3(&status, WNOHANG, 0) > 0);
+	    while((pid = wait3(&status, WNOHANG, 0)) > 0) {
+		// In case the child was a nowait external action
+		string sname = conf.getdirname() + "centericq-external-tmp." + i2str(pid);
+		unlink(sname.c_str());
+	    }
 	    break;
 
 	case SIGUSR1:
@@ -537,6 +722,17 @@ void centericq::handlesignal(int signum) {
 	case SIGALRM:
 	case SIGUSR2:
 	    break;
+
+	case SIGTERM:
+	    lst.save();
+	    clist.save();
+	    groups.save();
+	    conf.save();
+	    face.done();
+
+	    signal(SIGTERM, 0);
+	    kill(0, SIGTERM);
+	    break;
     }
 }
 
@@ -650,10 +846,31 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 		sendev = new immessage(m->getcontact(), imevent::outgoing, "");
 		break;
 	}
+
+    } else if(ev.gettype() == imevent::authorization) {
+	const imauthorization *m = dynamic_cast<const imauthorization *>(&ev);
+
+	text = m->getmessage();
+
+	switch(r) {
+	    case icqface::ok:
+		sendev = new imauthorization(m->getcontact(), imevent::outgoing,
+		    m->getauthtype(), text);
+		break;
+
+	    case icqface::forward:
+		text = fwdnote + m->gettext();
+	    case icqface::reply:
+		/* reply to an URL is a message */
+		sendev = new immessage(m->getcontact(), imevent::outgoing, text);
+		break;
+	}
+
     } else if(ev.gettype() == imevent::sms) {
 	const imsms *m = dynamic_cast<const imsms *>(&ev);
+	string phone = m->getphone();
 
-	if(c = clist.get(ev.getcontact())) {
+	if(phone.empty() && (c = clist.get(ev.getcontact()))) {
 	    icqcontact::basicinfo b = c->getbasicinfo();
 
 	    if(b.cellular.find_first_of("0123456789") == -1) {
@@ -666,9 +883,11 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 		c->setbasicinfo(b);
 		c->save();
 	    }
+
+	    phone = b.cellular;
 	}
 
-	text = m->gettext();
+	text = m->getmessage();
 
 	if(r == icqface::reply) {
 	    if(conf.getquote()) {
@@ -680,7 +899,27 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 	    text = fwdnote + text;
 	}
 
-	sendev = new imsms(m->getcontact(), imevent::outgoing, text);
+	sendev = new imsms(m->getcontact(), imevent::outgoing, text, phone);
+
+    } else if(ev.gettype() == imevent::contacts) {
+	const imcontacts *m = dynamic_cast<const imcontacts *>(&ev);
+
+	switch(r) {
+	    case icqface::forward:
+	    case icqface::ok:
+		sendev = new imcontacts(m->getcontact(), imevent::outgoing,
+		    m->getcontacts());
+		break;
+
+	    case icqface::reply:
+		/* reply to contacts is a message */
+		sendev = new immessage(m->getcontact(), imevent::outgoing, "");
+		break;
+	}
+
+    } else if(ev.gettype() == imevent::file) {
+	sendev = new imfile(ev);
+
     }
 
     if(proceed = sendev) {
@@ -717,8 +956,32 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
     return proceed;
 }
 
-void centericq::readevent(const imevent &ev, bool &enough, bool &fin) {
-    icqface::eventviewresult r = face.eventview(&ev);
+icqface::eventviewresult centericq::readevent(const imevent &ev, const vector<icqface::eventviewresult> &buttons) {
+    bool b1, b2;
+    return readevent(ev, b1, b2, buttons);
+}
+
+icqface::eventviewresult centericq::readevent(const imevent &ev, bool &enough, bool &fin, const vector<icqface::eventviewresult> &buttons) {
+    icqface::eventviewresult r = face.eventview(&ev, buttons);
+
+    imcontact cont = ev.getcontact();
+    string nickname, tmp;
+
+    if(ev.gettype() == imevent::contacts) {
+	const imcontacts *m = static_cast<const imcontacts *>(&ev);
+
+	if(face.extk <= m->getcontacts().size() && face.extk > 0) {
+	    vector< pair<unsigned int, string> >::const_iterator icc =
+		m->getcontacts().begin()+face.extk-1;
+
+	    nickname = icc->second;
+	    cont = imcontact();
+	    cont.pname = ev.getcontact().pname;
+
+	    if(!(cont.uin = icc->first))
+		cont.nickname = nickname;
+	}
+    }
 
     switch(r) {
 	case icqface::forward:
@@ -744,21 +1007,59 @@ void centericq::readevent(const imevent &ev, bool &enough, bool &fin) {
 	    break;
 
 	case icqface::accept:
-	    em.store(imauthorization(ev.getcontact(),
-		imevent::outgoing, imauthorization::Granted, "accepted"));
-	    enough = true;
+	    switch(ev.gettype()) {
+		case imevent::authorization:
+		    em.store(imauthorization(ev.getcontact(),
+			imevent::outgoing, imauthorization::Granted, "accepted"));
+		    enough = true;
+		    break;
+
+		case imevent::file:
+		    tmp = face.inputdir(_("directory to save the file(s) to: "), getenv("HOME"));
+
+		    if(enough = (!tmp.empty() && !access(tmp.c_str(), X_OK))) {
+			if(enough = !access(tmp.c_str(), W_OK)) {
+			    const imfile *f = static_cast<const imfile *>(&ev);
+			    gethook(ev.getcontact().pname).replytransfer(*f, true, tmp);
+			} else {
+			    face.status(_("The specified directory is not writable"));
+			}
+
+		    } else {
+			face.status(_("The specified directory does not exist"));
+
+		    }
+		    break;
+	    }
+
 	    break;
 
 	case icqface::reject:
-	    em.store(imauthorization(ev.getcontact(),
-		imevent::outgoing, imauthorization::Rejected, "rejected"));
+	    switch(ev.gettype()) {
+		case imevent::authorization:
+		    em.store(imauthorization(ev.getcontact(),
+			imevent::outgoing, imauthorization::Rejected, "rejected"));
+		    break;
+
+		case imevent::file:
+		    gethook(ev.getcontact().pname).replytransfer(*static_cast<const imfile *>(&ev), false);
+		    break;
+	    }
+
 	    enough = true;
 	    break;
 
 	case icqface::info:
-	    cicq.userinfo(ev.getcontact());
+	    cicq.userinfo(cont);
+	    break;
+
+	case icqface::add:
+	    icqcontact *c = cicq.addcontact(cont);
+	    if(c) c->setdispnick(nickname);
 	    break;
     }
+
+    return r;
 }
 
 void centericq::readevents(const imcontact cont) {
@@ -769,8 +1070,9 @@ void centericq::readevents(const imcontact cont) {
 
     if(c) {
 	fin = false;
+	face.saveworkarea();
 
-	while(c->getmsgcount() && !fin) {
+	while(c->hasevents() && !fin) {
 	    events = em.getevents(cont, c->getlastread());
 	    fin = events.empty();
 
@@ -793,6 +1095,7 @@ void centericq::readevents(const imcontact cont) {
 	}
 
 	c->save();
+	face.restoreworkarea();
 	face.update();
     }
 }
@@ -803,48 +1106,29 @@ void centericq::history(const imcontact &cont) {
     bool enough;
     vector<imevent *> events;
     vector<imevent *>::iterator i;
+    vector<icqface::eventviewresult> buttons;
+
+    buttons.push_back(icqface::prev);
+    buttons.push_back(icqface::next);
 
     events = em.getevents(cont, 0);
 
     if(!events.empty()) {
+	face.saveworkarea();
 	face.histmake(events);
 
 	while(face.histexec(im)) {
 	    enough = false;
 
 	    while(!enough) {
-		vector<icqface::eventviewresult> buttons(1, icqface::next);
-
-		r = face.eventview(im, buttons);
+		r = readevent(*im, buttons);
 
 		switch(r) {
-		    case icqface::forward:
-		    case icqface::reply:
-			sendevent(*im, r);
-			break;
-
-		    case icqface::accept:
-			enough = true;
-			em.store(imauthorization(cont, imevent::outgoing,
-			    imauthorization::Granted, "accepted"));
-			break;
-
-		    case icqface::reject:
-			enough = true;
-			em.store(imauthorization(cont, imevent::outgoing,
-			    imauthorization::Rejected, "rejected"));
-			break;
-
 		    case icqface::cancel:
 		    case icqface::ok:
 			enough = true;
 			break;
 
-		    case icqface::open:
-			if(const imurl *m = static_cast<const imurl *>(im))
-			    conf.openurl(m->geturl());
-			break;
-
 		    case icqface::next:
 			i = ::find(events.begin(), events.end(), im);
 			enough = i == events.begin();
@@ -855,14 +1139,21 @@ void centericq::history(const imcontact &cont) {
 			}
 			break;
 
-		    case icqface::info:
-			cicq.userinfo(cont);
+		    case icqface::prev:
+			i = ::find(events.begin(), events.end(), im);
+			enough = i == events.end()-1;
+
+			if(!enough) {
+			    i++;
+			    im = *i;
+			}
 			break;
 		}
 	    }
 	}
 
-	face.status("");
+	face.restoreworkarea();
+	face.status("@");
 
 	while(!events.empty()) {
 	    delete events.back();
@@ -913,8 +1204,8 @@ icqcontact *centericq::addcontact(const imcontact &ic) {
     }
 
     c->setgroupid(groupid);
-    face.log(_("+ %s has been added to the list"), ic.totext().c_str());
-    face.update();
+    face.log(_("+ %s has been added to the list"), c->getdesc().totext().c_str());
+    face.relaxedupdate();
 
     return c;
 }
@@ -949,7 +1240,7 @@ bool centericq::idle(int options ) {
 	    }
 	}
 
-	tv.tv_sec = face.updaterequested() ? 1 : 30;
+	tv.tv_sec = face.updaterequested() ? 1 : 20;
 	tv.tv_usec = 0;
 
 	select(hsockfd+1, &rfds, &wfds, &efds, &tv);
@@ -1033,11 +1324,12 @@ void centericq::setauto(imstatus astatus) {
 #define MINCK0(x, y)       (x ? (y ? (x > y ? y : x) : x) : y)
 
 void centericq::exectimers() {
-    time_t timer_current = time(0);
     protocolname pname;
     int paway, pna;
     bool fonline = false;
 
+    time(&timer_current);
+
     /*
     *
     * Execute regular actions for all the hooks.
@@ -1045,7 +1337,7 @@ void centericq::exectimers() {
     */
 
     for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	if(!conf.getourid(pname).empty()) {
+	if(!conf.getourid(pname).empty() || (pname == rss)) {
 	    abstracthook &hook = gethook(pname);
 
 	    /*
@@ -1067,7 +1359,9 @@ void centericq::exectimers() {
 
 		if(!hook.logged()) {
 		    time(&reconnect[pname].timer);
-		    reconnect[pname].period += reconnect[pname].period/2;
+
+		    if(reconnect[pname].period < 180)
+			reconnect[pname].period += reconnect[pname].period/2;
 
 		    if(hook.online()) {
 			hook.disconnect();
@@ -1133,6 +1427,7 @@ void centericq::exectimers() {
 	*
 	*/
 
+	checkconfigs();
 	conf.checkdiskspace();
 
 	if(!conf.enoughdiskspace()) {
@@ -1154,126 +1449,77 @@ void centericq::exectimers() {
 	    face.relaxedupdate();
 	}
 
-	time(&timer_resend);
+	timer_resend = timer_current;
     }
 
     if(timer_current-timer_checkmail > PERIOD_CHECKMAIL) {
-	cicq.checkmail();
-	time(&timer_checkmail);
+	checkmail();
+	timer_checkmail = timer_current;
+    }
+
+    if(timer_current-timer_autosave > PERIOD_AUTOSAVE) {
+	clist.save(false);
+	timer_autosave = timer_current;
     }
 
     if(face.updaterequested())
     if(timer_current-timer_update > PERIOD_DISPUPDATE) {
 	face.update();
-	time(&timer_update);
+	timer_update = timer_current;
+    }
+}
+
+void centericq::createconference(const imcontact &ic) {
+    int apid = 0;
+    imcontact confid("", ic.pname);
+
+    set<protocolname> ps;
+    ps.insert(ic.pname);
+
+    face.muins.clear();
+    if(!ic.empty())
+	face.muins.push_back(ic);
+
+    if(face.multicontacts(_("Invite to conference.."), ps))
+    if(!face.muins.empty()) {
+	do {
+	    confid.nickname = (string) "#" + conf.getourid(ic.pname).nickname + "-" + i2str(getpid() + ++apid);
+	} while(clist.get(confid));
+
+	icqcontact *nc = clist.addnew(confid);
+	nc->setstatus(available);
+
+	abstracthook &hook = gethook(ic.pname);
+	hook.conferencecreate(nc, face.muins);
     }
 }
 
 // ----------------------------------------------------------------------------
 
-string rushtmlconv(const string &tdir, const string &text) {
-    int pos;
-    string r = rusconv(tdir, text);
-
-    if(tdir == "kw") {
-	pos = 0;
-	while((pos = r.find_first_of("&<>", pos)) != -1) {
-	    switch(r[pos]) {
-		case '&':
-		    if(r.substr(pos, 4) != "&lt;"
-		    && r.substr(pos, 4) != "&gt;")
-			r.replace(pos, 1, "&amp;");
-		    break;
-		case '<': r.replace(pos, 1, "&lt;"); break;
-		case '>': r.replace(pos, 1, "&gt;"); break;
-	    }
-	    pos++;
-	}
-
-    } else if(tdir == "wk") {
-	pos = 0;
-	while((pos = r.find("&", pos)) != -1) {
-	    if(r.substr(pos+1, 4) == "amp;") r.replace(pos, 5, "&"); else
-	    if(r.substr(pos+1, 3) == "lt;") r.replace(pos, 4, "<"); else
-	    if(r.substr(pos+1, 3) == "gt;") r.replace(pos, 4, ">"); 
-	    pos++;
-	}
-    }
-
-    return r;
-}
-
-string ruscrlfconv(const string &tdir, const string &text) {
-    string r = rusconv(tdir, text);
-    int pos;
-
-    for(pos = 0; (pos = r.find("\r")) != -1; pos++) {
-	r.erase(pos, 1);
-    }
-
-    return r;
-}
-
-string rusconv(const string &tdir, const string &text) {
-    string r;
-
-#ifndef HAVE_ICONV_H
-    static unsigned char kw[] = {
-	128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
-	144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
-	160,161,162,184,164,165,166,167,168,169,170,171,172,173,174,175,
-	176,177,178,168,180,181,182,183,184,185,186,187,188,189,190,191,
-	254,224,225,246,228,229,244,227,245,232,233,234,235,236,237,238,
-	239,255,240,241,242,243,230,226,252,251,231,248,253,249,247,250,
-	222,192,193,214,196,197,212,195,213,200,201,202,203,204,205,206,
-	207,223,208,209,210,211,198,194,220,219,199,216,221,217,215,218
-    };
-
-    static unsigned char wk[] = {
-	128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,
-	144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,
-	160,161,162,163,164,165,166,167,179,169,170,171,172,173,174,175,
-	176,177,178,179,180,181,182,183,163,185,186,187,188,189,190,191,
-	225,226,247,231,228,229,246,250,233,234,235,236,237,238,239,240,
-	242,243,244,245,230,232,227,254,251,253,255,249,248,252,224,241,
-	193,194,215,199,196,197,214,218,201,202,203,204,205,206,207,208,
-	210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209
-    };
-
-    unsigned char c;
-    string::const_iterator i;
-    unsigned char *table = 0;
-
-    if(tdir == "kw") table = kw; else
-    if(tdir == "wk") table = wk;
-
-    if(conf.getrussian() && table) {
-	for(i = text.begin(); i != text.end(); ++i) {
-	    c = (unsigned char) *i;
-	    c &= 0377;
-	    if(c & 0200) c = table[c & 0177];
-	    r += c;
-	}
-    } else {
-	r = text;
-    }
-#else
-
-    if(conf.getrussian()) {
-	if(tdir == "kw") r = siconv(text, "koi8-r", "cp1251"); else
-	if(tdir == "wk") r = siconv(text, "cp1251", "koi8-r"); else
-	    r = text;
-    } else {
-	r = text;
-    }
-
-#endif
-
-    return r;
-}
-
 bool ischannel(const imcontact &cont) {
-    return
-	(cont.nickname.substr(0, 1) == "#") &&
-	(cont.pname == irc || cont.pname == yahoo);
+    if(cont.nickname.substr(0, 1) == "#")
+    if(gethook(cont.pname).getCapabs().count(hookcapab::conferencing))
+	return true;
+
+    return false;
+}
+
+string up(string s) {
+    string::iterator is = s.begin();
+    while(is != s.end()) {
+	*is = toupper(*is);
+	++is;
+    }
+
+    return s;
+}
+
+string lo(string s) {
+    string::iterator is = s.begin();
+    while(is != s.end()) {
+	*is = tolower(*is);
+	++is;
+    }
+
+    return s;
 }
