@@ -6,7 +6,7 @@
 #include <fstream>
 
 #include "kkstrtext.h"
-#include "kkfsys.h"
+#include "icqconf.h"
 #include "icqcontact.h"
 
 #include "eventmanager.h"
@@ -27,12 +27,14 @@ class centericq {
 	    time_t timer;
 	    int period;
 
-	    reconnectInfo(): timer(0), period(40) {}
+	    reconnectInfo(): timer(0), period(30) {}
 	};
 
-	time_t timer_checkmail, timer_keypress, timer_update, timer_resend;
+	time_t timer_checkmail, timer_keypress, timer_update, timer_resend, timer_autosave;
 	bool regmode;
 
+	map<string, time_t> configstats;
+
 	static void handlesignal(int signum);
 
 	void checkparallel();
@@ -40,9 +42,15 @@ class centericq {
 	bool checkpasswords();
 	void rereadstatus();
 
+	void checkmail();
+	void checkconfigs();
+
 	string quotemsg(const string &text);
 
 	void setauto(imstatus astatus);
+	void createconference(const imcontact &ic);
+
+	void defaultcontacts(bool rus);
 
     public:
 	centericq();
@@ -57,14 +65,20 @@ class centericq {
 	bool updateconf();
 
 	void find();
-	void checkmail();
+	void joindialog();
+	void linkfeed();
 
 	icqcontact *addcontact(const imcontact &ic);
 
 	bool idle(int options = 0);
 	void exectimers();
 
-	void readevent(const imevent &ev, bool &enough, bool &fin);
+	icqface::eventviewresult readevent(const imevent &ev,
+	    const vector<icqface::eventviewresult> &buttons = vector<icqface::eventviewresult>());
+
+	icqface::eventviewresult readevent(const imevent &ev, bool &enough, bool &fin,
+	    const vector<icqface::eventviewresult> &buttons = vector<icqface::eventviewresult>());
+
 	void readevents(const imcontact cont);
 
 	void history(const imcontact &cont);
@@ -74,10 +88,4 @@ class centericq {
 
 extern centericq cicq;
 
-string rusconv(const string &tdir, const string &text);
-string rushtmlconv(const string &tdir, const string &text);
-string ruscrlfconv(const string &tdir, const string &text);
-
-bool ischannel(const imcontact &cont);
-
 #endif
