@@ -1,3 +1,27 @@
+/*
+*
+* centericq event manager class
+* $Id: eventmanager.cc,v 1.22 2003/07/18 00:39:59 konst Exp $
+*
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or (at
+* your option) any later version.
+*
+* This program is distributed in the hope that it will be useful, but
+* WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+* General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+* USA
+*
+*/
+
 #include "eventmanager.h"
 #include "icqmlist.h"
 #include "icqconf.h"
@@ -15,10 +39,23 @@ imeventmanager::imeventmanager(): unsent(0), lastevent(0), recentlysent(0) {
 imeventmanager::~imeventmanager() {
 }
 
-void imeventmanager::store(const imevent &ev) {
+void imeventmanager::store(const imevent &cev) {
     icqcontact *c;
     bool proceed;
 
+    auto_ptr<imevent> imev(cev.getevent());
+    imevent &ev = *imev.get();
+
+    static int preoptions[imevent::imdirection_size] = {
+	imexternal::aoprereceive,
+	imexternal::aopresend
+    };
+
+    external.exec(&ev, proceed, preoptions[ev.getdirection()]);
+
+    if(!proceed)
+	return;
+
     if(ev.getdirection() == imevent::incoming) {
 	if(!lst.inlist(ev.getcontact(), csignore)) {
 	    logger.putevent(ev);
@@ -36,7 +73,7 @@ void imeventmanager::store(const imevent &ev) {
 	    if(c) {
 		eventwrite(ev, history);
 
-		c->setmsgcount(c->getmsgcount()+1);
+		c->sethasevents(true);
 		c->playsound(ev.gettype());
 
 		external.exec(ev);
@@ -45,33 +82,40 @@ void imeventmanager::store(const imevent &ev) {
 	    }
 	}
     } else if(ev.getdirection() == imevent::outgoing) {
-	abstracthook &hook = gethook(ev.getcontact().pname);
+	abstracthook *hook;
+
+	// All the SMSes are to be sent through the icqhook class..
+
+	if(ev.gettype() == imevent::sms) {
+	    hook = &gethook(icq);
+	} else {
+	    hook = &gethook(ev.getcontact().pname);
+	}
+
+	// Some self-flood protection..
 
 	if(time(0)-lastevent > PERIOD_ATONCE)
 	    recentlysent = 0;
 
-	proceed = hook.logged() && recentlysent < MAX_ATONCE;
+	proceed = hook->logged() && recentlysent < MAX_ATONCE;
+
+	// 3.. 2.. 1.. LAUNCH!
 
 	if(proceed) {
-	    if(hook.send(ev)) {
-		eventwrite(ev, history);
-		logger.putevent(ev);
+	    if(hook->send(ev)) {
+		eventwrite(cev, history);
+		logger.putevent(cev);
 		time(&lastevent);
 		recentlysent++;
 	    } else {
-		eventwrite(ev, offline);
+		eventwrite(cev, offline);
 	    }
 	} else {
-	    eventwrite(ev, offline);
+	    eventwrite(cev, offline);
 	}
     }
 }
 
-// if incoming, store to the history immediately
-// if outgoing, return id
-
-// void accept(const imcontact cont);
-
 vector<imevent *> imeventmanager::getevents(const imcontact &cont, time_t lastread) const {
     ifstream fhist;
     imevent *rev;
