@@ -1,7 +1,7 @@
 /*
 *
 * centericq AIM protocol handling class
-* $Id: aimhook.cc,v 1.19 2002/07/13 11:18:57 konst Exp $
+* $Id: aimhook.cc,v 1.35 2003/07/07 18:51:00 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -23,11 +23,14 @@
 */
 
 #include "aimhook.h"
+
+#ifdef BUILD_AIM
+
 #include "icqface.h"
 #include "accountmanager.h"
 #include "icqcontacts.h"
-#include "centericq.h"
 #include "imlogger.h"
+#include "eventmanager.h"
 
 #ifdef DEBUG
 #define DLOG(s) face.log("aim %s", s)
@@ -38,14 +41,15 @@
 aimhook ahook;
 
 aimhook::aimhook()
-    : handle(firetalk_create_handle(FP_AIMTOC, 0)),
-      fonline(false), flogged(false), ourstatus(offline)
+    : abstracthook(aim), handle(0), fonline(false), flogged(false),
+      ourstatus(offline)
 {
-    fcapabilities =
-	hoptCanSetAwayMsg |
-	hoptCanChangePassword |
-	hoptCanUpdateDetails |
-	hoptChangableServer;
+    fcapabs.insert(hookcapab::setaway);
+    fcapabs.insert(hookcapab::changepassword);
+    fcapabs.insert(hookcapab::changedetails);
+    fcapabs.insert(hookcapab::synclist);
+    fcapabs.insert(hookcapab::directadd);
+    fcapabs.insert(hookcapab::changeabout);
 }
 
 aimhook::~aimhook() {
@@ -53,6 +57,19 @@ aimhook::~aimhook() {
 
 void aimhook::init() {
     manualstatus = conf.getstatus(aim);
+}
+
+void aimhook::connect() {
+    icqconf::imaccount acc = conf.getourid(aim);
+    face.log(_("+ [aim] connecting to the server"));
+
+    if(handle) {
+	firetalk_register_callback(handle, FC_DISCONNECT, 0);
+	firetalk_disconnect(handle);
+	firetalk_destroy_handle(handle);
+    }
+
+    handle = firetalk_create_handle(FP_AIMTOC, 0);
 
     firetalk_register_callback(handle, FC_CONNECTED, &connected);
     firetalk_register_callback(handle, FC_CONNECTFAILED, &connectfailed);
@@ -66,15 +83,7 @@ void aimhook::init() {
     firetalk_register_callback(handle, FC_IM_BUDDYAWAY, &buddyaway);
     firetalk_register_callback(handle, FC_IM_BUDDYUNAWAY, &buddyonline);
     firetalk_register_callback(handle, FC_NEEDPASS, &needpass);
-}
-
-void aimhook::connect() {
-    icqconf::imaccount acc = conf.getourid(aim);
-    face.log(_("+ [aim] connecting to the server"));
-
-    firetalk_register_callback(handle, FC_DISCONNECT, 0);
-    firetalk_disconnect(handle);
-    firetalk_register_callback(handle, FC_DISCONNECT, &disconnected);
+    firetalk_register_callback(handle, FC_IM_LISTBUDDY, &listbuddy);
 
     fonline = firetalk_signon(handle, acc.server.c_str(), acc.port, acc.nickname.c_str()) == FE_SUCCESS;
 
@@ -184,16 +193,24 @@ void aimhook::sendnewuser(const imcontact &ic) {
     if(online()) {
 	face.log(_("+ [aim] adding %s to the contacts"), ic.nickname.c_str());
 	firetalk_im_add_buddy(handle, ic.nickname.c_str());
-	firetalk_im_upload_buddies(handle);
+	firetalk_save_config(handle);
 	requestinfo(ic);
     }
 }
 
 void aimhook::removeuser(const imcontact &ic) {
+    removeuser(ic, true);
+}
+
+void aimhook::removeuser(const imcontact &ic, bool report) {
     if(online()) {
-	face.log(_("+ [aim] removing %s from the contacts"), ic.nickname.c_str());
+	if(report) {
+	    face.log(_("+ [aim] removing %s from the contacts"),
+		ic.nickname.c_str());
+	}
+
 	firetalk_im_remove_buddy(handle, ic.nickname.c_str());
-	firetalk_im_upload_buddies(handle);
+	firetalk_save_config(handle);
     }
 }
 
@@ -232,8 +249,13 @@ imstatus aimhook::getstatus() const {
 }
 
 void aimhook::requestinfo(const imcontact &c) {
-    if(c != imcontact(conf.getourid(aim).uin, aim)) {
-	firetalk_im_get_info(handle, c.nickname.c_str());
+    if(c != imcontact(conf.getourid(aim).nickname, aim)) {
+	icqcontact *cc = clist.get(c);
+
+	if(cc)
+	if(cc->getstatus() != offline)
+	    firetalk_im_get_info(handle, c.nickname.c_str());
+
     } else {
 	/*
 	*
@@ -248,16 +270,17 @@ void aimhook::requestinfo(const imcontact &c) {
 
 	cc->setabout(profile.info);
 	cc->setnick(acc.nickname);
+
     }
 }
 
-void aimhook::sendupdateuserinfo(icqcontact &c, const string &newpass) {
+void aimhook::sendupdateuserinfo(icqcontact &c) {
     icqconf::imaccount acc = conf.getourid(aim);
 
-    if(!newpass.empty())
-    if(newpass != acc.password) {
-	firetalk_set_password(handle, acc.password.c_str(), newpass.c_str());
-    }
+    if(!c.getreginfo().password.empty())
+    if(c.getreginfo().password != acc.password)
+	firetalk_set_password(handle, acc.password.c_str(),
+	    c.getreginfo().password.c_str());
 
     profile.info = c.getabout();
     firetalk_set_info(handle, profile.info.c_str());
@@ -310,6 +333,24 @@ void aimhook::resolve() {
     }
 }
 
+vector<icqcontact *> aimhook::getneedsync() {
+    vector<icqcontact *> r;
+
+    buddies.clear();
+    firetalk_im_list_buddies(handle);
+
+    for(int i = 0; i < clist.count; i++) {
+	icqcontact *c = (icqcontact *) clist.at(i);
+
+	if(c->getdesc().pname == aim) {
+	    if(find(buddies.begin(), buddies.end(), c->getdesc().nickname) == buddies.end())
+		r.push_back(c);
+	}
+    }
+
+    return r;
+}
+
 // ----------------------------------------------------------------------------
 
 void aimhook::connected(void *connection, void *cli, ...) {
@@ -317,19 +358,6 @@ void aimhook::connected(void *connection, void *cli, ...) {
     face.log(_("+ [aim] logged in"));
     face.update();
 
-    int i;
-    icqcontact *c;
-
-    for(i = 0; i < clist.count; i++) {
-	c = (icqcontact *) clist.at(i);
-
-	if(c->getdesc().pname == aim) {
-	    firetalk_im_add_buddy(ahook.handle, c->getdesc().nickname.c_str());
-	}
-    }
-
-    firetalk_im_upload_buddies(ahook.handle);
-
     ahook.ourstatus = available;
     ahook.setautostatus(ahook.manualstatus);
 
@@ -344,7 +372,7 @@ void aimhook::disconnected(void *connection, void *cli, ...) {
     logger.putourstatus(aim, ahook.getstatus(), offline);
     clist.setoffline(aim);
 
-    face.log(_("+ [aim] disconnected from the network"));
+    face.log(_("+ [aim] disconnected"));
     face.update();
 }
 
@@ -423,7 +451,7 @@ void aimhook::getmessage(void *conn, void *cli, ...) {
     if(sender && message)
     if(strlen(sender) && strlen(message)) {
 	em.store(immessage(imcontact(sender, aim),
-	    imevent::incoming, rushtmlconv("wk", cuthtml(message, true))));
+	    imevent::incoming, ahook.rushtmlconv("wk", cuthtml(message, true))));
     }
 
     DLOG("getmessage");
@@ -441,10 +469,6 @@ void aimhook::userstatus(const string &nickname, imstatus st) {
 	if(st != c->getstatus()) {
 	    logger.putonline(ic, c->getstatus(), st);
 	    c->setstatus(st);
-
-	    if(c->getstatus() != offline) {
-		c->setlastseen();
-	    }
 	}
     }
 }
@@ -504,3 +528,20 @@ void aimhook::needpass(void *conn, void *cli, ...) {
 	face.log(_("+ [aim] password sent"));
     }
 }
+
+void aimhook::listbuddy(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *nickname = va_arg(ap, char *);
+    char online = va_arg(ap, int);
+    char away = va_arg(ap, int);
+    long idletime = va_arg(ap, long);
+    va_end(ap);
+
+    if(nickname) {
+	ahook.buddies.push_back(nickname);
+    }
+}
+
+#endif
