@@ -1,9 +1,9 @@
 /*
 *
 * centericq icq protocol handling class
-* $Id: icqhook.cc,v 1.99 2002/08/17 14:08:40 konst Exp $
+* $Id: icqhook.cc,v 1.137 2003/09/30 11:38:43 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,16 +25,16 @@
 #include "icqhook.h"
 #include "icqconf.h"
 #include "icqface.h"
-#include "accountmanager.h"
-#include "centericq.h"
 #include "imlogger.h"
 
+#include "accountmanager.h"
+#include "eventmanager.h"
+
 #include <libicq2000/userinfohelpers.h>
 
 #define PERIOD_ICQPOLL  5
-#define PERIOD_RESOLVE  25
-
-#define MAX_REQUESTS    20
+#define PERIOD_RESOLVE  10
+#define DELAY_SENDNEW   5
 
 icqhook ihook;
 
@@ -49,70 +49,75 @@ static const Status stat2int[imstatus_size] = {
     STATUS_AWAY
 };
 
-icqhook::icqhook() {
+icqhook::icqhook(): abstracthook(icq) {
     fonline = false;
     blockmode = Normal;
 
-    fcapabilities =
-	hoptCanSendURL |
-	hoptCanSendSMS |
-	hoptCanSetAwayMsg |
-	hoptCanFetchAwayMsg |
-	hoptCanChangeNick |
-	hoptCanUpdateDetails |
-	hoptChangableServer |
-	hoptCanSyncList |
-	hoptControlableVisibility;
+    fcapabs.insert(hookcapab::urls);
+    fcapabs.insert(hookcapab::setaway);
+    fcapabs.insert(hookcapab::fetchaway);
+    fcapabs.insert(hookcapab::changenick);
+    fcapabs.insert(hookcapab::changepassword);
+    fcapabs.insert(hookcapab::changedetails);
+    fcapabs.insert(hookcapab::synclist);
+    fcapabs.insert(hookcapab::authrequests);
+    fcapabs.insert(hookcapab::authreqwithmessages);
+    fcapabs.insert(hookcapab::contacts);
+    fcapabs.insert(hookcapab::visibility);
+    fcapabs.insert(hookcapab::cltemporary);
+    fcapabs.insert(hookcapab::changeabout);
 
     cli.setServerSideGroup("centericq", 0x0666);
 
-    cli.connected.connect(slot(this, &icqhook::connected_cb));
-    cli.disconnected.connect(slot(this, &icqhook::disconnected_cb));
-    cli.socket.connect(slot(this, &icqhook::socket_cb));
-    cli.messaged.connect(slot(this, &icqhook::messaged_cb));
-    cli.messageack.connect(slot(this, &icqhook::messageack_cb));
-    cli.contactlist.connect(slot(this, &icqhook::contactlist_cb));
-    cli.contact_userinfo_change_signal.connect(slot(this, &icqhook::contact_userinfo_change_signal_cb));
-    cli.contact_status_change_signal.connect(slot(this, &icqhook::contact_status_change_signal_cb));
-    cli.newuin.connect(slot(this, &icqhook::newuin_cb));
-    cli.rate.connect(slot(this, &icqhook::rate_cb));
-    cli.want_auto_resp.connect(slot(this, &icqhook::want_auto_resp_cb));
-    cli.search_result.connect(slot(this, &icqhook::search_result_cb));
-    cli.server_based_contact_list.connect(slot(this, &icqhook::server_based_contact_list_cb));
-    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
-    cli.self_contact_status_change_signal.connect(slot(this, &icqhook::self_contact_status_change_cb));
+    cli.connected.connect(this, &icqhook::connected_cb);
+    cli.disconnected.connect(this, &icqhook::disconnected_cb);
+    cli.socket.connect(this, &icqhook::socket_cb);
+    cli.messaged.connect(this, &icqhook::messaged_cb);
+    cli.messageack.connect(this, &icqhook::messageack_cb);
+    cli.contactlist.connect(this, &icqhook::contactlist_cb);
+    cli.contact_userinfo_change_signal.connect(this, &icqhook::contact_userinfo_change_signal_cb);
+    cli.contact_status_change_signal.connect(this, &icqhook::contact_status_change_signal_cb);
+    cli.newuin.connect(this, &icqhook::newuin_cb);
+    cli.rate.connect(this, &icqhook::rate_cb);
+    cli.password_changed.connect(this, &icqhook::password_changed_cb);
+    cli.want_auto_resp.connect(this, &icqhook::want_auto_resp_cb);
+    cli.search_result.connect(this, &icqhook::search_result_cb);
+    cli.server_based_contact_list.connect(this, &icqhook::server_based_contact_list_cb);
+    cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
+    cli.self_contact_status_change_signal.connect(this, &icqhook::self_contact_status_change_cb);
 
 #ifdef DEBUG
-    cli.logger.connect(slot(this, &icqhook::logger_cb));
+    cli.logger.connect(this, &icqhook::logger_cb);
 #endif
 }
 
 icqhook::~icqhook() {
-    cli.logger.clear();
-    cli.connected.clear();
-    cli.disconnected.clear();
-    cli.socket.clear();
-    cli.messaged.clear();
-    cli.messageack.clear();
-    cli.contactlist.clear();
-    cli.contact_userinfo_change_signal.clear();
-    cli.contact_status_change_signal.clear();
-    cli.newuin.clear();
-    cli.rate.clear();
-    cli.search_result.clear();
-    cli.want_auto_resp.clear();
-    cli.self_contact_userinfo_change_signal.clear();
-    cli.self_contact_status_change_signal.clear();
-    cli.server_based_contact_list.clear();
+    cli.logger.disconnect(this);
+    cli.connected.disconnect(this);
+    cli.disconnected.disconnect(this);
+    cli.socket.disconnect(this);
+    cli.messaged.disconnect(this);
+    cli.messageack.disconnect(this);
+    cli.contactlist.disconnect(this);
+    cli.contact_userinfo_change_signal.disconnect(this);
+    cli.contact_status_change_signal.disconnect(this);
+    cli.newuin.disconnect(this);
+    cli.rate.disconnect(this);
+    cli.search_result.disconnect(this);
+    cli.want_auto_resp.disconnect(this);
+    cli.self_contact_userinfo_change_signal.disconnect(this);
+    cli.self_contact_status_change_signal.disconnect(this);
+    cli.server_based_contact_list.disconnect(this);
+    cli.password_changed.disconnect(this);
 }
 
 void icqhook::init() {
-    manualstatus = conf.getstatus(icq);
+    manualstatus = conf.getstatus(proto);
 }
 
 void icqhook::connect() {
-    icqconf::imaccount acc = conf.getourid(icq);
-    int i;
+    icqconf::imaccount acc = conf.getourid(proto);
+    int i, ptpmin, ptpmax;
     icqcontact *c;
 
     if(acc.additional["autosync"].empty()) acc.additional["autosync"] = "1";
@@ -122,8 +127,9 @@ void icqhook::connect() {
     for(i = 0; i < clist.count; i++) {
 	c = (icqcontact *) clist.at(i);
 
-	if(c->getdesc().pname == icq && c->getdesc().uin) {
-	    cli.addContact(new Contact(c->getdesc().uin));
+	if(c->getdesc().pname == proto && c->getdesc().uin) {
+	    ContactRef ct(new Contact(c->getdesc().uin));
+	    cli.addContact(ct);
 	}
     }
 
@@ -133,9 +139,21 @@ void icqhook::connect() {
     if(!conf.getsmtphost().empty()) cli.setSMTPServerHost(conf.getsmtphost());
     if(conf.getsmtpport()) cli.setSMTPServerPort(conf.getsmtpport());
 
+    if(!conf.getbindhost().empty()) cli.setClientBindHost(conf.getbindhost());
+
+    conf.getpeertopeer(ptpmin, ptpmax);
+    if(ptpmax) {
+	cli.setAcceptInDC(true);
+	cli.setPortRangeLowerBound(ptpmin);
+	cli.setPortRangeUpperBound(ptpmax);
+	cli.setUsePortRange(true);
+    } else {
+	cli.setAcceptInDC(false);
+    }
+
     face.log(_("+ [icq] connecting to the server"));
 
-    cli.self_contact_userinfo_change_signal.clear();
+    cli.self_contact_userinfo_change_signal.disconnect(this);
 
     cli.setUIN(acc.uin);
     cli.setPassword(acc.password);
@@ -144,7 +162,7 @@ void icqhook::connect() {
     sendinvisible();
     cli.setStatus(stat2int[manualstatus], manualstatus == invisible);
 
-    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
+    cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
 
     fonline = true;
     flogged = false;
@@ -152,20 +170,24 @@ void icqhook::connect() {
 
 void icqhook::disconnect() {
     cli.setStatus(STATUS_OFFLINE);
+    fonline = flogged = false;
 }
 
 void icqhook::resolve() {
-    int i, cnt;
     icqcontact *c;
+    vector<icqcontact *> toresolve;
 
-    for(i = cnt = 0; i < clist.count && cnt < MAX_REQUESTS; i++) {
+    for(int i = 0; i < clist.count; i++) {
 	c = (icqcontact *) clist.at(i);
 
-	if(c->getdesc().pname == icq)
-	if(c->getdispnick() == i2str(c->getdesc().uin)) {
-	    requestinfo(c);
-	    cnt++;
-	}
+	if(c->getdesc().pname == proto)
+	if(c->getdispnick() == i2str(c->getdesc().uin))
+	    toresolve.push_back(c);
+    }
+
+    if(toresolve.size()) {
+	c = toresolve[randlimit(0, toresolve.size()-1)];
+	requestinfo(c);
     }
 }
 
@@ -175,7 +197,7 @@ void icqhook::sendinvisible() {
     icqlist::iterator i;
 
     for(i = lst.begin(); i != lst.end(); ++i) {
-	if(i->getdesc().pname == icq) {
+	if(i->getdesc().pname == proto) {
 	    switch(i->getstatus()) {
 		case csvisible:
 		case csinvisible:
@@ -212,18 +234,32 @@ void icqhook::sendinvisible() {
 }
 
 void icqhook::exectimers() {
-    time_t tcurrent = time(0);
-
     if(logged()) {
-	if(tcurrent-timer_poll > PERIOD_ICQPOLL) {
+	if(timer_current-timer_poll > PERIOD_ICQPOLL) {
 	    cli.Poll();
 	    sendinvisible();
-	    time(&timer_poll);
+	    timer_poll = timer_current;
+
+	    /*
+	    *
+	    * Will to the mass-adding stuff here too..
+	    *
+	    */
+
+	    vector<imcontact>::iterator ic = uinstosend.begin();
+
+	    if(ic != uinstosend.end()) {
+		imcontact imc = *ic;
+		uinstosend.erase(ic);
+
+		ContactRef icont = cli.getContact(imc.uin);
+		if(!icont.get()) sendnewuser(imc);
+	    }
 	}
 
-	if(tcurrent-timer_resolve > PERIOD_RESOLVE) {
+	if(timer_current-timer_resolve > PERIOD_RESOLVE) {
 	    resolve();
-	    time(&timer_resolve);
+	    timer_resolve = timer_current;
 	}
     }
 }
@@ -340,13 +376,37 @@ bool icqhook::send(const imevent &ev) {
     ICQMessageEvent *iev;
     icqcontact *c;
 
-    if(!ic.get()) {
+    if(ev.getcontact().pname != proto)
+	uin = 0;
+
+    if(!uin)
+	ic = cli.getSelfContact();
+
+    if(!ic.get())
 	return false;
-    }
 
     if(ev.gettype() == imevent::message) {
 	const immessage *m = static_cast<const immessage *> (&ev);
-	sev = new NormalMessageEvent(ic, ruscrlfconv("kw", m->gettext()));
+	string text = m->gettext(), sub;
+
+	if(ic->getStatus() == STATUS_OFFLINE)
+	if(text.size() > 450) {
+	    while(!text.empty()) {
+		if(text.size() > 450) {
+		    sub = text.substr(0, 450);
+		    text.erase(0, 450);
+		} else {
+		    sub = text;
+		    text = "";
+		}
+
+		cli.SendEvent(new NormalMessageEvent(ic, ruscrlfconv("kw", sub)));
+	    }
+
+	    return true;
+	}
+
+	sev = new NormalMessageEvent(ic, ruscrlfconv("kw", text));
 
     } else if(ev.gettype() == imevent::url) {
 	const imurl *m = static_cast<const imurl *> (&ev);
@@ -358,17 +418,24 @@ bool icqhook::send(const imevent &ev) {
 	const imsms *m = static_cast<const imsms *> (&ev);
 
 	if(!uin) {
-	    ic = cli.getSelfContact();
-	    if(ic->getMainHomeInfo().getMobileNo().empty()) {
-		cli.fetchSelfDetailContactInfo();
-		return false;
+	    if(m->getphone().empty()) {
+		if(ic->getMainHomeInfo().getMobileNo().empty()) {
+		    cli.fetchSelfDetailContactInfo();
+		    return false;
+		}
+	    } else {
+		ic = new Contact(m->getphone());
+		ic->setMobileNo(m->getphone());
+		cli.addContact(ic);
 	    }
 
 	} else {
+	    cli.contact_userinfo_change_signal.disconnect(this);
 	    ic->setMobileNo(clist.get(ev.getcontact())->getbasicinfo().cellular);
+	    cli.contact_userinfo_change_signal.connect(this, &icqhook::contact_userinfo_change_signal_cb);
 	}
 
-	sev = new SMSMessageEvent(ic, ruscrlfconv("kw", m->gettext()), true);
+	sev = new SMSMessageEvent(ic, siconv(m->getmessage(), conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset(), "utf8"), true);
 
     } else if(ev.gettype() == imevent::authorization) {
 	const imauthorization *m = static_cast<const imauthorization *> (&ev);
@@ -390,6 +457,19 @@ bool icqhook::send(const imevent &ev) {
 		break;
 	}
 
+    } else if(ev.gettype() == imevent::contacts) {
+	const imcontacts *m = static_cast<const imcontacts *> (&ev);
+	vector< pair<unsigned int, string> >::const_iterator icc;
+	list<ContactRef> slst;
+
+	for(icc = m->getcontacts().begin(); icc != m->getcontacts().end(); ++icc) {
+	    ContactRef ct(new Contact(icc->first));
+	    ct->setAlias(ruscrlfconv("kw", icc->second));
+	    slst.push_back(ct);
+	}
+
+	sev = new ContactMessageEvent(ic, slst);
+
     } else {
 	return false;
     }
@@ -405,17 +485,25 @@ bool icqhook::send(const imevent &ev) {
     return true;
 }
 
-void icqhook::sendnewuser(const imcontact &c) {
-    if(logged() && c.uin) {
-	cli.addContact(new Contact(c.uin));
-	cli.fetchSimpleContactInfo(cli.getContact(c.uin));
-	cli.fetchDetailContactInfo(cli.getContact(c.uin));
+void icqhook::sendnewuser(const imcontact &ic) {
+    static time_t lastadd = 0;
 
-	if(conf.getourid(icq).additional["autosync"] == "1") {
-	    icqcontact *cc = clist.get(c);
-	    if(cc)
-	    if(cc->inlist())
-		cli.uploadServerBasedContactList(cli.getContact(c.uin));
+    if(logged() && ic.uin) {
+	if(timer_current-lastadd > DELAY_SENDNEW) {
+	    cli.addContact(new Contact(ic.uin));
+	    cli.fetchSimpleContactInfo(cli.getContact(ic.uin));
+	    cli.fetchDetailContactInfo(cli.getContact(ic.uin));
+
+	    if(conf.getourid(icq).additional["autosync"] == "1") {
+		icqcontact *cc = clist.get(ic);
+		if(cc)
+		if(cc->inlist())
+		    cli.uploadServerBasedContactList(cli.getContact(ic.uin));
+	    }
+
+	    lastadd = timer_current;
+	} else {
+	    uinstosend.push_back(ic);
 	}
     }
 }
@@ -561,8 +649,11 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	params.gender == genderFemale ? ICQ2000::SEX_FEMALE :
 	ICQ2000::SEX_UNSPECIFIED;
 
-    if(!params.kwords.empty()) {
-	searchevent = cli.searchForContacts(params.kwords);
+    if(params.uin) {
+	searchevent = cli.searchForContacts(params.uin);
+
+    } else if(!params.kwords.empty()) {
+	searchevent = cli.searchForContacts(rusconv("kw", params.kwords));
 
     } else if(params.randomgroup) {
 	searchevent = cli.searchForContacts((ICQ2000::RandomChatGroup) params.randomgroup);
@@ -573,15 +664,17 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
     !params.position.empty() || params.onlineonly ||
     params.country || params.language ||
     (sex != ICQ2000::SEX_UNSPECIFIED) || (params.agerange != range_NoRange)) {
-	searchevent = cli.searchForContacts(params.nick, params.firstname,
-	    params.lastname, params.email, params.agerange, sex,
-	    params.language, params.city, params.state, params.country,
-	    params.company, params.department, params.position,
-	    params.onlineonly);
+	searchevent = cli.searchForContacts(rusconv("kw", params.nick),
+	    rusconv("kw", params.firstname), rusconv("kw", params.lastname),
+	    rusconv("kw", params.email), params.agerange, sex,
+	    params.language, rusconv("kw", params.city),
+	    rusconv("kw", params.state), params.country,
+	    rusconv("kw", params.company), rusconv("kw", params.department),
+	    rusconv("kw", params.position), params.onlineonly);
 
     } else {
-	searchevent = cli.searchForContacts(params.nick, params.firstname,
-	    params.lastname);
+	searchevent = cli.searchForContacts(rusconv("kw", params.nick),
+	    rusconv("kw", params.firstname), rusconv("kw", params.lastname));
 
     }
 }
@@ -589,15 +682,15 @@ void icqhook::lookup(const imsearchparams &params, verticalmenu &dest) {
 void icqhook::sendupdateuserinfo(const icqcontact &c) {
     ContactRef ic = cli.getSelfContact();
 
-    cli.self_contact_userinfo_change_signal.clear();
+    cli.self_contact_userinfo_change_signal.disconnect(this);
 
     Contact::MainHomeInfo &home = ic->getMainHomeInfo();
     Contact::HomepageInfo &hpage = ic->getHomepageInfo();
     Contact::WorkInfo &work = ic->getWorkInfo();
 
-    icqcontact::basicinfo cbinfo = c.getbasicinfo();
-    icqcontact::moreinfo cminfo = c.getmoreinfo();
-    icqcontact::workinfo cwinfo = c.getworkinfo();
+    const icqcontact::basicinfo &cbinfo = c.getbasicinfo();
+    const icqcontact::moreinfo &cminfo = c.getmoreinfo();
+    const icqcontact::workinfo &cwinfo = c.getworkinfo();
 
     /* basic information */
 
@@ -666,7 +759,10 @@ void icqhook::sendupdateuserinfo(const icqcontact &c) {
     cli.setRandomChatGroup(cbinfo.randomgroup);
 
     cli.uploadSelfDetails();
-    cli.self_contact_userinfo_change_signal.connect(slot(this, &icqhook::self_contact_userinfo_change_cb));
+    cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
+
+    if(!c.getreginfo().password.empty())
+	cli.changePassword(c.getreginfo().password);
 }
 
 void icqhook::requestawaymsg(const imcontact &c) {
@@ -793,6 +889,9 @@ void icqhook::updateinforecord(ContactRef ic, icqcontact *c) {
 }
 
 void icqhook::processemailevent(const string &sender, const string &email, const string &message) {
+    if(email.empty() && sender.empty() && message.empty())
+	return;
+
     icqcontact *c = clist.getemail(email);
 
     if(!c)
@@ -839,11 +938,13 @@ void icqhook::synclist() {
     */
 
     ContactList tcl;
-    vector<icqcontact *> tobestored, needauth;
+    vector<icqcontact *> tobestored;
     vector<icqcontact *>::iterator ic;
 
-    getsyncstatus(s, tobestored, needauth);
-    for(ic = tobestored.begin(); ic != tobestored.end(); ++ic) {
+    cli.contact_userinfo_change_signal.disconnect(this);
+    getsyncstatus(s, tobestored);
+
+    for(ic = tobestored.begin(); ic != tobestored.end() && tcl.size() <= 10; ++ic) {
 	ContactRef ct = cli.getContact((*ic)->getdesc().uin);
 	if(ct.get()) {
 	    ct->setAlias((*ic)->getnick());
@@ -851,6 +952,7 @@ void icqhook::synclist() {
 	}
     }
 
+    cli.contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
     cli.uploadServerBasedContactList(tcl);
 
     while(syncstatus != ackFetch && logged() && time(0)-t <= 30) {
@@ -882,7 +984,7 @@ void icqhook::synclist() {
     blockmode = Normal;
 }
 
-void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored, vector<icqcontact *> &needauth) {
+void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored) {
     ContactList &cl = cli.getContactList();
     ContactList::iterator ic = cl.begin();
 
@@ -893,16 +995,11 @@ void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored,
 	imcontact imc = imcontact((*ic)->getUIN(), icq);
 	icqcontact *c = clist.get(imc);
 
-	if(c) {
+	if(c)
+	if(c->inlist()) {
 	    if(!(*ic)->getServerBased()) {
-		if(!(*ic)->getAuthAwait()) {
-		    tobestored.push_back(c);
+		tobestored.push_back(c);
 
-		} else if(!c->getbasicinfo().authawait) {
-		    if(find(needauth.begin(), needauth.end(), c) == needauth.end())
-			needauth.push_back(c);
-
-		}
 	    } else {
 		synchronized++;
 
@@ -913,31 +1010,13 @@ void icqhook::getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored,
     }
 }
 
-void icqhook::sendaddauth() {
-    int synchronized;
-    ContactList tcl;
-    ContactRef ct;
-    vector<icqcontact *> tobestored, needauth;
-    vector<icqcontact *>::const_iterator in;
-
-    getsyncstatus(synchronized, tobestored, needauth);
-
-    for(in = needauth.begin(); in != needauth.end(); ++in) {
-	ct = cli.getContact((*in)->getdesc().uin);
-	ct->setAuthAwait(false);
-	tcl.add(ct);
-    }
-
-    cli.uploadServerBasedContactList(tcl);
-}
-
 // ----------------------------------------------------------------------------
 
 void icqhook::connected_cb(ConnectedEvent *ev) {
     flogged = true;
 
     time(&timer_poll);
-    time(&timer_resolve);
+    timer_resolve = timer_poll-PERIOD_RESOLVE+3;
 
     logger.putourstatus(icq, offline, manualstatus);
 
@@ -1010,91 +1089,84 @@ void icqhook::messaged_cb(MessageEvent *ev) {
     ic = imcontact(ev->getContact()->getUIN(), icq);
 
     if(ev->getType() == MessageEvent::Normal) {
-	NormalMessageEvent *r;
-	if(r = dynamic_cast<NormalMessageEvent *>(ev)) {
-	    em.store(immessage(ic, imevent::incoming,
-		rusconv("wk", r->getMessage())));
-	}
+	NormalMessageEvent *r = static_cast<NormalMessageEvent *>(ev);
+	em.store(immessage(ic, imevent::incoming, rusconv("wk", r->getMessage())));
 
     } else if(ev->getType() == MessageEvent::URL) {
-	URLMessageEvent *r;
-	if(r = dynamic_cast<URLMessageEvent *>(ev)) {
-	    em.store(imurl(ic, imevent::incoming,
-		rusconv("wk", r->getURL()),
-		rusconv("wk", r->getMessage())));
-	}
+	URLMessageEvent *r = static_cast<URLMessageEvent *>(ev);
+	em.store(imurl(ic, imevent::incoming,
+	    rusconv("wk", r->getURL()),
+	    rusconv("wk", r->getMessage())));
 
     } else if(ev->getType() == MessageEvent::SMS) {
-	SMSMessageEvent *r;
+	SMSMessageEvent *r = static_cast<SMSMessageEvent *>(ev);
+	icqcontact *c = clist.getmobile(r->getSender());
 
-	if(r = dynamic_cast<SMSMessageEvent *>(ev)) {
-	    icqcontact *c = clist.getmobile(r->getSender());
+	if(!c)
+	if(c = clist.addnew(imcontact(0, infocard), true)) {
+	    icqcontact::basicinfo b = c->getbasicinfo();
+	    b.cellular = r->getSender();
+	    c->setbasicinfo(b);
 
-	    if(!c)
-	    if(c = clist.addnew(imcontact(0, infocard), true)) {
-		icqcontact::basicinfo b = c->getbasicinfo();
-		b.cellular = r->getSender();
+	    c->setdispnick(b.cellular);
+	    c->setnick(b.cellular);
+	}
 
-		c->setbasicinfo(b);
-
-		c->setdispnick(b.cellular);
-		c->setnick(b.cellular);
-	    }
-
-	    if(c) {
-		em.store(imsms(c, imevent::incoming,
-		    rusconv("wk", r->getMessage())));
-	    }
+	if(c) {
+	    em.store(imsms(c, imevent::incoming, siconv(r->getMessage(),
+		"utf8", conf.getrussian(proto) ? "koi8-u" : conf.getdefcharset())));
 	}
 
     } else if(ev->getType() == MessageEvent::AuthReq) {
-	AuthReqEvent *r;
-	if(r = dynamic_cast<AuthReqEvent *>(ev)) {
-	    em.store(imauthorization(ic, imevent::incoming, imauthorization::Request,
-		rusconv("wk", r->getMessage())));
-	}
+	AuthReqEvent *r = static_cast<AuthReqEvent *>(ev);
+	em.store(imauthorization(ic, imevent::incoming, imauthorization::Request,
+	    rusconv("wk", r->getMessage())));
 
     } else if(ev->getType() == MessageEvent::AuthAck) {
-	AuthAckEvent *r;
-	if(r = dynamic_cast<AuthAckEvent *>(ev)) {
-	    icqcontact *c = clist.get(ic);
-	    if(c) {
-		icqcontact::basicinfo bi = c->getbasicinfo();
-		bi.authawait = false;
-		c->setbasicinfo(bi);
-	    }
+	AuthAckEvent *r = static_cast<AuthAckEvent *>(ev);
+	icqcontact *c = clist.get(ic);
 
-	    if(r->isGranted()) {
-		em.store(imnotification(ic, _("The user has accepted your authorization request")));
+	if(c) {
+	    icqcontact::basicinfo bi = c->getbasicinfo();
+	    bi.authawait = false;
+	    c->setbasicinfo(bi);
+	}
 
-	    } else {
-		em.store(imnotification(ic, (string)
-		    _("The user has rejected your authorization request; the message was: ") + r->getMessage()));
+	if(r->isGranted()) {
+	    em.store(imnotification(ic, _("The user has accepted your authorization request")));
+
+	} else {
+	    em.store(imnotification(ic, (string)
+		_("The user has rejected your authorization request; the message was: ") + r->getMessage()));
 
-	    }
 	}
 
     } else if(ev->getType() == MessageEvent::EmailEx) {
-	EmailExEvent *r;
-
-	if(r = dynamic_cast<EmailExEvent *>(ev)) {
-	    processemailevent(r->getSender(), r->getEmail(), r->getMessage());
-	}
+	EmailExEvent *r = static_cast<EmailExEvent *>(ev);
+	processemailevent(r->getSender(), r->getEmail(), r->getMessage());
 
     } else if(ev->getType() == MessageEvent::WebPager) {
-	WebPagerEvent *r;
-
-	if(r = dynamic_cast<WebPagerEvent *>(ev)) {
-	    processemailevent(r->getSender(), r->getEmail(), r->getMessage());
-	}
+	WebPagerEvent *r = static_cast<WebPagerEvent *>(ev);
+	processemailevent(r->getSender(), r->getEmail(), r->getMessage());
 
     } else if(ev->getType() == MessageEvent::UserAdd) {
-	UserAddEvent *r;
-	if(r = dynamic_cast<UserAddEvent *>(ev)) {
-	    em.store(imnotification(ic,
-		_("The user has added you to his/her contact list")));
+	UserAddEvent *r = static_cast<UserAddEvent *>(ev);
+	em.store(imnotification(ic,
+	    _("The user has added you to his/her contact list")));
+
+    } else if(ev->getType() == MessageEvent::Contacts) {
+	ContactMessageEvent *r = static_cast<ContactMessageEvent *>(ev);
+	vector< pair<unsigned int, string> > lst;
+
+	list<ContactRef> clst = r->getContacts();
+	list<ContactRef>::iterator il;
+
+	for(il = clst.begin(); il != clst.end(); ++il) {
+	    lst.push_back(make_pair((*il)->getUIN(), (*il)->getAlias()));
 	}
 
+	em.store(imcontacts(ic, imevent::incoming, lst));
+
     }
 
     ev->setDelivered(true);
@@ -1114,17 +1186,18 @@ void icqhook::messageack_cb(MessageEvent *ev) {
     }
 
     if(ev->isFinished()) {
-	/*
-	*
-	* The FINAL decision.
-	*
-	*/
-	
+	// The FINAL decision.
+
 	switch(ev->getType()) {
 	    case MessageEvent::SMS:
 		if(!ev->isDelivered()) {
-		    face.log(_("+ [icq] failed SMS to %s, %s"),
-			c->getdispnick().c_str(), ic.totext().c_str());
+		    if(ev->getContact()->getUIN()) {
+			face.log(_("+ [icq] failed SMS to %s, %s"),
+			    c->getdispnick().c_str(), ic.totext().c_str());
+		    } else {
+			face.log(_("+ [icq] failed SMS to %s"),
+			    ev->getContact()->getAlias().c_str());
+		    }
 		}
 		break;
 
@@ -1166,9 +1239,6 @@ void icqhook::contact_status_change_signal_cb(StatusChangeEvent *ev) {
 
 	logger.putonline(c, c->getstatus(), nst);
 	c->setstatus(nst);
-
-	if(c->getstatus() != offline)
-	    c->setlastseen();
     }
 }
 
@@ -1190,20 +1260,22 @@ void icqhook::contact_userinfo_change_signal_cb(UserInfoChangeEvent *ev) {
 	updateinforecord(ic, c);
 
     } else {
-	char buf[64];
+	char *p;
 	string lastip, sbuf;
-	int ip;
+	struct in_addr addr;
 
-	if(inet_ntop(AF_INET, &(ip = ntohl(ic->getExtIP())), buf, 64))
-	    lastip = buf;
+	addr.s_addr = ntohl(ic->getExtIP());
+	if(p = inet_ntoa(addr)) lastip = p;
 
-	if(lastip.find_first_not_of(".0") != -1)
-	if(inet_ntop(AF_INET, &(ip = ntohl(ic->getLanIP())), buf, 64)) {
-	    sbuf = buf;
-	
-	    if(sbuf.find_first_not_of(".0") != -1 && lastip != sbuf) {
-		if(!lastip.empty()) lastip += " ";
-		lastip += sbuf;
+	if(lastip.find_first_not_of(".0") != -1) {
+	    addr.s_addr = ntohl(ic->getLanIP());
+	    if(p = inet_ntoa(addr)) {
+		sbuf = p;
+
+		if(sbuf.find_first_not_of(".0") != -1 && lastip != sbuf) {
+		    if(!lastip.empty()) lastip += " ";
+		    lastip += sbuf;
+		}
 	    }
 
 	    c->setlastip(lastip);
@@ -1268,16 +1340,16 @@ void icqhook::want_auto_resp_cb(ICQMessageEvent *ev) {
 
     ident = cont.totext();
     if(c) ident += " (" + c->getdispnick() + ")";
-
+/*
+    face.log(_("+ [icq] %s has requested our away message"), ident.c_str());
     sprintf(buf, _("%s requested our away message, sent the response"), ident.c_str());
-
+*/
     logger.putmessage(buf);
     ev->setAwayMessage(rusconv("kw", conf.getawaymsg(icq)));
 }
 
 void icqhook::search_result_cb(SearchResultEvent *ev) {
     if(ev == searchevent) {
-	string line;
 	ContactRef cont = ev->getLastContactAdded();
 
 	if(searchdest && cont.get()) {
@@ -1297,9 +1369,7 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 
 	    }
 
-	    c->setbasicinfo(binfo);
-
-	    line = (cont->getStatus() == STATUS_ONLINE ? "o " : "  ") + c->getnick();
+	    string line = (cont->getStatus() == STATUS_ONLINE ? "o " : "  ") + c->getnick();
 
 	    if(line.size() > 12) line.resize(12);
 	    else line += string(12-line.size(), ' ');
@@ -1307,15 +1377,17 @@ void icqhook::search_result_cb(SearchResultEvent *ev) {
 	    line += " " + binfo.fname + " " + binfo.lname;
 	    if(!binfo.email.empty()) line += " <" + binfo.email + ">";
 
+	    c->setbasicinfo(binfo);
+
 	    foundguys.push_back(c);
 	    searchdest->additem(conf.getcolor(cp_clist_icq), c, line);
 	    searchdest->redraw();
 	}
 
-	if(ev->isFinished()) {
+	if(ev->isFinished() && searchdest) {
 	    face.findready();
 
-	    face.log(_("+ [icq] whitepages search finished, %d found"),
+	    face.log(_("+ [icq] search finished, %d found"),
 		ev->getContactList().size());
 
 	    searchdest = 0;
@@ -1349,6 +1421,16 @@ void icqhook::self_contact_status_change_cb(StatusChangeEvent *ev) {
     face.update();
 }
 
+void icqhook::password_changed_cb(PasswordChangeEvent *ev) {
+    if(ev->isSuccess()) {
+	icqconf::imaccount acc = conf.getourid(icq);
+	acc.password = ev->getPassword();
+	conf.setourid(acc);
+
+	face.log(_("+ [icq] password was changed successfully"));
+    }
+}
+
 void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
     const ContactList &lst = ev->getContactList();
     ContactList::const_iterator i = lst.begin();
@@ -1358,7 +1440,6 @@ void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
 	syncstatus = ackFetch;
 
 	while(i != lst.end()) {
-	    cli.addContact(*i);
 	    imcontact cont((*i)->getUIN(), icq);
 	    if(!clist.get(cont)) clist.addnew(cont, false);
 	    ++i;
@@ -1367,42 +1448,7 @@ void icqhook::server_based_contact_list_cb(ServerBasedContactEvent *ev) {
     } else if(ev->getType() == ServerBasedContactEvent::Upload) {
 	syncstatus = ackUpload;
 
-	map<unsigned int, ServerBasedContactEvent::UploadResult> r = ev->getUploadResults();
-
-	while(i != lst.end()) {
-	    if(r[(*i)->getUIN()] == ServerBasedContactEvent::AuthRequired) {
-		c = clist.get(imcontact((*i)->getUIN(), icq));
-
-		if(c) {
-		    icqcontact::basicinfo bi = c->getbasicinfo();
-
-		    if(blockmode == Normal) {
-			if(!bi.authawait) {
-			    face.log(_("+ [icq] auto-requesting authorization to store %lu"), (*i)->getUIN());
-			    em.store(imauthorization(c->getdesc(), imevent::outgoing, imauthorization::Request));
-			    cli.uploadServerBasedContactList(*i);
-			}
-
-		    } else if(blockmode == SyncList) {
-			ContactRef ct = cli.getContact((*i)->getUIN());
-			if(ct.get()) {
-			    ct->setAuthAwait(true);
-			    bi.authawait = false;
-			    c->setbasicinfo(bi);
-			    // So that it doesn't get into the next upload
-			    // packet sent..
-			}
-
-		    }
-		}
-	    }
-	    ++i;
-	}
-
     } else if(ev->getType() == ServerBasedContactEvent::Remove) {
-	while(i != lst.end()) {
-	    ++i;
-	}
 
     }
 }
