@@ -10,7 +10,7 @@
 
 using namespace ICQ2000;
 
-class icqhook: public abstracthook, public SigC::Object {
+class icqhook: public abstracthook, public sigslot::has_slots<> {
     protected:
 	Client cli;
 
@@ -20,7 +20,8 @@ class icqhook: public abstracthook, public SigC::Object {
 	    reqUpload,
 	    ackUpload,
 	    reqFetch,
-	    ackFetch
+	    ackFetch,
+	    noSync
 	} syncstatus;
 
 	enum {
@@ -36,6 +37,7 @@ class icqhook: public abstracthook, public SigC::Object {
 	typedef pair<unsigned int, contactstatus> visInfo;
 	vector<visInfo> vislist;
 	vector<icqcontact *> foundguys;
+	vector<imcontact> uinstosend;
 
 	void connected_cb(ConnectedEvent *ev);
 	void disconnected_cb(DisconnectedEvent *ev);
@@ -53,6 +55,7 @@ class icqhook: public abstracthook, public SigC::Object {
 	void server_based_contact_list_cb(ServerBasedContactEvent *ev);
 	void self_contact_userinfo_change_cb(UserInfoChangeEvent *ev);
 	void self_contact_status_change_cb(StatusChangeEvent *ev);
+	void password_changed_cb(PasswordChangeEvent *ev);
 
 	imstatus icq2imstatus(const Status st) const;
 
@@ -98,7 +101,7 @@ class icqhook: public abstracthook, public SigC::Object {
 	void sendupdateuserinfo(const icqcontact &c);
 
 	void synclist();
-	void getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored, vector<icqcontact *> &needauth);
+	void getsyncstatus(int &synchronized, vector<icqcontact *> &tobestored);
 	void sendaddauth();
 };
 
