@@ -1,7 +1,7 @@
 /*
 *
 * centericq IRC protocol handling class
-* $Id: irchook.cc,v 1.40 2002/08/16 16:48:28 konst Exp $
+* $Id: irchook.cc,v 1.70 2003/07/07 18:51:01 konst Exp $
 *
 * Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -23,39 +23,64 @@
 */
 
 #include "irchook.h"
+
+#ifdef BUILD_IRC
+
 #include "icqface.h"
-#include "accountmanager.h"
 #include "icqcontacts.h"
-#include "centericq.h"
 #include "imlogger.h"
 
+#include "accountmanager.h"
+#include "eventmanager.h"
+
+#include <iterator>
+
 #ifdef DEBUG
 #define DLOG(s) face.log("irc %s", s)
 #else
 #define DLOG(s)
 #endif
 
+// ----------------------------------------------------------------------------
+
 irchook irhook;
 
 irchook::irchook()
-    : handle(firetalk_create_handle(FP_IRC, 0)),
+    : abstracthook(irc), handle(firetalk_create_handle(FP_IRC, 0)),
       fonline(false), flogged(false), ourstatus(offline)
 {
-    fcapabilities =
-	hoptCanSetAwayMsg |
-	hoptCanFetchAwayMsg |
-	hoptCanChangeNick |
-	hoptNoPasswords |
-	hoptChangableServer;
+    fcapabs.insert(hookcapab::setaway);
+    fcapabs.insert(hookcapab::fetchaway);
+    fcapabs.insert(hookcapab::changenick);
+    fcapabs.insert(hookcapab::optionalpassword);
+    fcapabs.insert(hookcapab::ping);
+    fcapabs.insert(hookcapab::version);
+    fcapabs.insert(hookcapab::files);
+    fcapabs.insert(hookcapab::cltemporary);
+    fcapabs.insert(hookcapab::directadd);
+    fcapabs.insert(hookcapab::conferencing);
+    fcapabs.insert(hookcapab::channelpasswords);
 }
 
 irchook::~irchook() {
 }
 
 void irchook::init() {
+    int i;
+
     manualstatus = conf.getstatus(irc);
 
-    loadconfig();
+    for(i = 0; i < clist.count; i++) {
+	icqcontact *c = (icqcontact *) clist.at(i);
+
+	if(c->getdesc().pname == irc)
+	if(ischannel(c))
+	if(c->getbasicinfo().requiresauth) {
+	    channels.push_back(channelInfo(c->getdesc().nickname));
+	    channels.back().joined = 1;
+	    channels.back().passwd = c->getbasicinfo().zip;
+	}
+    }
 
     firetalk_register_callback(handle, FC_CONNECTED, &connected);
     firetalk_register_callback(handle, FC_CONNECTFAILED, &connectfailed);
@@ -77,8 +102,26 @@ void irchook::init() {
     firetalk_register_callback(handle, FC_CHAT_JOINED, &chatjoined);
     firetalk_register_callback(handle, FC_CHAT_LEFT, &chatleft);
     firetalk_register_callback(handle, FC_CHAT_KICKED, &chatkicked);
+    firetalk_register_callback(handle, FC_CHAT_OPPED, &chatopped);
+    firetalk_register_callback(handle, FC_CHAT_DEOPPED, &chatdeopped);
     firetalk_register_callback(handle, FC_ERROR, &errorhandler);
     firetalk_register_callback(handle, FC_IM_USER_NICKCHANGED, &nickchanged);
+    firetalk_register_callback(handle, FC_NEEDPASS, &needpass);
+    firetalk_register_callback(handle, FC_FILE_OFFER, &fileoffer);
+    firetalk_register_callback(handle, FC_FILE_START, &filestart);
+    firetalk_register_callback(handle, FC_FILE_PROGRESS, &fileprogress);
+    firetalk_register_callback(handle, FC_FILE_FINISH, &filefinish);
+    firetalk_register_callback(handle, FC_FILE_ERROR, &fileerror);
+    firetalk_register_callback(handle, FC_CHAT_USER_JOINED, &chatuserjoined);
+    firetalk_register_callback(handle, FC_CHAT_USER_LEFT, &chatuserleft);
+    firetalk_register_callback(handle, FC_CHAT_GOTTOPIC, &chatgottopic);
+    firetalk_register_callback(handle, FC_CHAT_USER_OPPED, &chatuseropped);
+    firetalk_register_callback(handle, FC_CHAT_USER_DEOPPED, &chatuserdeopped);
+
+    firetalk_subcode_register_request_callback(handle, "VERSION", &subrequest);
+
+    firetalk_subcode_register_reply_callback(handle, "PING", &subreply);
+    firetalk_subcode_register_reply_callback(handle, "VERSION", &subreply);
 
 #ifdef DEBUG
     firetalk_register_callback(handle, FC_LOG, &log);
@@ -183,23 +226,40 @@ bool irchook::send(const imevent &ev) {
     if(c) {
 	if(ev.gettype() == imevent::message) {
 	    const immessage *m = static_cast<const immessage *>(&ev);
-	    if(m) text = rushtmlconv("kw", m->gettext());
+	    text = rushtmlconv("kw", m->gettext());
 
 	} else if(ev.gettype() == imevent::url) {
 	    const imurl *m = static_cast<const imurl *>(&ev);
-	    if(m) text = rushtmlconv("kw", m->geturl()) + "\n\n" + rushtmlconv("kw", m->getdescription());
+	    text = rushtmlconv("kw", m->geturl()) + "\n\n" + rushtmlconv("kw", m->getdescription());
 
+	} else if(ev.gettype() == imevent::file) {
+	    const imfile *m = static_cast<const imfile *>(&ev);
+	    vector<imfile::record> files = m->getfiles();
+	    vector<imfile::record>::const_iterator ir;
+
+	    for(ir = files.begin(); ir != files.end(); ++ir) {
+		imfile::record r;
+		r.fname = ir->fname;
+		r.size = ir->size;
+
+		imfile fr(c->getdesc(), imevent::outgoing, "",
+		    vector<imfile::record>(1, r));
+
+		firetalk_file_offer(handle, c->getdesc().nickname.c_str(),
+		    ir->fname.c_str(), &transferinfo[fr].first);
+	    }
+
+	    return true;
+	}
+
+	if(text.substr(0, 1) == "/") {
+	    rawcommand(text.substr(1));
+	    return true;
 	}
 
 	if(ischannel(c)) {
-	    if(text.substr(0, 1) == "/") {
-		rawcommand(text.substr(1));
-		return true;
-
-	    } else {
-		return firetalk_chat_send_message(handle,
-		    c->getdesc().nickname.c_str(), text.c_str(), 0) == FE_SUCCESS;
-	    }
+	    return firetalk_chat_send_message(handle,
+		c->getdesc().nickname.c_str(), text.c_str(), 0) == FE_SUCCESS;
 
 	} else {
 	    return firetalk_im_send_message(handle,
@@ -212,9 +272,23 @@ bool irchook::send(const imevent &ev) {
 }
 
 void irchook::sendnewuser(const imcontact &ic) {
-    if(online() && !ischannel(ic)) {
-	firetalk_im_add_buddy(handle, ic.nickname.c_str());
-	requestinfo(ic);
+    icqcontact *c;
+
+    if(online()) {
+	if(!ischannel(ic)) {
+	    firetalk_im_add_buddy(handle, ic.nickname.c_str());
+	    requestinfo(ic);
+	} else {
+	    vector<channelInfo> ch = getautochannels();
+	    if(find(ch.begin(), ch.end(), ic.nickname) == ch.end()) {
+		ch.push_back(ic.nickname);
+		ch.back().joined = true;
+		if(c = clist.get(ic)) {
+		    ch.back().passwd = c->getbasicinfo().zip;
+		}
+		setautochannels(ch);
+	    }
+	}
     }
 }
 
@@ -222,13 +296,15 @@ void irchook::removeuser(const imcontact &ic) {
     if(online()) {
 	if(!ischannel(ic)) {
 	    firetalk_im_remove_buddy(handle, ic.nickname.c_str());
+	} else {
+	    vector<channelInfo> ch = getautochannels();
+	    vector<channelInfo>::iterator ich = find(ch.begin(), ch.end(), ic.nickname);
+	    if(ich != ch.end()) {
+		ch.erase(ich);
+		setautochannels(ch);
+	    }
 	}
     }
-
-    if(ischannel(ic)) {
-	vector<channelInfo>::iterator i = find(channels.begin(), channels.end(), ic.nickname);
-	if(i != channels.end()) i->contactlist = false;
-    }
 }
 
 void irchook::setautostatus(imstatus st) {
@@ -277,8 +353,6 @@ void irchook::sendupdateuserinfo(icqcontact &c, const string &newpass) {
 	if(!ircname.empty()) ircname += " ";
 	ircname += b.lname;
     }
-
-    saveconfig();
 }
 
 void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
@@ -313,7 +387,7 @@ void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
 	    }
 
 	    if(!ic->joined) {
-		firetalk_chat_join(handle, room.c_str());
+		firetalk_chat_join(handle, room.c_str(), ic->passwd.c_str());
 		ready = false;
 	    }
 
@@ -349,7 +423,7 @@ void irchook::lookup(const imsearchparams &params, verticalmenu &dest) {
 void irchook::processnicks() {
     string dnick;
     char *nick;
-    vector<string> foundnicks, prelastfound;
+    vector<string> foundnicks;
     static vector<string> lastfoundnicks;
     vector<string>::iterator in, isn;
     vector<channelInfo>::iterator ir;
@@ -396,20 +470,20 @@ void irchook::processnicks() {
 	}
     }
 
-    prelastfound = foundnicks;
-
     if(searchsincelast) {
-	for(in = foundnicks.begin(); in != foundnicks.end(); ) {
-	    if(find(lastfoundnicks.begin(), lastfoundnicks.end(), *in) != lastfoundnicks.end()) {
-		foundnicks.erase(in);
-		in = foundnicks.begin();
-	    } else {
-		in++;
-	    }
-	}
-    }
+	vector<string> tnicks;
+	insert_iterator< vector<string> > tins(tnicks, tnicks.begin());
 
-    lastfoundnicks = prelastfound;
+	set_difference(foundnicks.begin(), foundnicks.end(),
+	    lastfoundnicks.begin(), lastfoundnicks.end(), tins);
+
+	lastfoundnicks = foundnicks;
+	foundnicks = tnicks;
+
+    } else {
+	lastfoundnicks = foundnicks;
+
+    }
 
     for(in = foundnicks.begin(); in != foundnicks.end(); ++in) {
 	dnick = *in;
@@ -471,12 +545,6 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
 	    if(!r) r = !iac->joined;
 	    if(r) firetalk_chat_part(irhook.handle, ic->name.c_str());
 	}
-
-	if(ic->contactlist) {
-	    r = iac == achannels.end();
-	    if(!r) r = !iac->contactlist;
-	    if(r) clist.remove(imcontact(ic->name, irc));
-	}
     }
 
     /*
@@ -492,21 +560,11 @@ void irchook::setautochannels(vector<channelInfo> &achannels) {
 	if(iac->joined) {
 	    r = ic == channels.end();
 	    if(!r) r = !ic->joined;
-	    if(r) firetalk_chat_join(irhook.handle, iac->name.c_str());
-	}
-
-	if(iac->contactlist) {
-	    r = ic == channels.end();
-	    if(!r) r = !ic->contactlist;
-	    if(r) {
-		icqcontact *c = clist.addnew(imcontact(iac->name, irc), false);
-		c->setstatus((iac->joined && gethook(irc).logged()) ? available : offline);
-	    }
+	    if(r) firetalk_chat_join(irhook.handle, iac->name.c_str(), iac->passwd.c_str());
 	}
     }
 
     channels = achannels;
-    saveconfig();
 }
 
 void irchook::requestawaymsg(const imcontact &ic) {
@@ -514,48 +572,6 @@ void irchook::requestawaymsg(const imcontact &ic) {
 	_("Away message:") + "\n\n" + awaymessages[ic.nickname]));
 }
 
-void irchook::saveconfig() const {
-    vector<channelInfo>::iterator ic;
-    vector<channelInfo> savech = getautochannels();
-    ofstream f(conf.getconfigfname("irc-profile").c_str());
-
-    if(f.is_open()) {
-	f << "name\t" << ircname << endl;
-
-	for(ic = savech.begin(); ic != savech.end(); ++ic)
-	    f << "channel\t" << ic->name
-		<< "\t" << (ic->joined ? "1" : "0")
-		<< endl;
-
-	f.close();
-    }
-}
-
-void irchook::loadconfig() {
-    string buf, param;
-    vector<channelInfo>::iterator ic;
-
-    ifstream f(conf.getconfigfname("irc-profile").c_str());
-
-    if(f.is_open()) {
-	channels.clear();
-
-	while(!f.eof()) {
-	    getstring(f, buf);
-	    param = getword(buf);
-
-	    if(param == "name") ircname = buf; else
-	    if(param == "channel") {
-		channels.push_back(channelInfo(getword(buf)));
-		channels.back().joined = (getword(buf) == "1");
-		channels.back().contactlist = clist.get(imcontact(channels.back().name, irc));
-	    }
-	}
-
-	f.close();
-    }
-}
-
 void irchook::rawcommand(const string &cmd) {
     int *r, *w, *e, sock;
     if(online()) {
@@ -592,11 +608,75 @@ void irchook::channelfatal(string room, const char *fmt, ...) {
 	if(!c) c = clist.addnew(cont);
 	c->setstatus(offline);
 	i->joined = i->fetched = false;
-	i->contactlist = true;
 	em.store(imnotification(cont, buf));
     }
 }
 
+void irchook::ouridchanged(const icqconf::imaccount &ia) {
+    if(logged()) {
+	firetalk_set_nickname(handle, ia.nickname.c_str());
+    }
+}
+
+void irchook::requestversion(const imcontact &c) {
+    if(logged()) {
+	firetalk_subcode_send_request(handle, c.nickname.c_str(), "VERSION", 0);
+    }
+}
+
+void irchook::ping(const imcontact &c) {
+    if(logged()) {
+	firetalk_subcode_send_request(handle, c.nickname.c_str(), "PING", 0);
+	time(&pingtime[up(c.nickname)]);
+    }
+}
+
+bool irchook::knowntransfer(const imfile &fr) const {
+    return transferinfo.find(fr) != transferinfo.end();
+}
+
+void irchook::replytransfer(const imfile &fr, bool accept, const string &localpath) {
+    if(accept) {
+	transferinfo[fr].second = localpath;
+
+	if(transferinfo[fr].second.substr(transferinfo[fr].second.size()-1) != "/")
+	    transferinfo[fr].second += "/";
+
+	transferinfo[fr].second += justfname(fr.getfiles().begin()->fname);
+
+	firetalk_file_accept(handle, transferinfo[fr].first, 0,
+	    transferinfo[fr].second.c_str());
+
+    } else {
+	firetalk_file_refuse(handle, transferinfo[fr].first);
+	transferinfo.erase(fr);
+
+    }
+}
+
+void irchook::aborttransfer(const imfile &fr) {
+    firetalk_file_cancel(handle, transferinfo[fr].first);
+
+    face.transferupdate(fr.getfiles().begin()->fname, fr,
+	icqface::tsCancel, 0, 0);
+
+    irhook.transferinfo.erase(fr);
+}
+
+bool irchook::getfevent(void *fhandle, imfile &fr) {
+    map<imfile, pair<void *, string> >::const_iterator i = transferinfo.begin();
+
+    while(i != transferinfo.end()) {
+	if(i->second.first == fhandle) {
+	    fr = i->first;
+	    return true;
+	}
+	++i;
+    }
+
+    return false;
+}
+
 // ----------------------------------------------------------------------------
 
 void irchook::userstatus(const string &nickname, imstatus st) {
@@ -608,9 +688,6 @@ void irchook::userstatus(const string &nickname, imstatus st) {
 	if(st != c->getstatus()) {
 	    logger.putonline(ic, c->getstatus(), st);
 	    c->setstatus(st);
-
-	    if(c->getstatus() != offline)
-		c->setlastseen();
 	}
     }
 }
@@ -635,13 +712,14 @@ void irchook::connected(void *conn, void *cli, ...) {
 
     for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic) {
 	if(ic->joined) {
-	    firetalk_chat_join(irhook.handle, ic->name.c_str());
+	    firetalk_chat_join(irhook.handle, ic->name.c_str(), ic->passwd.c_str());
 	}
     }
 
     irhook.ourstatus = available;
     irhook.setautostatus(irhook.manualstatus);
     irhook.awaymessages.clear();
+    irhook.sentpass = false;
 }
 
 void irchook::disconnected(void *conn, void *cli, ...) {
@@ -654,7 +732,7 @@ void irchook::disconnected(void *conn, void *cli, ...) {
     for(ic = irhook.channels.begin(); ic != irhook.channels.end(); ++ic)
 	ic->fetched = false;
 
-    face.log(_("+ [irc] disconnected from the network"));
+    face.log(_("+ [irc] disconnected"));
     face.update();
 }
 
@@ -722,14 +800,14 @@ void irchook::gotinfo(void *conn, void *cli, ...) {
 	}
 
 	cbinfo.lname = "";
-	cbinfo.email = rushtmlconv("wk", email);
+	cbinfo.email = irhook.rushtmlconv("wk", email);
 
 	if((pos = about.find(" ")) != -1) {
-	    cbinfo.lname = rushtmlconv("wk", about.substr(pos+1));
+	    cbinfo.lname = irhook.rushtmlconv("wk", about.substr(pos+1));
 	    about.erase(pos);
 	}
 
-	cbinfo.fname = rushtmlconv("wk", about);
+	cbinfo.fname = irhook.rushtmlconv("wk", about);
 
 	c->setnick(nick);
 	c->setbasicinfo(cbinfo);
@@ -776,8 +854,16 @@ void irchook::getmessage(void *conn, void *cli, ...) {
 
     if(sender && message)
     if(strlen(sender) && strlen(message)) {
+	if(!irhook.sentpass)
+	if(up(sender) == "NICKSERV") {
+	    char buf[512];
+	    needpass(0, 0, buf, 512);
+	    firetalk_im_send_message(irhook.handle, "NickServ",
+		((string) "identify " + buf).c_str(), 0);
+	}
+
 	em.store(immessage(imcontact(sender, irc),
-	    imevent::incoming, rushtmlconv("wk", message)));
+	    imevent::incoming, irhook.rushtmlconv("wk", cuthtml(message, true))));
     }
 
     DLOG("getmessage");
@@ -824,7 +910,7 @@ void irchook::buddyaway(void *conn, void *cli, ...) {
     if(nick)
     if(strlen(nick)) {
 	irhook.userstatus(nick, away);
-	if(msg) irhook.awaymessages[nick] = rushtmlconv("wk", msg);
+	if(msg) irhook.awaymessages[nick] = irhook.rushtmlconv("wk", msg);
     }
 
     DLOG("buddyaway");
@@ -1023,9 +1109,7 @@ void irchook::chatleft(void *connection, void *cli, ...) {
     va_end(ap);
 
     icqcontact *c = clist.get(imcontact(room, irc));
-    if(c) {
-	c->setstatus(offline);
-    }
+    if(c) c->setstatus(offline);
 }
 
 void irchook::chatkicked(void *connection, void *cli, ...) {
@@ -1038,7 +1122,7 @@ void irchook::chatkicked(void *connection, void *cli, ...) {
     va_end(ap);
 
     irhook.channelfatal(room, _("Kicked by %s; reason: %s"),
-	by, rushtmlconv("wk", reason).c_str());
+	by, irhook.rushtmlconv("wk", reason).c_str());
 }
 
 void irchook::errorhandler(void *connection, void *cli, ...) {
@@ -1083,8 +1167,9 @@ void irchook::nickchanged(void *connection, void *cli, ...) {
 	if(c = clist.get(imcontact(oldnick, irc))) {
 	    if(!clist.get(imcontact(newnick, irc))) {
 		if(!c->inlist()) {
-		    c->setnick(newnick);
 		    if(c->getdispnick() == oldnick) c->setdispnick(newnick);
+		    c->setdesc(imcontact(newnick, irc));
+		    c->setnick(newnick);
 		}
 
 	    } else {
@@ -1098,18 +1183,278 @@ void irchook::nickchanged(void *connection, void *cli, ...) {
     }
 }
 
-// ----------------------------------------------------------------------------
+void irchook::needpass(void *conn, void *cli, ...) {
+    va_list ap;
 
-static string up(string s) {
-    int k;
-    string r;
+    va_start(ap, cli);
+    char *pass = va_arg(ap, char *);
+    int size = va_arg(ap, int);
+    va_end(ap);
 
-    for(k = 0; k < s.size(); k++)
-	r += toupper(s[k]);
+    if(pass) {
+	icqconf::imaccount acc = conf.getourid(irc);
 
-    return r;
+	if(!acc.password.empty()) {
+	    strncpy(pass, acc.password.c_str(), size-1);
+	    pass[size-1] = 0;
+	    face.log(_("+ [irc] password sent"));
+	} else {
+	    face.log(_("+ [irc] password was requested, but it's not set"));
+	}
+
+	irhook.sentpass = true;
+    }
 }
 
+void irchook::subrequest(void *conn, void *cli, const char * const nick,
+const char * const command, const char * const args) {
+
+    if(!strcmp(command, "VERSION")) {
+	firetalk_subcode_send_reply(conn, nick, "VERSION", PACKAGE " " VERSION);
+
+    }
+}
+
+void irchook::subreply(void *conn, void *cli, const char * const nick,
+const char * const command, const char * const args) {
+    char buf[512];
+
+    if(!strcmp(command, "PING")) {
+	map<string, time_t>::iterator i = irhook.pingtime.find(up(nick));
+
+	if(i != irhook.pingtime.end()) {
+	    sprintf(buf, _("PING reply from the user: %d second(s)"), time(0)-i->second);
+	    em.store(imnotification(imcontact(nick, irc), buf));
+	}
+
+    } else if(!strcmp(command, "VERSION")) {
+	sprintf(buf, _("The remote is using %s"), args);
+	em.store(imnotification(imcontact(nick, irc), buf));
+
+    }
+}
+
+void irchook::fileoffer(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    void *filehandle = va_arg(ap, void *);
+    char *from = va_arg(ap, char *);
+    char *filename = va_arg(ap, char *);
+    long size = va_arg(ap, long);
+    va_end(ap);
+
+    imfile::record r;
+    r.fname = filename;
+    r.size = size;
+
+    imfile fr(imcontact(from, irc), imevent::incoming, "",
+	vector<imfile::record>(1, r));
+
+    irhook.transferinfo[fr].first = filehandle;
+    em.store(fr);
+
+    face.transferupdate(filename, fr, icqface::tsInit, size, 0);
+}
+
+void irchook::filestart(void *conn, void *cli, ...) {
+    va_list ap;
+    imfile fr;
+
+    va_start(ap, cli);
+    void *filehandle = va_arg(ap, void *);
+    void *clientfilestruct = va_arg(ap, void *);
+    va_end(ap);
+
+    if(irhook.getfevent(filehandle, fr)) {
+	face.transferupdate(fr.getfiles().begin()->fname, fr,
+	    icqface::tsStart, 0, 0);
+    }
+}
+
+void irchook::fileprogress(void *conn, void *cli, ...) {
+    va_list ap;
+    imfile fr;
+
+    va_start(ap, cli);
+    void *filehandle = va_arg(ap, void *);
+    void *clientfilestruct = va_arg(ap, void *);
+    long bytes = va_arg(ap, long);
+    long size = va_arg(ap, long);
+    va_end(ap);
+
+    if(irhook.getfevent(filehandle, fr)) {
+	face.transferupdate(fr.getfiles().begin()->fname, fr,
+	    icqface::tsProgress, size, bytes);
+    }
+}
+
+void irchook::filefinish(void *conn, void *cli, ...) {
+    va_list ap;
+    imfile fr;
+
+    va_start(ap, cli);
+    void *filehandle = va_arg(ap, void *);
+    void *clientfilestruct = va_arg(ap, void *);
+    long size = va_arg(ap, long);
+    va_end(ap);
+
+    if(irhook.getfevent(filehandle, fr)) {
+	face.transferupdate(fr.getfiles().begin()->fname, fr,
+	    icqface::tsFinish, size, 0);
+
+	irhook.transferinfo.erase(fr);
+    }
+}
+
+void irchook::fileerror(void *conn, void *cli, ...) {
+    va_list ap;
+    imfile fr;
+
+    va_start(ap, cli);
+    void *filehandle = va_arg(ap, void *);
+    void *clientfilestruct = va_arg(ap, void *);
+    int error = va_arg(ap, int);
+    va_end(ap);
+
+    if(irhook.getfevent(filehandle, fr)) {
+	face.transferupdate(fr.getfiles().begin()->fname, fr,
+	    icqface::tsError, 0, 0);
+
+	irhook.transferinfo.erase(fr);
+    }
+}
+
+void irchook::chatuserjoined(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *who = va_arg(ap, char *);
+    char *email = va_arg(ap, char *);
+    va_end(ap);
+
+    if(conf.getourid(irc).nickname != who) {
+	string uname = who;
+
+	if(email)
+	if(strlen(email))
+	    uname += (string) " (" + email + ")";
+
+	char buf[512];
+	sprintf(buf, _("%s has joined."), uname.c_str());
+	em.store(imnotification(imcontact(room, irc), buf));
+    }
+}
+
+void irchook::chatuserleft(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *who = va_arg(ap, char *);
+    char *reason = va_arg(ap, char *);
+    va_end(ap);
+
+    if(conf.getourid(irc).nickname != who) {
+	string text;
+	char buf[512];
+
+	sprintf(buf, _("%s has left"), who); text = buf;
+
+	if(reason)
+	if(strlen(reason)) {
+	    sprintf(buf, _("reason: %s"), reason);
+	    text += (string) "; " + buf + ".";
+	}
+
+	em.store(imnotification(imcontact(room, irc), text));
+    }
+}
+
+void irchook::chatgottopic(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *topic = va_arg(ap, char *);
+    char *author = va_arg(ap, char *);
+    va_end(ap);
+
+    string text;
+    char buf[1024];
+    sprintf(buf, _("Channel topic now is: %s"), topic);
+    text = buf;
+
+    if(author)
+    if(strlen(author)) {
+	sprintf(buf, _("set by %s"), author);
+	text += (string) "; " + buf + ".";
+    }
+
+    em.store(imnotification(imcontact(room, irc), text));
+}
+
+void irchook::chatuseropped(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *who = va_arg(ap, char *);
+    char *by = va_arg(ap, char *);
+    va_end(ap);
+
+    if(by) {
+	char buf[512];
+	sprintf(buf, _("%s has been opped by %s."), who, by);
+	em.store(imnotification(imcontact(room, irc), buf));
+    }
+}
+
+void irchook::chatuserdeopped(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *who = va_arg(ap, char *);
+    char *by = va_arg(ap, char *);
+    va_end(ap);
+
+    if(by) {
+	char buf[512];
+	sprintf(buf, _("%s has been deopped by %s."), who, by);
+	em.store(imnotification(imcontact(room, irc), buf));
+    }
+}
+
+void irchook::chatopped(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *by = va_arg(ap, char *);
+    va_end(ap);
+
+    char buf[512];
+    sprintf(buf, _("%s has opped us."), by);
+    em.store(imnotification(imcontact(room, irc), buf));
+}
+
+void irchook::chatdeopped(void *conn, void *cli, ...) {
+    va_list ap;
+
+    va_start(ap, cli);
+    char *room = va_arg(ap, char *);
+    char *by = va_arg(ap, char *);
+    va_end(ap);
+
+    char buf[512];
+    sprintf(buf, _("%s has deopped us."), by);
+    em.store(imnotification(imcontact(room, irc), buf));
+}
+
+// ----------------------------------------------------------------------------
+
 bool irchook::channelInfo::operator != (const string &aname) const {
     return up(aname) != up(name);
 }
@@ -1117,3 +1462,5 @@ bool irchook::channelInfo::operator != (const string &aname) const {
 bool irchook::channelInfo::operator == (const string &aname) const {
     return up(aname) == up(name);
 }
+
+#endif
