@@ -3,26 +3,28 @@
 
 #include "abstracthook.h"
 
+#ifdef BUILD_IRC
+
 extern "C" {
 #include "firetalk.h"
 }
 
 class irchook: public abstracthook {
-    public:
+    protected:
 	struct channelInfo {
 	    string name;
-	    bool joined, fetched, contactlist;
+	    string passwd;
+	    bool joined, fetched;
 
 	    vector<string> nicks;
 
 	    channelInfo(const string &aname):
-		name(aname), joined(false), fetched(false) {}
+		name(aname), joined(false), fetched(false), passwd("") {}
 
 	    bool operator != (const string &aname) const;
 	    bool operator == (const string &aname) const;
 	};
 
-    protected:
 	enum searchMode {
 	    Channel,
 	    Email
@@ -30,21 +32,23 @@ class irchook: public abstracthook {
 
 	searchMode smode;
 
-	bool fonline, flogged, searchsincelast;
+	bool fonline, flogged, searchsincelast, sentpass;
 	firetalk_t handle;
 	imstatus ourstatus;
 	string ircname, emailsub, namesub;
 
 	vector<channelInfo> channels;
 	vector<string> searchchannels, extlisted;
-	map<string, string> awaymessages;
 	vector<icqcontact *> foundguys;
 
+	map<string, string> awaymessages;
+	map<string, time_t> pingtime;
+	map<imfile, pair<void *, string> > transferinfo;
+
 	void userstatus(const string &nickname, imstatus st);
 	void processnicks();
 
-	void saveconfig() const;
-	void loadconfig();
+	bool getfevent(void *fhandle, imfile &fr);
 
 	static void connected(void *conn, void *cli, ...);
 	static void disconnected(void *conn, void *cli, ...);
@@ -67,10 +71,34 @@ class irchook: public abstracthook {
 	static void chatkicked(void *connection, void *cli, ...);
 	static void errorhandler(void *connection, void *cli, ...);
 	static void nickchanged(void *connection, void *cli, ...);
+	static void needpass(void *conn, void *cli, ...);
+	static void fileoffer(void *conn, void *cli, ...);
+	static void filestart(void *conn, void *cli, ...); 
+	static void fileprogress(void *conn, void *cli, ...);
+	static void filefinish(void *conn, void *cli, ...);
+	static void fileerror(void *conn, void *cli, ...);
+	static void chatuserjoined(void *conn, void *cli, ...);
+	static void chatuserleft(void *conn, void *cli, ...);
+	static void chatgottopic(void *conn, void *cli, ...);
+	static void chatuseropped(void *conn, void *cli, ...);
+	static void chatuserdeopped(void *conn, void *cli, ...);
+	static void chatopped(void *conn, void *cli, ...);
+	static void chatdeopped(void *conn, void *cli, ...);
+
+	static void subrequest(void *conn, void *cli,
+	    const char * const nick, const char * const command,
+	    const char * const args);
+
+	static void subreply(void *conn, void *cli,
+	    const char * const nick, const char * const command,
+	    const char * const args);
 
 	void rawcommand(const string &cmd);
 	void channelfatal(string room, const char *fmt, ...);
 
+	vector<channelInfo> getautochannels() const;
+	void setautochannels(vector<channelInfo> &achannels);
+
     public:
 	irchook();
 	~irchook();
@@ -99,16 +127,24 @@ class irchook: public abstracthook {
 
 	imstatus getstatus() const;
 
+	void requestversion(const imcontact &c);
+	void ping(const imcontact &c);
+
 	void requestinfo(const imcontact &c);
 	void requestawaymsg(const imcontact &c);
 	void sendupdateuserinfo(icqcontact &c, const string &newpass);
 
 	void lookup(const imsearchparams &params, verticalmenu &dest);
 
-	vector<channelInfo> getautochannels() const;
-	void setautochannels(vector<channelInfo> &achannels);
+	void ouridchanged(const icqconf::imaccount &ia);
+
+	bool knowntransfer(const imfile &fr) const;
+	void replytransfer(const imfile &fr, bool accept, const string &localpath = string());
+	void aborttransfer(const imfile &fr);
 };
 
 extern irchook irhook;
 
 #endif
+
+#endif
