@@ -8,10 +8,12 @@
 #include <algorithm>
 #include <list>
 #include <map>
+#include <set>
 #include <memory>
 
 #include <sys/time.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <unistd.h>
 #include <regex.h>
 
@@ -21,15 +23,14 @@
 #include "kkstrtext.h"
 #include "conf.h"
 
-
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
 #include <sys/param.h>
+
+#ifdef BSD
 #include <sys/mount.h>
 #else
 #include <sys/vfs.h>
 #endif
 
-
 #define PERIOD_RECONNECT        40
 
 #ifdef ENABLE_NLS
@@ -57,8 +58,21 @@ using namespace std;
 #define PERIOD_RESEND           20
 #define PERIOD_CHECKMAIL        30
 #define PERIOD_DISPUPDATE       2
+#define PERIOD_AUTOSAVE         120
+#define PERIOD_TYPING           6
 
 #define PERIOD_ATONCE           5
 #define MAX_ATONCE              10
 
+#define DEFAULT_CHARSET "ISO-8859-1"
+
+/*
+*
+* Several helper routines
+*
+*/
+
+string up(string s);
+string lo(string s);
+
 #endif
