@@ -1,9 +1,9 @@
 /*
 *
 * centericq configuration handling routines
-* $Id: icqconf.cc,v 1.78 2002/08/19 08:25:17 konst Exp $
+* $Id: icqconf.cc,v 1.118 2003/10/01 00:27:45 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "imexternal.h"
 #include "eventmanager.h"
 #include "imlogger.h"
+#include "connwrap.h"
 
 icqconf::icqconf() {
     rs = rscard;
@@ -45,18 +46,26 @@ icqconf::icqconf() {
 
     autoaway = autona = 0;
 
-    hideoffline = antispam = russian = makelog = askaway = chatmode = false;
+    hideoffline = antispam = makelog = askaway = logtimestamps =
+	logonline = emacs = false;
+
     savepwd = mailcheck = fenoughdiskspace = true;
 
+    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1) {
+	chatmode[pname] = true;
+	russian[pname] = false;
+	entersends[pname] = false;
+    }
+
     basedir = (string) getenv("HOME") + "/.centericq/";
 }
 
 icqconf::~icqconf() {
 }
 
-icqconf::imaccount icqconf::getourid(protocolname pname) {
+icqconf::imaccount icqconf::getourid(protocolname pname) const {
     imaccount im;
-    vector<imaccount>::iterator i;
+    vector<imaccount>::const_iterator i;
 
     i = find(accounts.begin(), accounts.end(), pname);
     im = i == accounts.end() ? imaccount(pname) : *i;
@@ -64,20 +73,20 @@ icqconf::imaccount icqconf::getourid(protocolname pname) {
     return im;
 }
 
+icqconf::imserver icqconf::defservers[protocolname_size] = {
+    { "login.icq.com", 5190, 0 },
+    { "scs.msg.yahoo.com", 5050, 0 },
+    { "messenger.hotmail.com", 1863, 0 },
+    { "toc.oscar.aol.com", 9898, 0 },
+    { "irc.oftc.net", 6667, 0 },
+    { "jabber.com", 5222, 5223 },
+    { "", 0, 0 },
+    { "livejournal.com", 80, 0 }
+};
+
 void icqconf::setourid(const imaccount &im) {
     vector<imaccount>::iterator i;
 
-    static struct {
-	string server;
-	int port;
-    } defservers[protocolname_size] = {
-	{ "login.icq.com", 5190 },
-	{ "scs.yahoo.com", 5050 },
-	{ "messenger.hotmail.com", 1863 },
-	{ "toc.oscar.aol.com", 9898 },
-	{ "irc.openprojects.net", 6667 }
-    };
-
     i = find(accounts.begin(), accounts.end(), im.pname);
 
     if(i != accounts.end()) {
@@ -98,7 +107,17 @@ void icqconf::setourid(const imaccount &im) {
 	    i->port = defservers[i->pname].port;
 	}
 
-	if(!i->port) i->port = defservers[i->pname].port;
+	if(!i->port)
+	    i->port = defservers[i->pname].port;
+
+	switch(i->pname) {
+	    case icq:
+		if(i->password.size() > 8)
+		    i->password = i->password.substr(0, 8);
+		break;
+	}
+
+	gethook(i->pname).ouridchanged(*i);
     }
 }
 
@@ -106,7 +125,7 @@ string icqconf::getawaymsg(protocolname pname) const {
     string r, buf, fname;
     ifstream f;
 
-    if(!(gethook(pname).getcapabilities() & hoptCanSetAwayMsg))
+    if(!gethook(pname).getCapabs().count(hookcapab::setaway))
 	return "";
 
     fname = conf.getconfigfname("awaymsg-" + getprotocolname(pname));
@@ -138,7 +157,7 @@ void icqconf::setawaymsg(protocolname pname, const string &amsg) {
     string fname;
     ofstream f;
 
-    if(!(gethook(pname).getcapabilities() & hoptCanSetAwayMsg))
+    if(!gethook(pname).getCapabs().count(hookcapab::setaway))
 	return;
 
     fname = conf.getconfigfname("awaymsg-" + getprotocolname(pname));
@@ -151,9 +170,11 @@ void icqconf::setawaymsg(protocolname pname, const string &amsg) {
 }
 
 void icqconf::checkdir() {
-    if(access(getdirname().c_str(), F_OK)) {
-	mkdir(getdirname().c_str(), S_IREAD | S_IWRITE | S_IEXEC);
-    }
+    string dname = getdirname();
+    dname.erase(dname.size()-1);
+
+    if(access(dname.c_str(), F_OK))
+	mkdir(dname.c_str(), S_IRUSR | S_IWUSR | S_IXUSR);
 }
 
 void icqconf::load() {
@@ -173,6 +194,8 @@ void icqconf::loadmainconfig() {
 	mailcheck = askaway = false;
 	savepwd = bidi = true;
 	setsmtphost("");
+	setpeertopeer(0, 65535);
+	defcharset = "ISO-8859-1";
 
 	while(!f.eof()) {
 	    getstring(f, buf);
@@ -180,7 +203,7 @@ void icqconf::loadmainconfig() {
 	    param = getword(buf);
 
 	    if(param == "hideoffline") hideoffline = true; else
-	    if(param == "russian") russian = true; else
+	    if(param == "emacs") emacs = true; else
 	    if(param == "autoaway") autoaway = atol(buf.c_str()); else
 	    if(param == "autona") autona = atol(buf.c_str()); else
 	    if(param == "antispam") antispam = true; else
@@ -192,10 +215,20 @@ void icqconf::loadmainconfig() {
 	    if((param == "usegroups") || (param == "group1")) fgroupmode = group1; else
 	    if(param == "group2") fgroupmode = group2; else
 	    if(param == "smtp") setsmtphost(buf); else
+	    if(param == "http_proxy") sethttpproxyhost(buf); else
 	    if(param == "log") makelog = true; else
-	    if(param == "chatmode") chatmode = true; else
+	    if(param == "chatmode") initmultiproto(chatmode, buf); else
+	    if(param == "entersends") initmultiproto(entersends, buf); else
+	    if(param == "russian") initmultiproto(russian, buf); else
 	    if(param == "nobidi") setbidi(false); else
-	    if(param == "askaway") askaway = true; else {
+	    if(param == "askaway") askaway = true; else
+	    if(param == "logtimestamps") logtimestamps = true; else
+	    if(param == "defcharset") setdefcharset(buf); else
+	    if(param == "logonline") logonline = true; else
+	    if(param == "ptp") {
+		ptpmin = atoi(getword(buf, "-").c_str());
+		ptpmax = atoi(buf.c_str());
+	    } else {
 		for(pname = icq; pname != protocolname_size; (int) pname += 1) {
 		    buf = getprotocolname(pname);
 		    if(param.substr(0, buf.size()) == buf) {
@@ -229,18 +262,44 @@ void icqconf::save() {
 
 	    getauto(away, na);
 
-	    if(russian) f << "russian" << endl;
 	    if(away) f << "autoaway\t" << i2str(away) << endl;
 	    if(na) f << "autona\t" << i2str(na) << endl;
 	    if(hideoffline) f << "hideoffline" << endl;
+	    if(emacs) f << "emacs" << endl;
 	    if(getquote()) f << "quotemsgs" << endl;
 	    if(getantispam()) f << "antispam" << endl;
 	    if(getmailcheck()) f << "mailcheck" << endl;
 	    if(getaskaway()) f << "askaway" << endl;
-	    if(getchatmode()) f << "chatmode" << endl;
-	    if(!getbidi()) f << "nobidi" << endl;
+	    f << "defcharset" << getdefcharset() << endl;
 
-	    f << "smtp\t" << getsmtphost() << ":" << dec << getsmtpport() << endl;
+	    param = "";
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		if(getchatmode(pname)) param += (string) " " + conf.getprotocolname(pname);
+	    if(!param.empty())
+		f << "chatmode" << param << endl;
+
+	    param = "";
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		if(getentersends(pname)) param += (string) " " + conf.getprotocolname(pname);
+	    if(!param.empty())
+		f << "entersends" << param << endl;
+
+	    param = "";
+	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+		if(getrussian(pname)) param += (string) " " + conf.getprotocolname(pname);
+	    if(!param.empty())
+		f << "russian" << param << endl;
+
+	    if(!getbidi()) f << "nobidi" << endl;
+	    if(logtimestamps) f << "logtimestamps" << endl;
+	    if(logonline) f << "logonline" << endl;
+
+	    f << "smtp\t" << getsmtphost() << ":" << getsmtpport() << endl;
+
+	    if(!gethttpproxyhost().empty())
+		f << "http_proxy\t" << gethttpproxyhost() << ":" << gethttpproxyport() << endl;
+
+	    f << "ptp\t" << ptpmin << "-" << ptpmax << endl;
 
 	    switch(getgroupmode()) {
 		case group1: f << "group1" << endl; break;
@@ -274,12 +333,17 @@ void icqconf::loadcolors() {
 	    schemer.push(cp_main_selected, "main_selected   black/white");
 	    schemer.push(cp_main_highlight, "main_highlight yellow/transparent  bold");
 	    schemer.push(cp_main_frame, "main_frame blue/transparent    bold");
+	    schemer.push(cp_main_history_incoming, "main_history_incoming  cyan/transparent");
+	    schemer.push(cp_main_history_outgoing, "main_history_outgoing  yellow/transparent  bold");
 	    schemer.push(cp_clist_icq, "clist_icq   green/transparent");
 	    schemer.push(cp_clist_yahoo, "clist_yahoo   magenta/transparent");
 	    schemer.push(cp_clist_infocard, "clist_infocard white/transparent");
 	    schemer.push(cp_clist_msn, "clist_msn   cyan/transparent");
 	    schemer.push(cp_clist_aim, "clist_aim   yellow/transparent");
 	    schemer.push(cp_clist_irc, "clist_irc    blue/transparent");
+	    schemer.push(cp_clist_jabber, "clist_jabber    red/transparent");
+	    schemer.push(cp_clist_rss, "clist_rss    white/transparent   bold");
+	    schemer.push(cp_clist_lj, "clist_lj    cyan/transparent   bold");
 	    break;
 
 	case rcblue:
@@ -294,12 +358,17 @@ void icqconf::loadcolors() {
 	    schemer.push(cp_main_selected, "main_selected   black/cyan");
 	    schemer.push(cp_main_highlight, "main_highlight  white/blue  bold");
 	    schemer.push(cp_main_frame, "main_frame  blue/blue   bold");
+	    schemer.push(cp_main_history_incoming, "main_history_incoming  white/blue");
+	    schemer.push(cp_main_history_outgoing, "main_history_outgoing  white/blue  bold");
 	    schemer.push(cp_clist_icq, "clist_icq   green/blue");
 	    schemer.push(cp_clist_yahoo, "clist_yahoo magenta/blue");
 	    schemer.push(cp_clist_msn, "clist_msn   cyan/blue");
 	    schemer.push(cp_clist_infocard, "clist_infocard  white/blue");
 	    schemer.push(cp_clist_aim, "clist_aim   yellow/blue");
 	    schemer.push(cp_clist_irc, "clist_irc   blue/blue   bold");
+	    schemer.push(cp_clist_jabber, "clist_jabber    red/blue");
+	    schemer.push(cp_clist_rss, "clist_rss    white/blue   bold");
+	    schemer.push(cp_clist_lj, "clist_lj    cyan/blue   bold");
 	    break;
     }
 
@@ -343,8 +412,8 @@ void icqconf::loadsounds() {
 
 	    fo << "# <id>\tid of a contact; can be one of the following" << endl;
 	    fo << "# *\tmeans default sound for all the contacts" << endl;
-	    fo << "# icq_<uin>\tfor icq contacts" << endl;
-	    fo << "# yahoo_<nickname>\tfor yahoo" << endl;
+	    fo << "# icq_<uin>\tfor icq contacts (e.g. icq_123)" << endl;
+	    fo << "# yahoo_<nickname>\tfor yahoo (e.g. yahoo_thekonst)" << endl;
 	    fo << "# .. etc. Similar for the other protocols" << endl << "#" << endl;
 
 	    fo << "# <event>\tcan be: ";
@@ -561,6 +630,10 @@ void icqconf::sethideoffline(bool fho) {
     hideoffline = fho;
 }
 
+void icqconf::setemacs(bool fem) {
+    emacs = fem;
+}
+
 void icqconf::setantispam(bool fas) {
     antispam = fas;
 }
@@ -573,8 +646,34 @@ void icqconf::setaskaway(bool faskaway) {
     askaway = faskaway;
 }
 
-void icqconf::setchatmode(bool fchatmode) {
-    chatmode = fchatmode;
+bool icqconf::getchatmode(protocolname pname) {
+    switch(pname) {
+	case infocard:
+	case rss:
+	    return false;
+    }
+
+    return chatmode[pname];
+}
+
+void icqconf::setchatmode(protocolname pname, bool fchatmode) {
+    chatmode[pname] = fchatmode;
+}
+
+bool icqconf::getentersends(protocolname pname) {
+    return entersends[pname];
+}
+
+void icqconf::setentersends(protocolname pname, bool fentersends) {
+    entersends[pname] = fentersends;
+}
+
+bool icqconf::getrussian(protocolname pname) {
+    return russian[pname];
+}
+
+void icqconf::setrussian(protocolname pname, bool frussian) {
+    russian[pname] = frussian;
 }
 
 void icqconf::openurl(const string &url) {
@@ -585,11 +684,13 @@ void icqconf::openurl(const string &url) {
 	torun.replace(npos, 5, url);
 
     system(torun.c_str());
+
+    face.log(_("+ launched the openurl command"));
 }
 
 string icqconf::getprotocolname(protocolname pname) const {
     static const string ptextnames[protocolname_size] = {
-	"icq", "yahoo", "msn", "aim", "irc", "infocard"
+	"icq", "yahoo", "msn", "aim", "irc", "jabber", "rss", "lj", "infocard"
     };
 
     return ptextnames[pname];
@@ -602,6 +703,9 @@ protocolname icqconf::getprotocolbyletter(char letter) const {
 	case 'i': return irc;
 	case 'm': return msn;
 	case 'n': return infocard;
+	case 'j': return jabber;
+	case 'r': return rss;
+	case 'l': return livejournal;
 	case '0':
 	case '1':
 	case '2':
@@ -619,7 +723,7 @@ protocolname icqconf::getprotocolbyletter(char letter) const {
 
 string icqconf::getprotocolprefix(protocolname pname) const {
     static const string pprefixes[protocolname_size] = {
-	"", "y", "m", "a", "i", "n"
+	"", "y", "m", "a", "i", "j", "r", "l", "n"
     };
 
     return pprefixes[pname];
@@ -648,20 +752,19 @@ void icqconf::savestatus(protocolname pname, imstatus st) {
 
 int icqconf::getprotcolor(protocolname pname) const {
     switch(pname) {
-	case      icq : return getcolor(cp_clist_icq);
-	case    yahoo : return getcolor(cp_clist_yahoo);
+	case icq : return getcolor(cp_clist_icq);
+	case yahoo : return getcolor(cp_clist_yahoo);
 	case infocard : return getcolor(cp_clist_infocard);
-	case      msn : return getcolor(cp_clist_msn);
-	case      aim : return getcolor(cp_clist_aim);
-	case      irc : return getcolor(cp_clist_irc);
-	default       : return getcolor(cp_main_text);
+	case msn : return getcolor(cp_clist_msn);
+	case aim : return getcolor(cp_clist_aim);
+	case irc : return getcolor(cp_clist_irc);
+	case jabber : return getcolor(cp_clist_jabber);
+	case rss : return getcolor(cp_clist_rss);
+	case livejournal : return getcolor(cp_clist_lj);
+	default : return getcolor(cp_main_text);
     }
 }
 
-void icqconf::setrussian(bool frussian) {
-    russian = frussian;
-}
-
 void icqconf::setbidi(bool fbidi) {
     bidi = fbidi;
 #ifdef USE_FRIBIDI
@@ -671,7 +774,7 @@ void icqconf::setbidi(bool fbidi) {
 
 void icqconf::commandline(int argc, char **argv) {
     int i;
-    string event, proto, dest;
+    string event, proto, dest, number;
     char st = 0;
 
     argv0 = argv[0];
@@ -688,6 +791,18 @@ void icqconf::commandline(int argc, char **argv) {
 		if(basedir.substr(basedir.size()-1) != "/") basedir += "/";
 	    }
 
+	} else if((args == "-B") || (args == "--bind")) {
+	    if(argv[++i]) {
+		bindhost = argv[i];
+		cw_setbind(bindhost.c_str());
+	    }
+
+	} else if((args == "-P") || (args == "--http-proxy")) {
+	    if(argv[++i]) {
+//                proxyhost = argv[i];
+//                cw_setproxy(proxyhost.c_str());
+	    }
+
 	} else if((args == "-s") || (args == "--send")) {
 	    if(argv[++i]) event = argv[i];
 
@@ -697,22 +812,41 @@ void icqconf::commandline(int argc, char **argv) {
 	} else if((args == "-t") || (args == "--to")) {
 	    if(argv[++i]) dest = argv[i];
 
+	} else if((args == "-n") || (args == "--number")) {
+	    if(argv[++i]) number = argv[i];
+
 	} else if((args == "-S") || (args == "--status")) {
 	    if(argv[++i])
 	    if(strlen(argv[i]))
 		st = argv[i][0];
 
+	} else if((args == "-v") || (args == "--version")) {
+	    cout << PACKAGE << " " << VERSION << endl
+		<< "Written by Konstantin Klyagin." << endl << endl
+		<< "This is free software; see the source for copying conditions.  There is NO" << endl
+		<< "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE." << endl;
+
+	    exit(0);
+
 	} else {
 	    usage();
 	    exit(0);
 	}
     }
 
-    constructevent(event, proto, dest);
+    if(!number.empty()) {
+	if(event.empty()) event = "sms";
+	if(proto.empty()) proto = "icq";
+	dest = "0";
+    }
+
+    constructevent(event, proto, dest, number);
     externalstatuschange(st, proto);
 }
 
-void icqconf::constructevent(const string &event, const string &proto, const string &dest) const {
+void icqconf::constructevent(const string &event, const string &proto,
+const string &dest, const string &number) const {
+
     imevent *ev = 0;
     imcontact cdest;
     string text, buf;
@@ -766,7 +900,7 @@ void icqconf::constructevent(const string &event, const string &proto, const str
 
 	ev = new imurl(cdest, imevent::outgoing, url, text);
     } else if(event == "sms") {
-	ev = new imsms(cdest, imevent::outgoing, text);
+	ev = new imsms(cdest, imevent::outgoing, text, number);
     } else {
 	cout << _("event sending error: unknown event type") << endl;
 	exit(1);
@@ -781,7 +915,8 @@ void icqconf::constructevent(const string &event, const string &proto, const str
 
 	    char buf[512];
 	    sprintf(buf, _("%s to %s has been put to the queue"),
-		eventnames[ev->gettype()], c->getdesc().totext().c_str());
+		streventname(ev->gettype()),
+		number.empty() ? c->getdesc().totext().c_str() : number.c_str());
 
 	    cout << buf << endl;
 	} else {
@@ -842,19 +977,24 @@ void icqconf::externalstatuschange(char st, const string &proto) const {
 }
 
 void icqconf::usage() const {
-    cout << "Usage: " << argv0 << " [OPTION].." << endl;
+    cout << _("Usage: ") << argv0 << " [OPTION].." << endl;
 
-    cout << endl << "General options:" << endl;
-    cout << "  --ascii, -a              use characters for windows and UI controls" << endl;
-    cout << "  --help                   display this stuff" << endl;
+    cout << endl << _("General options:") << endl;
+    cout << _("  --ascii, -a              use ASCII characters for windows and UI controls") << endl;
+    cout << _("  --basedir, -b <path>     set a custom base directory") << endl;
+    cout << _("  --bind, -B <host/ip>     bind a custom local IP") << endl;
+    cout << _("  --help                   display this stuff") << endl;
+    cout << _("  --version, -v            show the program version info") << endl;
 
-    cout << endl << "Events sending options:" << endl;
-    cout << "  -s, --send <event type>  event type; can be msg, sms or url" << endl;
-    cout << "  -S, --status <status>    change the current IM status" << endl;
-    cout << "  -p, --proto <protocol>   protocol type; can be icq, yahoo, msn, aim or irc" << endl;
-    cout << "  -t, --to <destination>   destination UIN or nick (depends on protocol)" << endl;
+    cout << endl << _("Events sending options:") << endl;
+    cout << _("  -s, --send <event type>  event type; can be msg, sms or url") << endl;
+    cout << _("  -S, --status <status>    change the current IM status") << endl;
+    cout << _("  -p, --proto <protocol>   protocol type; can be icq, yahoo, msn, aim, irc or jabber") << endl;
 
-    cout << endl << "Report bugs to <centericq-bugs@konst.org.ua>." << endl;
+    cout << _("  -t, --to <destination>   destination UIN or nick (depends on protocol)") << endl;
+    cout << _("  -n, --number <phone#>    mobile number to send an event to (sms only)") << endl;
+
+    cout << endl << _("Report bugs to <centericq-bugs@konst.org.ua>.") << endl;
 }
 
 void icqconf::setmakelog(bool slog) {
@@ -865,6 +1005,23 @@ void icqconf::setgroupmode(icqconf::groupmode amode) {
     fgroupmode = amode;
 }
 
+void icqconf::initmultiproto(bool p[], string buf) {
+    string w;
+    protocolname pname;
+
+    for(pname = icq; pname != protocolname_size; (int) pname += 1)
+	p[pname] = buf.empty();
+
+    while(!(w = getword(buf)).empty()) {
+	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	    if(getprotocolname(pname) == w) {
+		p[pname] = true;
+		break;
+	    }
+	}
+    }
+}
+
 void icqconf::setsmtphost(const string &asmtphost) {
     int pos;
 
@@ -887,6 +1044,23 @@ void icqconf::setsmtphost(const string &asmtphost) {
     }
 }
 
+void icqconf::sethttpproxyhost(const string &ahttpproxyhost) {
+    int pos;
+
+    httpproxyhost = "";
+    httpproxyport = 0;
+
+    if(!ahttpproxyhost.empty()) {
+	if((pos = ahttpproxyhost.find(":")) != -1) {
+	    httpproxyhost = ahttpproxyhost.substr(0, pos);
+	    httpproxyport = atol(ahttpproxyhost.substr(pos+1).c_str());
+	} else {
+	    httpproxyhost = ahttpproxyhost;
+	    httpproxyport = 8080;
+	}
+    }
+}
+
 string icqconf::getsmtphost() const {
     return smtphost.empty() ? "localhost" : smtphost;
 }
@@ -895,6 +1069,14 @@ unsigned int icqconf::getsmtpport() const {
     return smtpport ? smtpport : 25;
 }
 
+string icqconf::gethttpproxyhost() const {
+    return httpproxyhost;
+}
+
+unsigned int icqconf::gethttpproxyport() const {
+    return httpproxyport ? httpproxyport : 8080;
+}
+
 void icqconf::checkdiskspace() {
     fenoughdiskspace = true;
 
@@ -939,9 +1121,9 @@ void icqconf::imaccount::write(ofstream &f) {
 	f << endl;
     }
 
-    for(ia = additional.begin(); ia != additional.end(); ++ia) {
-	f << prefix << ia->first << "\t" << ia->second << endl;
-    }
+    for(ia = additional.begin(); ia != additional.end(); ++ia)
+	if(!ia->first.empty() && !ia->second.empty())
+	    f << prefix << ia->first << "\t" << ia->second << endl;
 }
 
 void icqconf::imaccount::read(const string &spec) {
@@ -966,6 +1148,11 @@ void icqconf::imaccount::read(const string &spec) {
 	    if(pname == icq)
 	    if(server == "icq.mirabilis.com")
 		server = "";
+
+	    if(pname == yahoo)
+	    if(server == "scs.yahoo.com")
+		server = "";
+
 	} else {
 	    additional[spname] = buf;
 	}
