@@ -1,13 +1,14 @@
 #ifndef __ICQCONF_H__
 #define __ICQCONF_H__
 
-#include <unistd.h>
+#include "icqcommon.h"
+
 #include <stdio.h>
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <fcntl.h>
 #include <dirent.h>
 
+#include "config.h"
+
 #include "kkstrtext.h"
 #include "kkfsys.h"
 #include "conscommon.h"
@@ -28,12 +29,17 @@ enum cicq_colorpairs {
     cp_main_selected,
     cp_main_highlight,
     cp_main_frame,
+    cp_main_history_incoming,
+    cp_main_history_outgoing,
     cp_clist_icq,
     cp_clist_yahoo,
     cp_clist_infocard,
     cp_clist_msn,
     cp_clist_aim,
-    cp_clist_irc
+    cp_clist_irc,
+    cp_clist_jabber,
+    cp_clist_rss,
+    cp_clist_lj
 };
 
 class icqconf {
@@ -42,6 +48,11 @@ class icqconf {
 	enum regcolor { rcdark, rcblue, rcdontchange };
 	enum groupmode { group1, group2, nogroups };
 
+	struct imserver {
+	    string server;
+	    int port, secureport;
+	};
+
 	struct imaccount {
 	    imaccount();
 	    imaccount(protocolname apname);
@@ -60,19 +71,25 @@ class icqconf {
 	    bool operator != (protocolname apname) const;
 	};
 
+	static imserver defservers[];
+
     protected:
 	vector<icqgroup> groups;
 	vector<imaccount> accounts;
 
-	unsigned short socksport, smtpport;
+	unsigned short socksport, smtpport, httpproxyport;
 
-	int autoaway, autona;
+	int autoaway, autona, ptpmin, ptpmax;
 
-	bool hideoffline, quote, savepwd, antispam, mailcheck, russian,
-	    makelog, fenoughdiskspace, askaway, chatmode, bidi;
+	bool hideoffline, quote, savepwd, antispam, mailcheck,
+	    makelog, fenoughdiskspace, askaway, bidi, logtimestamps,
+	    logonline, emacs;
+
+	bool chatmode[protocolname_size], russian[protocolname_size],
+	    entersends[protocolname_size];
 
 	string sockshost, socksuser, sockspass, openurlcommand, basedir,
-	    argv0, smtphost;
+	    argv0, smtphost, bindhost, httpproxyhost, defcharset;
 
 	regsound rs;
 	regcolor rc;
@@ -81,13 +98,13 @@ class icqconf {
 	colorschemer<cicq_colorpairs> schemer;
 
 	void loadmainconfig();
-
 	void usage() const;
 
 	void constructevent(const string &event, const string &proto,
-	    const string &dest) const;
+	    const string &dest, const string &number) const;
 
 	void externalstatuschange(char st, const string &proto) const;
+	void initmultiproto(bool p[], string buf);
 
     public:
 	icqconf();
@@ -112,6 +129,9 @@ class icqconf {
 	regsound getregsound() const;
 	void setregsound(regsound s);
 
+	bool getemacs() const { return emacs; }
+	void setemacs(bool fem);
+
 	bool gethideoffline() const { return hideoffline; }
 	void sethideoffline(bool fho);
 
@@ -133,8 +153,8 @@ class icqconf {
 	bool getmakelog() const { return makelog; }
 	void setmakelog(bool slog);
 
-	bool getrussian() const { return russian; }
-	void setrussian(bool frussian);
+	bool getrussian(protocolname pname);
+	void setrussian(protocolname pname, bool frussian);
 
 	bool getbidi() const { return bidi; }
 	void setbidi(bool fbidi);
@@ -142,12 +162,15 @@ class icqconf {
 	bool getaskaway() const { return askaway; }
 	void setaskaway(bool faskaway);
 
-	bool getchatmode() const { return chatmode; }
-	void setchatmode(bool fchatmode);
+	bool getchatmode(protocolname pname);
+	void setchatmode(protocolname pname, bool fchatmode);
+
+	bool getentersends(protocolname pname);
+	void setentersends(protocolname pname, bool fentersends);
 
 	groupmode getgroupmode() const { return fgroupmode; }
 	void setgroupmode(groupmode amode);
- 
+
 	string getsockshost() const;
 	unsigned int getsocksport() const;
 	void setsockshost(const string &nsockshost);
@@ -156,9 +179,16 @@ class icqconf {
 	unsigned int getsmtpport() const;
 	void setsmtphost(const string &asmtphost);
 
+	string gethttpproxyhost() const;
+	unsigned int gethttpproxyport() const;
+	void sethttpproxyhost(const string &ahttpproxyhost);
+
 	void getsocksuser(string &name, string &pass) const;
 	void setsocksuser(const string &name, const string &pass);
 
+	void getpeertopeer(int &min, int &max) const { min = ptpmin; max = ptpmax; }
+	void setpeertopeer(int min, int max) { ptpmin = min; ptpmax = max; }
+
 	void openurl(const string &url);
 
 	imstatus getstatus(protocolname pname);
@@ -169,7 +199,7 @@ class icqconf {
 	string getprotocolprefix(protocolname pname) const;
 
 	int getouridcount() const { return accounts.size(); }
-	imaccount getourid(protocolname pname);
+	imaccount getourid(protocolname pname) const;
 	void setourid(const imaccount &im);
 
 	string getawaymsg(protocolname pname) const;
@@ -182,6 +212,17 @@ class icqconf {
 
 	bool enoughdiskspace() const { return fenoughdiskspace; }
 	void checkdiskspace();
+
+	string getbindhost() const { return bindhost; }
+
+	void getlogoptions(bool &flogtimestamps, bool &flogonline)
+	    { flogtimestamps = logtimestamps, flogonline = logonline; }
+
+	void setlogoptions(bool flogtimestamps, bool flogonline)
+	    { logtimestamps = flogtimestamps, logonline = flogonline; }
+
+	const char *getdefcharset() const { return defcharset.c_str(); }
+	void setdefcharset(const string &adefcharset) { defcharset = adefcharset; }
 };
 
 extern icqconf conf;
