@@ -1,9 +1,9 @@
 /*
 *
-* centericq single icq contact class
-* $Id: icqcontact.cc,v 1.60 2002/08/16 16:48:26 konst Exp $
+* centericq single IM contact class
+* $Id: icqcontact.cc,v 1.84 2003/10/01 23:01:52 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,13 +25,14 @@
 #include "icqcontacts.h"
 #include "icqgroups.h"
 #include "icqconf.h"
-#include "centericq.h"
 #include "icqface.h"
 #include "abstracthook.h"
 #include "imexternal.h"
+#include "eventmanager.h"
 
 #include <time.h>
 #include <libicq2000/userinfohelpers.h>
+#include <strstream>
 
 icqcontact::icqcontact(const imcontact adesc) {
     string fname, tname;
@@ -39,7 +40,7 @@ icqcontact::icqcontact(const imcontact adesc) {
     int i;
 
     clear();
-    nmsgs = lastread = fhistoffset = 0;
+    lastread = fhistoffset = 0;
     status = offline;
     finlist = true;
     congratulated = false;
@@ -51,8 +52,9 @@ icqcontact::icqcontact(const imcontact adesc) {
 
     switch(cdesc.pname) {
 	case infocard:
+	case rss:
 	    if(!cdesc.uin) {
-		fname = conf.getdirname() + "n";
+		fname = conf.getdirname() + conf.getprotocolprefix(cdesc.pname);
 
 		for(i = 1; ; i++) {
 		    tname = fname + i2str(i);
@@ -61,8 +63,6 @@ icqcontact::icqcontact(const imcontact adesc) {
 
 		cdesc.uin = i;
 	    }
-	    load();
-	    break;
 
 	default:
 	    load();
@@ -91,23 +91,13 @@ string icqcontact::getdirname() const {
     ret = conf.getdirname();
 
     switch(cdesc.pname) {
-	case infocard:
-	    ret += "n" + i2str(cdesc.uin);
-	    break;
 	case icq:
-	    ret += i2str(cdesc.uin);
+	case infocard:
+	case rss:
+	    ret += conf.getprotocolprefix(cdesc.pname) + i2str(cdesc.uin);
 	    break;
-	case yahoo:
-	    ret += "y" + cdesc.nickname;
-	    break;
-	case msn:
-	    ret += "m" + cdesc.nickname;
-	    break;
-	case aim:
-	    ret += "a" + cdesc.nickname;
-	    break;
-	case irc:
-	    ret += "i" + cdesc.nickname;
+	default:
+	    ret += conf.getprotocolprefix(cdesc.pname) + cdesc.nickname;
 	    break;
     }
 
@@ -116,39 +106,53 @@ string icqcontact::getdirname() const {
 }
 
 void icqcontact::clear() {
-    nmsgs = fupdated = groupid = fhistoffset = 0;
+    fupdated = groupid = fhistoffset = lasttyping = 0;
     finlist = true;
+    modified = false;
     cdesc = contactroot;
 
     binfo = basicinfo();
     minfo = moreinfo();
     winfo = workinfo();
+    rinfo = reginfo();
 
     interests.clear();
     background.clear();
 
-    nick = dispnick = about = "";
+    nick = about = dispnick = postponed = lastip = "";
     lastseen = 0;
 }
 
 void icqcontact::save() {
-    string tname;
     ofstream f;
 
-    if(conf.enoughdiskspace()) {
-	mkdir(getdirname().c_str(), S_IREAD | S_IWRITE | S_IEXEC);
+    if(cdesc == contactroot)
+	return;
+
+    string lrname = getdirname() + "lastread";
+    string infoname = getdirname() + "info";
+    string aboutname = getdirname() + "about";
+
+    string dname = getdirname();
+    dname.erase(dname.size()-1);
+
+    modified = modified
+	|| access(lrname.c_str(), F_OK)
+	|| access(infoname.c_str(), F_OK)
+	|| access(aboutname.c_str(), F_OK);
+
+    if(modified && conf.enoughdiskspace()) {
+	mkdir(dname.c_str(), S_IRUSR | S_IWUSR | S_IXUSR);
 
 	if(!access(getdirname().c_str(), W_OK)) {
-	    tname = getdirname() + "lastread";
-	    f.open(tname.c_str());
+	    f.open(lrname.c_str());
 	    if(f.is_open()) {
 		f << lastread << endl;
 		f.close();
 		f.clear();
 	    }
 
-	    tname = getdirname() + "info";
-	    f.open(tname.c_str());
+	    f.open(infoname.c_str());
 	    if(f.is_open()) {
 		string options;
 		if(binfo.requiresauth) options += "a";
@@ -211,8 +215,7 @@ void icqcontact::save() {
 		f.clear();
 	    }
 
-	    tname = getdirname() + "about";
-	    f.open(tname.c_str());
+	    f.open(aboutname.c_str());
 	    if(f.is_open()) {
 		f << about;
 		f.close();
@@ -220,11 +223,12 @@ void icqcontact::save() {
 	    }
 
 	    if(!finlist) {
-		tname = getdirname() + "excluded";
-		f.open(tname.c_str());
+		f.open((getdirname() + "excluded").c_str());
 		if(f.is_open()) f.close();
 	    }
 	}
+
+	modified = false;
     }
 }
 
@@ -301,6 +305,11 @@ void icqcontact::load() {
 	    }
 	}
 	fclose(f);
+
+    } else {
+	if(cdesc.uin)
+	    nick = i2str(cdesc.uin);
+
     }
 
     if(f = fopen((fn = tname + "/about").c_str(), "r")) {
@@ -323,12 +332,11 @@ void icqcontact::load() {
     if(!isbirthday())
 	unlink((tname + "/congratulated").c_str());
 
-    if(nick.empty()) {
-	if(cdesc.uin) nick = i2str(cdesc.uin);
-	else nick = cdesc.nickname;
-    }
+    if(nick.empty())
+	nick = cdesc.nickname;
 
-    if(dispnick.empty()) dispnick = nick;
+    if(dispnick.empty())
+	dispnick = nick;
 
     if(conf.getgroupmode() != icqconf::nogroups)
     if(find(groups.begin(), groups.end(), groupid) == groups.end()) {
@@ -382,7 +390,12 @@ void icqcontact::excludefromlist() {
 }
 
 void icqcontact::includeintolist() {
-    gethook(cdesc.pname).sendnewuser(cdesc);
+    if(cdesc.pname != icq)
+	status = offline;
+
+    if(!ischannel(cdesc))
+	gethook(cdesc.pname).sendnewuser(cdesc);
+
     unlink((getdirname() + "excluded").c_str());
     finlist = true;
 }
@@ -392,40 +405,43 @@ bool icqcontact::inlist() const {
 }
 
 void icqcontact::scanhistory() {
-    string fn = getdirname() + "history";
-    char buf[512];
-    int line, pos;
+    string fn = getdirname() + "history", block;
+    char buf[65];
+    int pos, backstep, r;
     FILE *f = fopen(fn.c_str(), "r");
-    bool docount;
+    struct stat st;
+    strstream evdata;
 
-    setmsgcount(0);
+    sethasevents(false);
 
     if(f) {
-//      fseek(f, gethistoffset(), SEEK_SET);
+	fseek(f, 0, SEEK_END);
 	sethistoffset(0);
 	pos = 0;
 
-	while(!feof(f)) {
-	    freads(f, buf, 512);
+	while(ftell(f)) {
+	    backstep = 64;
+	    if(ftell(f) < backstep) backstep = ftell(f);
 
-	    if((string) buf == "\f") {
-		line = 0;
-		pos = ftell(f);
-	    } else {
-		switch(line) {
-		    case 1:
-			docount = ((string) buf == "IN");
-			break;
-		    case 4:
-			if(docount && (atol(buf) > lastread)) {
-			    nmsgs++;
-			    if(!gethistoffset()) sethistoffset(pos);
-			}
-			break;
+	    if(fseek(f, -backstep, SEEK_CUR)) break;
+	    if((r = fread(buf, 1, backstep, f)) <= 0) break;
+	    if(fseek(f, -backstep, SEEK_CUR)) break;
+
+	    buf[r] = 0;
+	    block.insert(0, buf);
+
+	    if((r = block.find("\f\nIN\n")) != -1) {
+		if(!stat(fn.c_str(), &st)) {
+		    pos = st.st_size-block.size()+r;
 		}
-	    }
 
-	    line++;
+		block.erase(0, r+2);
+		evdata << block;
+
+		for(r = 0; (r < 4) && getline(evdata, block); r++);
+		if(r == 4) sethasevents(strtoul(block.c_str(), 0, 0) > lastread);
+		break;
+	    }
 	}
 
 	fclose(f);
@@ -435,94 +451,90 @@ void icqcontact::scanhistory() {
 
 void icqcontact::setstatus(imstatus fstatus) {
     if(status != fstatus) {
-	if(status == offline) {
-	    external.exec(imrawevent(imevent::online, cdesc, imevent::incoming));
-	    playsound(imevent::online);
+	if(!ischannel(cdesc)) {
+	    if(status == offline) {
+		external.exec(imrawevent(imevent::online, cdesc, imevent::incoming));
+		playsound(imevent::online);
+	    }
+
+	} else {
+	    abstracthook &h = gethook(cdesc.pname);
+	    if(fstatus == offline) h.removeuser(cdesc);
+	    else h.sendnewuser(cdesc);
+
 	}
 
+	setlastseen();
 	status = fstatus;
 	face.relaxedupdate();
     }
 }
 
+void icqcontact::setdesc(const imcontact &ic) {
+    string dir = getdirname();
+    cdesc = ic;
+    rename(dir.c_str(), getdirname().c_str());
+}
+
 void icqcontact::setnick(const string &fnick) {
-    string dir;
-
     nick = fnick;
-
-    if(!cdesc.nickname.empty()) {
-	dir = getdirname();
-	cdesc.nickname = fnick;
-	rename(dir.c_str(), getdirname().c_str());
-    }
+    modified = true;
+    fupdated++;
 }
 
 void icqcontact::setdispnick(const string &fnick) {
     dispnick = fnick;
+    modified = true;
 }
 
 void icqcontact::setlastread(time_t flastread) {
     lastread = flastread;
     scanhistory();
+    modified = true;
 }
 
 void icqcontact::unsetupdated() {
     fupdated = 0;
 }
 
-void icqcontact::setmsgcount(int n) {
-    nmsgs = n;
-    face.relaxedupdate();
-}
-
 void icqcontact::setbasicinfo(const basicinfo &ainfo) {
     binfo = ainfo;
     fupdated++;
+    modified = true;
 }
 
 void icqcontact::setmoreinfo(const moreinfo &ainfo) {
     minfo = ainfo;
     fupdated++;
+    modified = true;
 }
 
 void icqcontact::setworkinfo(const workinfo &ainfo) {
     winfo = ainfo;
     fupdated++;
+    modified = true;
+}
+
+void icqcontact::setreginfo(const reginfo &arinfo) {
+    rinfo = arinfo;
 }
 
 void icqcontact::setinterests(const vector<string> &ainterests) {
     interests = ainterests;
     fupdated++;
+    modified = true;
 }
 
 void icqcontact::setbackground(const vector<string> &abackground) {
     background = abackground;
     fupdated++;
+    modified = true;
 }
 
 void icqcontact::setabout(const string &data) {
     about = data;
     fupdated++;
-}
-
-const icqcontact::basicinfo &icqcontact::getbasicinfo() const {
-    return binfo;
-}
-
-const icqcontact::moreinfo &icqcontact::getmoreinfo() const {
-    return minfo;
-}
-
-const icqcontact::workinfo &icqcontact::getworkinfo() const {
-    return winfo;
-}
-
-const vector<string> &icqcontact::getinterests() const {
-    return interests;
-}
-
-const vector<string> &icqcontact::getbackground() const {
-    return background;
+    modified = true;
 }
 
 void icqcontact::setsound(imevent::imeventtype event, const string &sf) {
@@ -542,14 +554,14 @@ void icqcontact::playsound(imevent::imeventtype event) const {
 
 	    if(sf.substr(1) == "spk1") {
 		for(i = 0; i < 3; i++) {
-		    if(i) delay(90);
+		    if(i) usleep(90000);
 		    setbeep((i+1)*100, 60);
 		    printf("\a");
 		    fflush(stdout);
 		}
 	    } else if(sf.substr(1) == "spk2") {
 		for(i = 0; i < 2; i++) {
-		    if(i) delay(90);
+		    if(i) usleep(90000);
 		    setbeep((i+1)*300, 60);
 		    printf("\a");
 		    fflush(stdout);
@@ -559,21 +571,21 @@ void icqcontact::playsound(imevent::imeventtype event) const {
 		    setbeep((i+1)*200, 60-i*10);
 		    printf("\a");
 		    fflush(stdout);
-		    delay(90-i*10);
+		    usleep(90000-i*10000);
 		}
 	    } else if(sf.substr(1) == "spk4") {
 		for(i = 0; i < 4; i++) {
 		    setbeep((i+1)*400, 60);
 		    printf("\a");
 		    fflush(stdout);
-		    delay(90);
+		    usleep(90000);
 		}
 	    } else if(sf.substr(1) == "spk5") {
 		for(i = 0; i < 4; i++) {
 		    setbeep((i+1)*250, 60+i);
 		    printf("\a");
 		    fflush(stdout);
-		    delay(90-i*5);
+		    usleep(90000-i*5000);
 		}
 	    }
 	} else {
@@ -596,6 +608,7 @@ void icqcontact::playsound(imevent::imeventtype event) const {
 void icqcontact::setlastip(const string &flastip) {
     lastip = flastip;
     fupdated++;
+    modified = true;
 }
 
 string icqcontact::getabout() const {
@@ -614,10 +627,6 @@ imstatus icqcontact::getstatus() const {
     return status;
 }
 
-int icqcontact::getmsgcount() const {
-    return nmsgs;
-}
-
 string icqcontact::getnick() const {
     return nick;
 }
@@ -633,6 +642,7 @@ int icqcontact::updated() const {
 void icqcontact::setlastseen() {
     time(&lastseen);
     fupdated++;
+    modified = true;
 }
 
 time_t icqcontact::getlastseen() const {
@@ -640,7 +650,11 @@ time_t icqcontact::getlastseen() const {
 }
 
 char icqcontact::getshortstatus() const {
-    return imstatus2char[status];
+    if(status >= offline && status < imstatus_size) {
+	return imstatus2char[status];
+    } else {
+	return imstatus2char[offline];
+    }
 }
 
 bool icqcontact::operator > (const icqcontact &acontact) const {
@@ -663,7 +677,7 @@ string icqcontact::getpostponed() const {
 
 void icqcontact::setgroupid(int agroupid) {
     groupid = agroupid;
-    save();
+    modified = true;
 }
 
 int icqcontact::getgroupid() const {
@@ -684,14 +698,13 @@ void icqcontact::sethistoffset(int aoffset) {
 
 void icqcontact::remindbirthday(bool r) {
     string tname = getdirname() + "congratulated";
-    ofstream f;
 
     if(!congratulated && r) {
 	congratulated = !access(tname.c_str(), F_OK);
 
 	if(!congratulated) {
-	    f.open(tname.c_str());
-	    if(f.is_open()) f.close();
+	    ofstream f(tname.c_str());
+	    if(f.is_open()) f.close(), f.clear();
 
 	    em.store(imnotification(getdesc(), _("The user has a birthday today")));
 	    congratulated = true;
