@@ -6,8 +6,6 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
-#include "kkstrtext.h"
-#include "kkfsys.h"
 #include "cmenus.h"
 
 #include "imcontact.h"
@@ -58,13 +56,18 @@ class icqcontact {
 	    unsigned short country;
 	};
 
+	struct reginfo {
+	    string service, password;
+	    vector<pair<string, string> > params;
+	};
+
     protected:
 	imcontact cdesc;
 	imstatus status;
 
-	int nmsgs, fupdated, groupid, fhistoffset;
-	bool finlist, congratulated;
-	time_t lastread, lastseen;
+	int fupdated, groupid, fhistoffset;
+	bool finlist, congratulated, modified, fhasevents;
+	time_t lastread, lastseen, lasttyping;
 
 	string sound[imevent::imeventtype_size];
 	string nick, about, dispnick, postponed, lastip;
@@ -72,6 +75,7 @@ class icqcontact {
 	basicinfo binfo;
 	moreinfo minfo;
 	workinfo winfo;
+	reginfo rinfo;
 	vector<string> interests, background;
 
 	void scanhistory();
@@ -91,16 +95,19 @@ class icqcontact {
 	void setbasicinfo(const basicinfo &ainfo);
 	void setmoreinfo(const moreinfo &ainfo);
 	void setworkinfo(const workinfo &ainfo);
+	void setreginfo(const reginfo &ainfo);
 	void setinterests(const vector<string> &ainterests);
 	void setbackground(const vector<string> &abackground);
 	void setabout(const string &data);
 	void setlastip(const string &flastip);
 
-	const basicinfo &getbasicinfo() const;
-	const moreinfo &getmoreinfo() const;
-	const workinfo &getworkinfo() const;
-	const vector<string> &getinterests() const;
-	const vector<string> &getbackground() const;
+	const basicinfo &getbasicinfo() const { return binfo; }
+	const moreinfo &getmoreinfo() const { return minfo; }
+	const workinfo &getworkinfo() const { return winfo; }
+	const reginfo &getreginfo() const { return rinfo; }
+	const vector<string> &getinterests() const { return interests; }
+	const vector<string> &getbackground() const { return background; }
+
 	string getabout() const;
 	string getlastip() const;
 
@@ -108,14 +115,15 @@ class icqcontact {
 	time_t getlastseen() const;
 	
 	imstatus getstatus() const;
-	int getmsgcount() const;
+
+	bool hasevents() const { return fhasevents; }
+	void sethasevents(bool n) { fhasevents = n; }
 
 	string getnick() const;
 	string getdispnick() const;
 
 	int updated() const;
 	void unsetupdated();
-	void setmsgcount(int n);
 
 	void setsound(imevent::imeventtype event, const string &sf);
 	void playsound(imevent::imeventtype event) const;
@@ -143,10 +151,14 @@ class icqcontact {
 	int getgroupid() const;
 
 	const imcontact getdesc() const;
+	void setdesc(const imcontact &ic);
 
 	void remindbirthday(bool r);
 
 	bool operator > (const icqcontact &acontact) const;
+
+	time_t getlasttyping() const { return lasttyping; }
+	void setlasttyping(time_t t) { lasttyping = t; }
 };
 
 #endif
