@@ -1,9 +1,9 @@
 /*
 *
 * centericq contact list class
-* $Id: icqcontacts.cc,v 1.38 2002/08/19 08:25:18 konst Exp $
+* $Id: icqcontacts.cc,v 1.46 2003/09/30 11:38:41 konst Exp $
 *
-* Copyright (C) 2001 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -43,10 +43,7 @@ icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist) {
 	    c->setdispnick(i2str(cinfo.uin));
 	    break;
 
-	case yahoo:
-	case msn:
-	case aim:
-	case irc:
+	default:
 	    c->setnick(cinfo.nickname);
 	    c->setdispnick(cinfo.nickname);
 	    break;
@@ -58,7 +55,11 @@ icqcontact *icqcontacts::addnew(const imcontact &cinfo, bool notinlist) {
     if(c)
     if(notinlist) {
 	c->excludefromlist();
-	gethook(cinfo.pname).sendnewuser(cinfo);
+
+	abstracthook &h = gethook(cinfo.pname);
+	if(h.getCapabs().count(hookcapab::cltemporary))
+	    h.sendnewuser(cinfo);
+
     } else {
 	c->includeintolist();
     }
@@ -93,13 +94,16 @@ void icqcontacts::load() {
 			break;
 
 		    case infocard:
+		    case rss:
 			c = new icqcontact(imcontact(atol(ent->d_name+1), pname));
 			break;
 
-		    case yahoo:
-		    case msn:
-		    case aim:
-		    case irc:
+		    case protocolname_size:
+			// Strange dir, doesn't belong to any of the
+			// supported protocols.
+			break;
+
+		    default:
 			c = new icqcontact(imcontact(ent->d_name+1, pname));
 			c->setnick(ent->d_name+1);
 			break;
@@ -114,46 +118,19 @@ void icqcontacts::load() {
 	closedir(d);
     }
 
-    checkdefault();
-
     if(!get(contactroot)) {
 	add(new icqcontact(contactroot));
     }
 }
 
-void icqcontacts::checkdefault() {
-    protocolname pname;
-    icqcontact *c;
-    int i;
-    bool found;
-
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	if(!conf.getourid(pname).empty()) {
-	    for(i = 0, found = false; i < count && !found; i++) {
-		c = (icqcontact *) at(i);
-		if(c->getdesc() != contactroot) {
-		    found = (c->getdesc().pname == pname);
-		}
-	    }
-
-	    if(!found)
-	    switch(pname) {
-		case icq:
-		    addnew(imcontact(17502151, pname), false);
-		    break;
-	    }
-	}
-    }
-}
-
-void icqcontacts::save() {
+void icqcontacts::save(bool removenil) {
     int i;
 
     for(i = 0; i < count; i++) {
 	icqcontact *c = (icqcontact *) at(i);
 
 	if(c->inlist()) c->save(); else
-	if(!c->getmsgcount()) {
+	if(removenil && !c->hasevents()) {
 	    string fname = c->getdirname() + "offline";
 	    if(access(fname.c_str(), F_OK)) c->remove();
 	}
@@ -212,7 +189,7 @@ int icqcontacts::clistsort(void *p1, void *p2) {
 
     if(makegroup)
     if(!strchr("!N", s1) && !strchr("!N", s2))
-    if(!c1->getmsgcount() && !c2->getmsgcount()) {
+    if(!c1->hasevents() && !c2->hasevents()) {
 	if(c1->getgroupid() > c2->getgroupid()) return -1; else
 	if(c1->getgroupid() < c2->getgroupid()) return 1;
     }
