@@ -20,11 +20,12 @@
 *
 */
 
-#define SORT_CONTACTS   "#fiodcan_!N"
+#define SORT_CONTACTS   "#fodcan_!N"
 
 #define SORTCHAR(c) ( \
-    c->getmsgcount() ? '#' : \
+    c->hasevents() ? '#' : \
     !c->inlist() ? '!' : \
+    c->getstatus() == invisible ? 'o' : \
     c->getshortstatus() \
 )
 
@@ -44,10 +45,9 @@ class icqcontacts: public linkedlist {
 
 	void remove(const imcontact &adesc);
 	void load();
-	void save();
+	void save(bool removenil = true);
 	void order();
 	void rearrange();
-	void checkdefault();
 
 	void setoffline(protocolname pname);
 
