@@ -1,3 +1,27 @@
+/*
+*
+* centericq IM contact basic info class
+* $Id: imcontact.cc,v 1.19 2003/07/24 17:39:05 konst Exp $
+*
+* Copyright (C) 2002 by Konstantin Klyagin <konst@konst.org.ua>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or (at
+* your option) any later version.
+*
+* This program is distributed in the hope that it will be useful, but
+* WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+* General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+* USA
+*
+*/
+
 #include "imcontact.h"
 #include "icqconf.h"
 
@@ -22,17 +46,14 @@ imcontact::imcontact(const icqcontact *c) {
 }
 
 bool imcontact::operator == (const imcontact &ainfo) const {
-    int k;
-    string nick1, nick2;
     bool r = (ainfo.uin == uin) && (ainfo.pname == pname);
 
+    if(r)
     switch(pname) {
 	case irc:
 	case yahoo:
-	    for(k = 0; k < ainfo.nickname.size(); k++) nick1 += toupper(ainfo.nickname[k]);
-	    for(k = 0; k < nickname.size(); k++) nick2 += toupper(nickname[k]);
-
-	    r = r & (nick1 == nick2);
+	case jabber:
+	    r = r & (up(ainfo.nickname) == up(nickname));
 	    break;
 
 	default:
@@ -65,15 +86,10 @@ string imcontact::totext() const {
     if(*this == contactroot) {
 	r = "[self]";
     } else {
-	switch(pname) {
-	    case icq:
-	    case infocard:
-		r = "[" + conf.getprotocolname(pname) + "] " + i2str(uin);
-		break;
-
-	    default:
-		r = "[" + conf.getprotocolname(pname) + "] " + nickname;
-		break;
+	if(uin) {
+	    r = "[" + conf.getprotocolname(pname) + "] " + i2str(uin);
+	} else {
+	    r = "[" + conf.getprotocolname(pname) + "] " + nickname;
 	}
     }
 
