@@ -4,12 +4,14 @@
 #include "icqcommon.h"
 
 enum protocolname {
-    proto_all = -1,
     icq = 0,
     yahoo,
     msn,
     aim,
     irc,
+    jabber,
+    rss,
+    livejournal,
     infocard,
 
     protocolname_size
@@ -24,23 +26,11 @@ enum imstatus {
     occupied,
     notavail,
     away,
-/*
-    berightback,
-    outtolunch,
-    steppedout,
-    busy,
-    onphone,
-    notathome,
-    notatdesk,
-    notinoffice,
-    onvacation,
-*/
     imstatus_size
 };
 
 static char imstatus2char[imstatus_size] = {
-    '_', 'o', 'i', 'f', 'd', 'c', 'n', 'a'/*, 'a',
-    'a', 'a', 'c', 'a', 'n', 'n', 'n', 'n'*/
+    '_', 'o', 'i', 'f', 'd', 'c', 'n', 'a'
 };
 
 class icqcontact;
@@ -67,4 +57,6 @@ struct imcontact {
 
 extern imcontact contactroot;
 
+bool ischannel(const imcontact &cont);
+
 #endif
