@@ -1,9 +1,35 @@
+/*
+*
+* centericq protocol specific user interface related routines
+* $Id: imcontroller.cc,v 1.46 2003/07/07 18:50:59 konst Exp $
+*
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or (at
+* your option) any later version.
+*
+* This program is distributed in the hope that it will be useful, but
+* WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+* General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+* USA
+*
+*/
+
 #include "imcontroller.h"
 #include "icqface.h"
 #include "icqhook.h"
 #include "yahoohook.h"
 #include "aimhook.h"
 #include "irchook.h"
+#include "msnhook.h"
+#include "jabberhook.h"
 #include "icqcontacts.h"
 #include "eventmanager.h"
 
@@ -17,7 +43,7 @@ imcontroller::imcontroller() {
 imcontroller::~imcontroller() {
 }
 
-bool imcontroller::icqregdialog() {
+bool imcontroller::regdialog(protocolname pname) {
     bool finished, success;
     int n, i, b;
     string pcheck;
@@ -27,11 +53,16 @@ bool imcontroller::icqregdialog() {
     ruin = 0;
     finished = success = false;
     rnick = rfname = rlname = remail = rpasswd = "";
-    rserver = "login.icq.com:5190";
+
+    rserver = icqconf::defservers[pname].server + ":" +
+	i2str(icqconf::defservers[pname].port);
 
     db.setwindow(new textwindow(0, 0, face.sizeDlg.width, face.sizeDlg.height,
-	clr(cp_dialog_frame), TW_CENTERED, clr(cp_dialog_highlight),
-	_(" Register on ICQ network ")));
+	clr(cp_dialog_frame), TW_CENTERED));
+
+    db.getwindow()->set_titlef(clr(cp_dialog_highlight),
+	_(" Register on the %s network "),
+	conf.getprotocolname(pname).c_str());
 
     db.settree(new treeview(clr(cp_dialog_text), clr(cp_dialog_selected),
 	clr(cp_dialog_highlight), clr(cp_dialog_text)));
@@ -47,6 +78,7 @@ bool imcontroller::icqregdialog() {
 	
 	i = db.gettree()->addnode(_(" Details "));
 	t.addleaff(i, 0, 4, _(" Nickname : %s "), rnick.c_str());
+
 	t.addleaff(i, 0, 5, _(" E-Mail : %s "), remail.c_str());
 	t.addleaff(i, 0, 6, _(" First name : %s "), rfname.c_str());
 	t.addleaff(i, 0, 7, _(" Last name : %s "), rlname.c_str());
@@ -94,7 +126,7 @@ bool imcontroller::icqregdialog() {
 bool imcontroller::icqregistration(icqconf::imaccount &account) {
     bool fin, success;
 
-    if(success = icqregdialog()) {
+    if(success = regdialog(account.pname)) {
 	unlink((conf.getdirname() + "icq-infoset").c_str());
 	face.progress.show(_(" Registration progress "));
 
@@ -149,6 +181,61 @@ bool imcontroller::icqregistration(icqconf::imaccount &account) {
     return success;
 }
 
+bool imcontroller::jabberregistration(icqconf::imaccount &account) {
+#ifdef BUILD_JABBER
+    bool success;
+    string err;
+    int pos;
+
+    if(success = regdialog(account.pname)) {
+	unlink((conf.getdirname() + "jabber-infoset").c_str());
+	face.progress.show(_(" Registration progress "));
+
+	face.progress.log(_("Trying to register %s at %s ..."),
+	    rnick.c_str(), rserver.c_str());
+
+	if(success = jhook.regnick(rnick, rpasswd, rserver, err)) {
+	    account.nickname = rnick;
+	    account.password = rpasswd;
+	    account.server = rserver;
+
+	    if((pos = account.server.find(":")) != -1) {
+		account.port = atoi(account.server.substr(pos+1).c_str());
+		account.server.erase(pos);
+	    } else {
+		account.port = icqconf::defservers[jabber].port;
+	    }
+
+	    face.progress.log(_("The Jabber ID was successfully registered"));
+
+	    conf.checkdir();
+	    ofstream f((conf.getdirname() + "jabber-infoset").c_str());
+	    if(f.is_open()) {
+		f << rnick << endl <<
+		    remail << endl <<
+		    rfname << endl <<
+		    rlname << endl;
+		f.close();
+	    }
+
+	} else {
+	    string msgerr = _("Failed");
+	    if(!err.empty()) msgerr += (string) ": " + err;
+	    face.progress.log("%s", msgerr.c_str());
+
+	}
+
+	face.progress.log(_("Disconnected"));
+	sleep(2);
+	face.progress.hide();
+    }
+
+    return success;
+#else
+    return false;
+#endif
+}
+
 void imcontroller::icqupdatedetails() {
     icqcontact *c = clist.get(contactroot);
 
@@ -164,24 +251,68 @@ void imcontroller::icqupdatedetails() {
     }
 }
 
+void imcontroller::aimupdateprofile() {
+#ifdef BUILD_AIM
+    icqcontact *c = clist.get(contactroot);
+
+    c->clear();
+    ahook.requestinfo(imcontact(conf.getourid(aim).nickname, aim));
+    if(face.updatedetails(0, aim)) ahook.sendupdateuserinfo(*c);
+#endif
+}
+
+void imcontroller::msnupdateprofile() {
+#ifdef BUILD_MSN
+    if(mhook.logged()) {
+	mhook.requestinfo(imcontact(conf.getourid(msn).nickname, msn));
+	string tmp = face.inputstr(_("new MSN friendly nick: "), clist.get(contactroot)->getnick());
+
+	if(face.getlastinputkey() != KEY_ESC && !tmp.empty()) {
+	    icqcontact *c = clist.get(contactroot);
+	    c->setnick(tmp);
+	    mhook.sendupdateuserinfo(*c);
+
+	    char buf[512];
+	    sprintf(buf, _("Your friendly nickname has been changed to %s"), tmp.c_str());
+	    face.status(buf);
+	}
+    } else {
+	face.status(_("You must be logged to the MSN network to update the friendly nick"));
+    }
+#endif
+}
+
+void imcontroller::jabberupdateprofile() {
+#ifdef BUILD_JABBER
+    if(jhook.logged()) {
+	icqcontact *c;
+
+	(c = clist.get(contactroot))->clear();
+	jhook.requestinfo(imcontact(conf.getourid(jabber).nickname, jabber));
+
+	if(face.updatedetails(0, jabber))
+	    jhook.sendupdateuserinfo(*c);
+
+    } else {
+	face.status(_("You must be logged to the Jabber network to update your details"));
+    }
+#endif
+}
+
 void imcontroller::icqsynclist() {
     bool fin;
     int synchronized, attempt = 1;
     string msg;
-    vector<icqcontact *> needauth, tobestored;
+    vector<icqcontact *> tobestored;
 
     face.progress.show(_(" Contact list synchronization "));
     face.progress.log(_("Starting the synchronization process"));
 
     for(fin = false; !fin; ) {
-	ihook.getsyncstatus(synchronized, tobestored, needauth);
+	ihook.getsyncstatus(synchronized, tobestored);
 	face.progress.log(_("Attempt #%d"), attempt);
 	face.progress.log(_(".. %d already synchronized, %d contacts to be stored"), synchronized, tobestored.size());
 
-	if(!needauth.empty()) {
-	    face.progress.log(_(".. %d contacts require authorization"), needauth.size());
-	}
-
 	msg = "";
 	if(tobestored.empty()) msg = _("Finished");
 	else if(!ihook.logged()) msg = _("Disconnected");
@@ -200,140 +331,45 @@ void imcontroller::icqsynclist() {
 	attempt++;
     }
 
-    if(!needauth.empty()) {
-	char buf[512];
-	sprintf(buf, _("%d contacts require authorization to be added. Request it now?"), needauth.size());
-
-	if(face.ask(buf, ASK_YES | ASK_NO, ASK_YES) == ASK_YES) {
-	    ihook.sendaddauth();
-	}
-    }
-
     face.progress.hide();
 }
 
-void imcontroller::yahoosynclist() {
-    vector<icqcontact *> tosync = yhook.getneedsync();
+void imcontroller::synclist(protocolname pname) {
+    vector<icqcontact *> tosync = gethook(pname).getneedsync();
     vector<icqcontact *>::iterator it;
 
+    face.progress.show(_(" Uploading buddies "));
+
     if(tosync.empty()) {
-	face.status(_("To contacts to be synchronized; all of them are already stored server-side"));
+	face.progress.log(_("No contacts to be synchronized"));
+	face.progress.log(_(".. all of them are already stored server-side"));
 
     } else {
 	char buf[512];
 	sprintf(buf, _("%d contacts are do be stored server-side. Add them now?"), tosync.size());
 
 	if(face.ask(buf, ASK_YES | ASK_NO, ASK_YES) == ASK_YES) {
-	    face.progress.show(_(" Uploading Yahoo! buddies "));
 
 	    for(it = tosync.begin(); it != tosync.end(); ++it) {
 		face.progress.log(_(".. adding %s"), (*it)->getdesc().nickname.c_str());
-		yhook.sendnewuser((*it)->getdesc());
+		gethook(pname).sendnewuser((*it)->getdesc());
 	    }
 
 	    face.progress.log(_("Finished"));
-	    sleep(2);
-	    face.progress.hide();
-	}
-    }
-}
-
-void imcontroller::aimupdateprofile() {
-    icqcontact *c = clist.get(contactroot);
-
-    c->clear();
-    ahook.requestinfo(imcontact(conf.getourid(aim).uin, aim));
-
-    if(face.updatedetails(0, aim)) {
-	ahook.sendupdateuserinfo(*c, "");
-    }
-}
-
-void imcontroller::ircchannels() {
-    bool finished = false, success;
-    int n, i, b;
-    dialogbox db;
-    string name, st;
-
-    vector<irchook::channelInfo> channels;
-    vector<irchook::channelInfo>::iterator ic;
-
-    db.setwindow(new textwindow(0, 0, face.sizeDlg.width, face.sizeDlg.height,
-	clr(cp_dialog_frame), TW_CENTERED, clr(cp_dialog_highlight),
-	_(" IRC channels manager ")));
-
-    db.settree(new treeview(clr(cp_dialog_text), clr(cp_dialog_selected),
-	clr(cp_dialog_highlight), clr(cp_dialog_text)));
-
-    db.setbar(new horizontalbar(clr(cp_dialog_text), clr(cp_dialog_selected),
-	_("Add"), _("Remove"), _("Join/leave"), _("Show on the list"), _("Done"), 0));
-
-    channels = irhook.getautochannels();
-
-    db.addkey(KEY_IC, 0);
-    db.addkey(KEY_DC, 1);
-    db.addkey(' ', 2);
-    db.addautokeys();
-
-    treeview &t = *db.gettree();
-
-    while(!finished) {
-	t.clear();
-
-	i = db.gettree()->addnode(_(" Channels "));
-
-	for(ic = channels.begin(); ic != channels.end(); ++ic) {
-	    st = (string) (ic->joined ? _("joined") : _("not joined")) + ", " +
-		(ic->contactlist ? _("shown") : _("hidden"));
-
-	    t.addleaff(i, 0, 0, " %s [%s] ", ic->name.c_str(), st.c_str());
+	} else {
+	    face.progress.log(_("Cancelled"));
 	}
 
-	finished = !db.open(n, b, (void **) &i);
-
-	if(!finished)
-	switch(b) {
-	    case 0:
-		name = face.inputstr(_("channel name to add: "));
-		if(!name.empty()) channels.push_back(name);
-		break;
-
-	    case 1:
-		if(n > 1) {
-		    ic = channels.begin()+n-2;
-		    channels.erase(ic);
-		}
-		break;
-
-	    case 2:
-		if(n > 1) {
-		    ic = channels.begin()+n-2;
-		    ic->joined = !ic->joined;
-		}
-		break;
-
-	    case 3:
-		if(n > 1) {
-		    ic = channels.begin()+n-2;
-		    ic->contactlist = !ic->contactlist;
-		}
-		break;
-
-	    case 4:
-		finished = true;
-		irhook.setautochannels(channels);
-		break;
-	}
     }
 
-    db.close();
+    sleep(2);
+    face.progress.hide();
 }
 
 void imcontroller::registration(icqconf::imaccount &account) {
     switch(account.pname) {
-	case icq:
-	    icqregistration(account);
-	    break;
+	case icq: icqregistration(account); break;
+	case jabber: jabberregistration(account); break;
 	default:
 	    face.status("[" + conf.getprotocolname(account.pname) + "] " +
 		_("registration is not supported"));
@@ -345,19 +381,20 @@ void imcontroller::updateinfo(icqconf::imaccount &account) {
     switch(account.pname) {
 	case icq: icqupdatedetails(); break;
 	case aim: aimupdateprofile(); break;
-    }
-}
-
-void imcontroller::channels(icqconf::imaccount &account) {
-    switch(account.pname) {
-	case irc: ircchannels(); break;
+	case msn: msnupdateprofile(); break;
+	case jabber: jabberupdateprofile(); break;
     }
 }
 
 void imcontroller::synclist(icqconf::imaccount &account) {
     switch(account.pname) {
-	case icq: icqsynclist(); break;
-	case yahoo: yahoosynclist(); break;
+	case icq:
+	    icqsynclist();
+	    break;
+
+	default:
+	    synclist(account.pname);
+	    break;
     }
 }
 
@@ -413,7 +450,18 @@ void imsearchparams::save(const string &prname) const {
 	    << department << "\t"
 	    << position << "\t"
 	    << email << "\t"
-	    << room << endl;
+	    << room << "\t"
+	    << (sincelast ? "1" : "0") << "\t"
+	    << (reverse ? "1" : "0") << "\t"
+	    << service << "\t";
+
+	vector<pair<string, string> >::const_iterator
+	    ifp = flexparams.begin();
+
+	while(ifp != flexparams.end()) {
+	    of << ifp->first << "\t" << ifp->second << "\t";
+	    ++ifp;
+	}
 
 	of.close();
     }
@@ -434,7 +482,7 @@ static string getfield(string &buf) {
 
 bool imsearchparams::load(const string &prname) {
     ifstream f(conf.getconfigfname("search-profiles").c_str());
-    string buf;
+    string buf, flexparam;
     bool r = false;
 
     if(f.is_open()) {
@@ -461,6 +509,14 @@ bool imsearchparams::load(const string &prname) {
 		position = getfield(buf);
 		email = getfield(buf);
 		room = getfield(buf);
+		sincelast = getfield(buf) == "1";
+		reverse = getfield(buf) == "1";
+		service = getfield(buf);
+
+		while(!buf.empty())
+		    if(!(flexparam = getfield(buf)).empty())
+			flexparams.push_back(make_pair(flexparam, getfield(buf)));
+
 		break;
 	    }
 	}
