@@ -7,8 +7,8 @@
 
 struct imsearchparams {
     imsearchparams() {
-	onlineonly = sincelast = false;
-	uin = 0;
+	onlineonly = sincelast = reverse = false;
+	uin = checkfrequency = 0;
 	minage = maxage = country = language = randomgroup = 0;
 	gender = genderUnspec;
 	agerange = ICQ2000::range_NoRange;
@@ -19,13 +19,17 @@ struct imsearchparams {
     };
 
     protocolname pname;
-    bool onlineonly, sincelast;
-    unsigned int uin;
+    bool onlineonly, sincelast, reverse;
+    unsigned int uin, checkfrequency;
     unsigned short minage, maxage, country, language, randomgroup;
     ICQ2000::AgeRange agerange;
     imgender gender;
-    string firstname, lastname, nick, city, state, kwords;
-    string company, department, position, email, room;
+
+    string firstname, lastname, nick, city, state, kwords, url;
+    string company, department, position, email, room, password;
+
+    string service;
+    vector<pair<string, string> > flexparams;
 
     void save(const string &prname) const;
     bool load(const string &prname);
@@ -36,13 +40,18 @@ class imcontroller {
 	unsigned int ruin;
 	string rnick, rfname, rlname, remail, rpasswd, rserver;
 
+	void synclist(protocolname pname);
+
     protected:
-	bool icqregdialog();
+	bool regdialog(protocolname pname);
 
 	bool icqregistration(icqconf::imaccount &account);
+	bool jabberregistration(icqconf::imaccount &account);
+
 	void icqupdatedetails();
 	void aimupdateprofile();
-	void ircchannels();
+	void msnupdateprofile();
+	void jabberupdateprofile();
 
 	void icqsynclist();
 	void yahoosynclist();
@@ -53,7 +62,6 @@ class imcontroller {
 
 	void registration(icqconf::imaccount &account);
 	void updateinfo(icqconf::imaccount &account);
-	void channels(icqconf::imaccount &account);
 	void synclist(icqconf::imaccount &account);
 };
 
