@@ -1,5 +1,30 @@
+/*
+*
+* centericq events serialization classes
+* $Id: imevents.cc,v 1.29 2002/11/22 20:23:43 konst Exp $
+*
+* Copyright (C) 2001,2002 by Konstantin Klyagin <konst@konst.org.ua>
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2 of the License, or (at
+* your option) any later version.
+*
+* This program is distributed in the hope that it will be useful, but
+* WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+* General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License
+* along with this program; if not, write to the Free Software
+* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+* USA
+*
+*/
+
 #include "imevents.h"
-#include "centericq.h"
+
+#include <strstream>
 
 // -- serialization constants -------------------------------------------------
 
@@ -10,16 +35,16 @@ static const string sdirection[imevent::imdirection_size] = {
 };
 
 static const string seventtype[imevent::imeventtype_size] = {
-    "MSG", "URL", "SMS", "AUTH", "", "EMAIL", "NOTE"
+    "MSG", "URL", "SMS", "AUTH", "", "EMAIL", "NOTE", "CONT", "FILE"
 };
 
 // -- basic imevent class -----------------------------------------------------
 
-imevent::imevent(): timestamp(time(0)) {
+imevent::imevent() {
 }
 
 imevent::imevent(const imcontact &acont, imdirection adir, imeventtype atype)
-: contact(acont), direction(adir), timestamp(time(0)), type(atype) {
+: contact(acont), direction(adir), type(atype), timestamp(time(0)) {
 }
 
 imevent::imevent(ifstream &f) {
@@ -136,8 +161,12 @@ imevent *imevent::getevent() const {
 	    return new imemail(*this);
 	case notification:
 	    return new imnotification(*this);
+	case contacts:
+	    return new imcontacts(*this);
+	case file:
+	    return new imfile(*this);
 	default:
-	    return 0;
+	    return new imrawevent(*this);
     }
 }
 
@@ -309,8 +338,8 @@ bool imauthorization::contains(const string &atext) const {
 
 // -- imsms class -------------------------------------------------------------
 
-imsms::imsms(const imcontact &acont, imdirection adirection, const string &atext)
-: imevent(acont, adirection, sms), text(atext) {
+imsms::imsms(const imcontact &acont, imdirection adirection, const string &atext, const string &aphone)
+: imevent(acont, adirection, sms), text(atext), phone(aphone) {
 }
 
 imsms::imsms(const imevent &ev) {
@@ -323,20 +352,34 @@ imsms::imsms(const imevent &ev) {
 
     if(m) {
 	text = m->text;
+	phone = m->phone;
     }
 }
 
 string imsms::gettext() const {
-    return text;
+    return (string) _("* SMS : ")  + text;
 }
 
 void imsms::write(ofstream &f) const {
     imevent::write(f);
+
+    if(!phone.empty()) {
+	if(phone.substr(0, 1) != "+") f << "+";
+	f << phone << endl;
+    }
+
     f << text << endl;
 }
 
 void imsms::read(ifstream &f) {
+    getstring(f, phone);
     text = readblock(f);
+
+    if(!phone.empty() && phone.substr(0, 1) != "+") {
+	if(!text.empty()) phone += "\n" + text;
+	text = phone;
+	phone = "";
+    }
 }
 
 bool imsms::empty() const {
@@ -427,8 +470,188 @@ void imnotification::read(ifstream &f) {
     text = readblock(f);
 }
 
+// -- imcontacts class --------------------------------------------------------
+
+imcontacts::imcontacts(const imcontact &acont, imdirection adirection, const vector< pair<unsigned int, string> > &lst)
+: imevent(acont, adirection, imevent::contacts), contacts(lst) {
+}
+
+imcontacts::imcontacts(const imevent &ev) {
+    type = imevent::contacts;
+    contact = ev.contact;
+    direction = ev.direction;
+    timestamp = ev.timestamp;
+
+    const imcontacts *m = dynamic_cast<const imcontacts *>(&ev);
+
+    if(m) {
+	contacts = m->contacts;
+    }
+}
+
+string imcontacts::gettext() const {
+    string text = _("* Contacts : ");
+    vector< pair<unsigned int, string> >::const_iterator ic;
+
+    for(ic = contacts.begin(); ic != contacts.end(); ++ic) {
+	if(ic->first) {
+	    text += (ic->second.empty() ? _("<no nick>") : ic->second)
+		+ " (" + i2str(ic->first) + ")";
+	} else {
+	    text += ic->second;
+	}
+
+	if(ic+1 != contacts.end())
+	    text += ", ";
+    }
+
+    return text;
+}
+
+const vector< pair<unsigned int, string> > &imcontacts::getcontacts() const {
+    return contacts;
+}
+
+bool imcontacts::empty() const {
+    return contacts.empty();
+}
+
+bool imcontacts::contains(const string &atext) const {
+    return gettext().find(atext) != -1;
+}
+
+void imcontacts::write(ofstream &f) const {
+    vector< pair<unsigned int, string> >::const_iterator ic;
+
+    imevent::write(f);
+
+    for(ic = contacts.begin(); ic != contacts.end(); ++ic) {
+	f
+	    << ic->first << "\t"
+	    << ic->second << "\n";
+    }
+}
+
+void imcontacts::read(ifstream &f) {
+    strstream st;
+    string buf;
+    int pos;
+
+    st << readblock(f);
+
+    while(getline(st, buf)) {
+	while((pos = buf.find_first_of("\r\n")) != -1)
+	    buf.erase(pos, 1);
+
+	if((pos = buf.find("\t")) != -1) {
+	    contacts.push_back(make_pair(atoi(buf.substr(0, pos).c_str()), buf.substr(pos+1)));
+	}
+    }
+}
+
+// -- imfile class ------------------------------------------------------------
+
+imfile::imfile(const imevent &ev) {
+    type = imevent::file;
+    contact = ev.contact;
+    direction = ev.direction;
+    timestamp = ev.timestamp;
+
+    const imfile *m = dynamic_cast<const imfile *>(&ev);
+
+    if(m) {
+	files = m->files;
+    }
+}
+
+imfile::imfile(const imcontact &acont, imdirection adirection, const string &amsg, const vector<record> &afiles)
+: imevent(acont, adirection, imevent::file), msg(amsg), files(afiles) {
+}
+
+string imfile::gettext() const {
+    string r;
+    r = _("* File transfer");
+
+    if(!msg.empty())
+	r += (string) " <" + msg + ">";
+
+    r += (string) " : ";
+
+    for(vector<record>::const_iterator i = files.begin(); i != files.end(); ++i) {
+	r += i->fname;
+
+	if(i->size != -1)       // -1 means the size is unknown
+	    r += (string) " (" + i2str(i->size) + ")";
+
+	if(i+1 != files.end())
+	    r += ", ";
+    }
+
+    return r;
+}
+
+string imfile::getmessage() const {
+    return msg;
+}
+
+const vector<imfile::record> &imfile::getfiles() const {
+    return files;
+}
+
+void imfile::setfiles(const vector<imfile::record> &lst) {
+    files = lst;
+}
+
+bool imfile::empty() const {
+    return files.empty();
+}
+
+bool imfile::contains(const string &atext) const {
+    return gettext().find(atext) != -1;
+}
+
+void imfile::write(ofstream &f) const {
+    imevent::write(f);
+
+    f << "|" << mime(msg) << endl;
+
+    for(vector<record>::const_iterator i = files.begin(); i != files.end(); ++i)
+	f << mime(i->fname) << "\t" << i->size << endl;
+}
+
+void imfile::read(ifstream &f) {
+    strstream st;
+    string buf;
+    int pos, line = 0;
+
+    st << readblock(f);
+
+    while(getline(st, buf)) {
+	while((pos = buf.find_first_of("\r\n")) != -1)
+	    buf.erase(pos, 1);
+
+	if(!line++) {
+	    msg = unmime(buf.substr(1));
+
+	} else if((pos = buf.find("\t")) != -1) {
+	    record r;
+	    r.fname = unmime(buf.substr(0, pos));
+	    r.size = strtoul(buf.substr(pos+1).c_str(), 0, 0);
+	    files.push_back(r);
+
+	}
+    }
+}
+
 // -- imrawevent class --------------------------------------------------------
 
 imrawevent::imrawevent(imeventtype atype, const imcontact &acont, imdirection adirection)
 : imevent(acont, adirection, atype) {
 }
+
+imrawevent::imrawevent(const imevent &ev) {
+    type = ev.type;
+    contact = ev.contact;
+    direction = ev.direction;
+    timestamp = ev.timestamp;
+}
