@@ -13,11 +13,14 @@ class imevent {
     friend class imemailexpress;
     friend class imemail;
     friend class imnotification;
+    friend class imcontacts;
+    friend class imfile;
+    friend class imrawevent;
 
     public:
 	enum imeventtype {
 	    message, url, sms, authorization, online, email, notification,
-	    imeventtype_size
+	    contacts, file, imeventtype_size
 	};
 
 	enum imdirection {
@@ -98,14 +101,16 @@ class imurl: public imevent {
 
 class imsms: public imevent {
     protected:
-	string text;
+	string text, phone;
 
     public:
 	imsms(const imevent &ev);
 	imsms(const imcontact &acont, imdirection adirection,
-	    const string &atext);
+	    const string &atext, const string &aphone = "");
 
 	string gettext() const;
+	string getmessage() const { return text; }
+	string getphone() const { return phone; }
 
 	bool empty() const;
 	bool contains(const string &atext) const;
@@ -178,8 +183,63 @@ class imnotification: public imevent {
 	void read(ifstream &f);
 };
 
+class imcontacts: public imevent {
+    protected:
+	vector< pair<unsigned int, string> > contacts;
+
+    public:
+	imcontacts(const imevent &ev);
+	imcontacts(const imcontact &acont, imdirection adirection,
+	    const vector< pair<unsigned int, string> > &lst);
+
+	string gettext() const;
+	const vector< pair<unsigned int, string> > &getcontacts() const;
+
+	bool empty() const;
+	bool contains(const string &atext) const;
+
+	void write(ofstream &f) const;
+	void read(ifstream &f);
+};
+
+class imfile: public imevent {
+    public:
+	struct record {
+	    string fname;
+	    long size;
+	};
+
+    protected:
+	vector<record> files;
+	string msg;
+
+    public:
+	imfile() { };
+	imfile(const imevent &ev);
+	imfile(const imcontact &acont, imdirection adirection,
+	    const string &amsg = "",
+	    const vector<record> &afiles = vector<record>());
+
+	string gettext() const;
+	string getmessage() const;
+
+	const vector<record> &getfiles() const;
+	void setfiles(const vector<record> &lst);
+
+	bool empty() const;
+	bool contains(const string &atext) const;
+
+	void write(ofstream &f) const;
+	void read(ifstream &f);
+
+	bool operator == (const imfile &fr) const   { return this->gettext() == fr.gettext(); }
+	bool operator != (const imfile &fr) const   { return !(*this == fr); }
+	bool operator < (const imfile &fr) const    { return this->gettext() < fr.gettext(); }
+};
+
 class imrawevent: public imevent {
     public:
+	imrawevent(const imevent &ev);
 	imrawevent(imeventtype atype, const imcontact &acont, imdirection adirection);
 };
 
